@@ -90,6 +90,11 @@ SQInteger sq_getvmstate(HSQUIRRELVM v)
 	}
 }
 
+void sq_decreaseops(HSQUIRRELVM v, int amount)
+{
+	v->DecreaseOps(amount);
+}
+
 bool sq_can_suspend(HSQUIRRELVM v)
 {
 	return v->_nnativecalls <= 2;
@@ -984,7 +989,7 @@ SQRESULT sq_suspendvm(HSQUIRRELVM v)
 	return v->Suspend();
 }
 
-SQRESULT sq_wakeupvm(HSQUIRRELVM v,SQBool wakeupret,SQBool retval,SQBool raiseerror)
+SQRESULT sq_wakeupvm(HSQUIRRELVM v,SQBool wakeupret,SQBool retval,SQBool raiseerror,SQBool throwerror)
 {
 	SQObjectPtr ret;
 	if(!v->_suspended)
@@ -994,8 +999,10 @@ SQRESULT sq_wakeupvm(HSQUIRRELVM v,SQBool wakeupret,SQBool retval,SQBool raiseer
 		v->Pop();
 	} else v->GetAt(v->_stackbase+v->_suspended_target)=_null_;
 	v->_can_suspend = false;
-	if(!v->Execute(_null_,v->_top,-1,-1,ret,raiseerror,SQVM::ET_RESUME_VM)) {
+	if(!v->Execute(_null_,v->_top,-1,-1,ret,raiseerror,throwerror?SQVM::ET_RESUME_THROW_VM : SQVM::ET_RESUME_VM))
 		return SQ_ERROR;
+	if(sq_getvmstate(v) == SQ_VMSTATE_IDLE) {
+		while (v->_top > 1) v->_stack[--v->_top] = _null_;
 	}
 	if(retval)
 		v->Push(ret);
@@ -1010,6 +1017,14 @@ bool sq_resumecatch(HSQUIRRELVM v, int suspend)
 	return v->Execute(_null_, v->_top, -1, -1, ret, SQTrue, SQVM::ET_RESUME_OPENTTD);
 }
 
+bool sq_resumeerror(HSQUIRRELVM v)
+{
+	SQObjectPtr ret;
+	v->_can_suspend = true;
+	v->_ops_till_suspend = 1;
+	return v->Execute(_null_, v->_top, -1, -1, ret, SQTrue, SQVM::ET_RESUME_THROW_VM);
+}
+
 void sq_setreleasehook(HSQUIRRELVM v,SQInteger idx,SQRELEASEHOOK hook)
 {
 	if(sq_gettop(v) >= 1){
