@@ -1,4 +1,4 @@
-/* $Id: ai_rail.hpp 18987 2010-02-02 22:27:03Z terkhen $ */
+/* $Id: ai_rail.hpp 20095 2010-07-08 19:59:13Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -38,6 +38,9 @@ public:
 
 		/** Non-uniform stations is diabled */
 		ERR_NONUNIFORM_STATIONS_DISABLED,  // [STR_ERROR_NONUNIFORM_STATIONS_DISALLOWED]
+
+		/** This railtype cannot have crossings */
+		ERR_RAILTYPE_DISALLOWS_CROSSING,   // [STR_ERROR_CROSSING_DISALLOWED]
 	};
 
 	/**
@@ -440,6 +443,18 @@ public:
 	 * @return The baseprice of building the given object.
 	 */
 	static Money GetBuildCost(RailType railtype, BuildType build_type);
+
+	/**
+	 * Get the maximum speed of trains running on this railtype.
+	 * @param railtype The railtype to get the maximum speed of.
+	 * @pre IsRailTypeAvailable(railtype)
+	 * @return The maximum speed trains can run on this railtype
+	 *   or 0 if there is no limit.
+	 * @note The speed is in OpenTTD's internal speed unit.
+	 *       This is mph / 1.6, which is roughly km/h.
+	 *       To get km/h multiply this number by 1.00584.
+	 */
+	static int32 GetMaxSpeed(RailType railtype);
 };
 
 #endif /* AI_RAIL_HPP */
