@@ -1,4 +1,4 @@
-/* $Id: aircraft_cmd.cpp 19132 2010-02-14 16:31:35Z alberth $ */
+/* $Id: aircraft_cmd.cpp 20428 2010-08-09 21:17:19Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -237,9 +237,10 @@ void GetAircraftSpriteSize(EngineID engine, uint &width, uint &height)
  */
 CommandCost CmdBuildAircraft(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!IsEngineBuildable(p1, VEH_AIRCRAFT, _current_company)) return_cmd_error(STR_ERROR_AIRCRAFT_NOT_AVAILABLE);
+	EngineID eid = GB(p1, 0, 16);
+	if (!IsEngineBuildable(eid, VEH_AIRCRAFT, _current_company)) return_cmd_error(STR_ERROR_AIRCRAFT_NOT_AVAILABLE);
 
-	const Engine *e = Engine::Get(p1);
+	const Engine *e = Engine::Get(eid);
 	const AircraftVehicleInfo *avi = &e->u.air;
 	CommandCost value(EXPENSES_NEW_VEHICLES, e->GetCost());
 
@@ -252,7 +253,7 @@ CommandCost CmdBuildAircraft(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 	if (!IsHangarTile(tile) || !IsTileOwner(tile, _current_company)) return CMD_ERROR;
 
 	/* Prevent building aircraft types at places which can't handle them */
-	if (!CanVehicleUseStation(p1, Station::GetByTile(tile))) return CMD_ERROR;
+	if (!CanVehicleUseStation(eid, Station::GetByTile(tile))) return CMD_ERROR;
 
 	/* We will need to allocate 2 or 3 vehicle structs, depending on type */
 	if (!Vehicle::CanAllocateItem(avi->subtype & AIR_CTOL ? 2 : 3)) {
@@ -299,8 +300,8 @@ CommandCost CmdBuildAircraft(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 
 		v->max_speed = avi->max_speed;
 		v->acceleration = avi->acceleration;
-		v->engine_type = p1;
-		u->engine_type = p1;
+		v->engine_type = eid;
+		u->engine_type = eid;
 
 		v->subtype = (avi->subtype & AIR_CTOL ? AIR_AIRCRAFT : AIR_HELICOPTER);
 		v->UpdateDeltaXY(INVALID_DIR);
@@ -362,7 +363,7 @@ CommandCost CmdBuildAircraft(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 		/* Aircraft with 3 vehicles (chopper)? */
 		if (v->subtype == AIR_HELICOPTER) {
 			Aircraft *w = new Aircraft();
-			w->engine_type = p1;
+			w->engine_type = eid;
 			w->direction = DIR_N;
 			w->owner = _current_company;
 			w->x_pos = v->x_pos;
@@ -387,7 +388,7 @@ CommandCost CmdBuildAircraft(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 		if (IsLocalCompany())
 			InvalidateAutoreplaceWindow(v->engine_type, v->group_id); // updates the replace Aircraft window
 
-		Company::Get(_current_company)->num_engines[p1]++;
+		Company::Get(_current_company)->num_engines[eid]++;
 	}
 
 	return value;
@@ -668,8 +669,8 @@ void UpdateAircraftCache(Aircraft *v)
 {
 	uint max_speed = GetVehicleProperty(v, PROP_AIRCRAFT_SPEED, 0);
 	if (max_speed != 0) {
-		/* Convert from original units to (approx) km/h */
-		max_speed = (max_speed * 129) / 10;
+		/* Convert from original units to km-ish/h */
+		max_speed = (max_speed * 128) / 10;
 
 		v->acache.cached_max_speed = max_speed;
 	} else {
@@ -876,7 +877,11 @@ static bool AircraftController(Aircraft *v)
 		/* Make sure the rotors don't rotate too fast */
 		if (u->cur_speed > 32) {
 			v->cur_speed = 0;
-			if (--u->cur_speed == 32) SndPlayVehicleFx(SND_18_HELICOPTER, v);
+			if (--u->cur_speed == 32) {
+				if (!PlayVehicleSound(v, VSE_START)) {
+					SndPlayVehicleFx(SND_18_HELICOPTER, v);
+				}
+			}
 		} else {
 			u->cur_speed = 32;
 			count = UpdateAircraftSpeed(v);
