@@ -1,4 +1,11 @@
-/* $Id: command_type.h 13708 2008-07-16 10:07:38Z rubidium $ */
+/* $Id: command_type.h 19603 2010-04-11 10:57:21Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file command_type.h Types related to commands. */
 
@@ -41,58 +48,97 @@ public:
 	 * @param ex_t the expense type
 	 * @param cst the initial cost of this command
 	 */
-	CommandCost(ExpensesType ex_t, Money cst) : expense_type(ex_t), cost(cst), message(INVALID_STRING_ID), success(true) {}
+	CommandCost(ExpensesType ex_t, const Money &cst) : expense_type(ex_t), cost(cst), message(INVALID_STRING_ID), success(true) {}
 
-	/**
-	 * Adds the cost of the given command return value to this cost.
-	 * Also takes a possible error message when it is set.
-	 * @param ret the command to add the cost of.
-	 * @return this class.
-	 */
-	CommandCost AddCost(CommandCost ret);
 
 	/**
 	 * Adds the given cost to the cost of the command.
 	 * @param cost the cost to add
-	 * @return this class.
 	 */
-	CommandCost AddCost(Money cost);
+	FORCEINLINE void AddCost(const Money &cost)
+	{
+		this->cost += cost;
+	}
+
+	void AddCost(const CommandCost &cmd_cost);
 
 	/**
 	 * Multiplies the cost of the command by the given factor.
-	 * @param cost factor to multiply the costs with
-	 * @return this class
+	 * @param factor factor to multiply the costs with
 	 */
-	CommandCost MultiplyCost(int factor);
+	FORCEINLINE void MultiplyCost(int factor)
+	{
+		this->cost *= factor;
+	}
 
 	/**
 	 * The costs as made up to this moment
 	 * @return the costs
 	 */
-	Money GetCost() const;
+	FORCEINLINE Money GetCost() const
+	{
+		return this->cost;
+	}
 
 	/**
 	 * The expense type of the cost
 	 * @return the expense type
 	 */
-	ExpensesType GetExpensesType() const;
+	FORCEINLINE ExpensesType GetExpensesType() const
+	{
+		return this->expense_type;
+	}
 
 	/**
 	 * Sets the global error message *if* this class has one.
 	 */
-	void SetGlobalErrorMessage() const;
+	FORCEINLINE void SetGlobalErrorMessage() const
+	{
+		extern StringID _error_message;
+		if (this->message != INVALID_STRING_ID) _error_message = this->message;
+	}
+
+	/**
+	 * Makes this CommandCost behave like an error command.
+	 * @param mesasge the error message.
+	 */
+	void MakeError(StringID message)
+	{
+		assert(message != INVALID_STRING_ID);
+		this->success = false;
+		this->message = message;
+	}
+
+	/**
+	 * Returns the error message of a command
+	 * @return the error message, if succeeded INVALID_STRING_ID
+	 */
+	StringID GetErrorMessage() const
+	{
+		extern StringID _error_message;
+
+		if (this->success) return INVALID_STRING_ID;
+		if (this->message != INVALID_STRING_ID) return this->message;
+		return _error_message;
+	}
 
 	/**
 	 * Did this command succeed?
 	 * @return true if and only if it succeeded
 	 */
-	bool Succeeded() const;
+	FORCEINLINE bool Succeeded() const
+	{
+		return this->success;
+	}
 
 	/**
 	 * Did this command fail?
 	 * @return true if and only if it failed
 	 */
-	bool Failed() const;
+	FORCEINLINE bool Failed() const
+	{
+		return !this->success;
+	}
 };
 
 /**
@@ -112,7 +158,7 @@ enum {
 	CMD_REMOVE_SINGLE_RAIL,           ///< remove a single rail track
 	CMD_LANDSCAPE_CLEAR,              ///< demolish a tile
 	CMD_BUILD_BRIDGE,                 ///< build a bridge
-	CMD_BUILD_RAILROAD_STATION,       ///< build a railroad station
+	CMD_BUILD_RAIL_STATION,           ///< build a rail station
 	CMD_BUILD_TRAIN_DEPOT,            ///< build a train depot
 	CMD_BUILD_SIGNALS,                ///< build a signal
 	CMD_REMOVE_SIGNALS,               ///< remove a signal
@@ -121,19 +167,18 @@ enum {
 	CMD_SELL_LAND_AREA,               ///< sell a bought tile before
 	CMD_BUILD_TUNNEL,                 ///< build a tunnel
 
-	CMD_REMOVE_FROM_RAILROAD_STATION, ///< remove a tile station
+	CMD_REMOVE_FROM_RAIL_STATION,     ///< remove a (rectangle of) tiles from a rail station
 	CMD_CONVERT_RAIL,                 ///< convert a rail type
 
-	CMD_BUILD_TRAIN_WAYPOINT,         ///< build a waypoint
+	CMD_BUILD_RAIL_WAYPOINT,          ///< build a waypoint
 	CMD_RENAME_WAYPOINT,              ///< rename a waypoint
-	CMD_REMOVE_TRAIN_WAYPOINT,        ///< remove a waypoint
+	CMD_REMOVE_FROM_RAIL_WAYPOINT,    ///< remove a (rectangle of) tiles from a rail waypoint
 
 	CMD_BUILD_ROAD_STOP,              ///< build a road stop
 	CMD_REMOVE_ROAD_STOP,             ///< remove a road stop
 	CMD_BUILD_LONG_ROAD,              ///< build a complete road (not a "half" one)
 	CMD_REMOVE_LONG_ROAD,             ///< remove a complete road (not a "half" one)
 	CMD_BUILD_ROAD,                   ///< build a "half" road
-	CMD_REMOVE_ROAD,                  ///< remove a "half" road
 	CMD_BUILD_ROAD_DEPOT,             ///< build a road depot
 
 	CMD_BUILD_AIRPORT,                ///< build an airport
@@ -148,8 +193,6 @@ enum {
 	CMD_BUILD_RAIL_VEHICLE,           ///< build a rail vehicle
 	CMD_MOVE_RAIL_VEHICLE,            ///< move a rail vehicle (in the depot)
 
-	CMD_START_STOP_TRAIN,             ///< start or stop a train
-
 	CMD_SELL_RAIL_WAGON,              ///< sell a rail wagon
 
 	CMD_SEND_TRAIN_TO_DEPOT,          ///< send a train to a depot
@@ -166,22 +209,21 @@ enum {
 	CMD_BUILD_INDUSTRY,               ///< build a new industry
 
 	CMD_BUILD_COMPANY_HQ,             ///< build the company headquarter
-	CMD_SET_PLAYER_FACE,              ///< set the face of the player/company
-	CMD_SET_PLAYER_COLOR,             ///< set the color of the player/company
+	CMD_SET_COMPANY_MANAGER_FACE,     ///< set the manager's face of the company
+	CMD_SET_COMPANY_COLOUR,            ///< set the colour of the company
 
 	CMD_INCREASE_LOAN,                ///< increase the loan from the bank
 	CMD_DECREASE_LOAN,                ///< decrease the loan from the bank
 
 	CMD_WANT_ENGINE_PREVIEW,          ///< confirm the preview of an engine
 
-	CMD_NAME_VEHICLE,                 ///< rename a whole vehicle
+	CMD_RENAME_VEHICLE,               ///< rename a whole vehicle
 	CMD_RENAME_ENGINE,                ///< rename a engine (in the engine list)
-	CMD_CHANGE_COMPANY_NAME,          ///< change the company name
-	CMD_CHANGE_PRESIDENT_NAME,        ///< change the president name
+	CMD_RENAME_COMPANY,               ///< change the company name
+	CMD_RENAME_PRESIDENT,             ///< change the president name
 	CMD_RENAME_STATION,               ///< rename a station
 
 	CMD_SELL_AIRCRAFT,                ///< sell an aircraft
-	CMD_START_STOP_AIRCRAFT,          ///< start/stop an aircraft
 	CMD_BUILD_AIRCRAFT,               ///< build an aircraft
 	CMD_SEND_AIRCRAFT_TO_HANGAR,      ///< send an aircraft to a hanger
 	CMD_REFIT_AIRCRAFT,               ///< refit the cargo space of an aircraft
@@ -190,7 +232,6 @@ enum {
 	CMD_RENAME_SIGN,                  ///< rename a sign
 
 	CMD_BUILD_ROAD_VEH,               ///< build a road vehicle
-	CMD_START_STOP_ROADVEH,           ///< start/stop a road vehicle
 	CMD_SELL_ROAD_VEH,                ///< sell a road vehicle
 	CMD_SEND_ROADVEH_TO_DEPOT,        ///< send a road vehicle to the depot
 	CMD_TURN_ROADVEH,                 ///< turn a road vehicle around
@@ -202,16 +243,10 @@ enum {
 	CMD_SELL_SHARE_IN_COMPANY,        ///< sell a share from a company
 	CMD_BUY_COMPANY,                  ///< buy a company which is bankrupt
 
-	CMD_BUILD_TOWN,                   ///< build a town
-
+	CMD_FOUND_TOWN,                   ///< found a town
 	CMD_RENAME_TOWN,                  ///< rename a town
 	CMD_DO_TOWN_ACTION,               ///< do a action from the town detail window (like advertises or bribe)
 
-	CMD_SET_ROAD_DRIVE_SIDE,          ///< set the side where the road vehicles drive
-
-	CMD_CHANGE_DIFFICULTY_LEVEL,      ///< change the difficult of a game (each setting for it own)
-
-	CMD_START_STOP_SHIP,              ///< start/stop a ship
 	CMD_SELL_SHIP,                    ///< sell a ship
 	CMD_BUILD_SHIP,                   ///< build a new ship
 	CMD_SEND_SHIP_TO_DEPOT,           ///< send a ship to a depot
@@ -224,7 +259,7 @@ enum {
 	CMD_MONEY_CHEAT,                  ///< do the money cheat
 	CMD_BUILD_CANAL,                  ///< build a canal
 
-	CMD_PLAYER_CTRL,                  ///< used in multiplayer to create a new player etc.
+	CMD_COMPANY_CTRL,                 ///< used in multiplayer to create a new companies etc.
 	CMD_LEVEL_LAND,                   ///< level land
 
 	CMD_REFIT_RAIL_VEHICLE,           ///< refit the cargo space of a train
@@ -234,13 +269,16 @@ enum {
 	CMD_BUILD_SIGNAL_TRACK,           ///< add signals along a track (by dragging)
 	CMD_REMOVE_SIGNAL_TRACK,          ///< remove signals along a track (by dragging)
 
-	CMD_GIVE_MONEY,                   ///< give money to an other player
-	CMD_CHANGE_PATCH_SETTING,         ///< change a patch setting
+	CMD_GIVE_MONEY,                   ///< give money to another company
+	CMD_CHANGE_SETTING,               ///< change a setting
+	CMD_CHANGE_COMPANY_SETTING,       ///< change a company etting
 
 	CMD_SET_AUTOREPLACE,              ///< set an autoreplace entry
 
 	CMD_CLONE_VEHICLE,                ///< clone a vehicle
+	CMD_START_STOP_VEHICLE,           ///< start or stop a vehicle
 	CMD_MASS_START_STOP,              ///< start/stop all vehicles (in a depot)
+	CMD_AUTOREPLACE_VEHICLE,          ///< replace/renew a vehicle while it is in a depot
 	CMD_DEPOT_SELL_ALL_VEHICLES,      ///< sell all vehicles which are in a given depot
 	CMD_DEPOT_MASS_AUTOREPLACE,       ///< force the autoreplace to take action in a given depot
 
@@ -256,6 +294,7 @@ enum {
 	CMD_CHANGE_TIMETABLE,             ///< change the timetable for a vehicle
 	CMD_SET_VEHICLE_ON_TIME,          ///< set the vehicle on time feature (timetable)
 	CMD_AUTOFILL_TIMETABLE,           ///< autofill the timetable
+	CMD_SET_TIMETABLE_START,          ///< set the date that a timetable should start
 };
 
 /**
@@ -263,17 +302,20 @@ enum {
  *
  * This enums defines some flags which can be used for the commands.
  */
-enum {
-	DC_EXEC            = 0x001, ///< execute the given command
-	DC_AUTO            = 0x002, ///< don't allow building on structures
-	DC_QUERY_COST      = 0x004, ///< query cost only,  don't build.
-	DC_NO_WATER        = 0x008, ///< don't allow building on water
-	DC_NO_RAIL_OVERLAP = 0x010, ///< don't allow overlap of rails (used in buildrail)
-	DC_AI_BUILDING     = 0x020, ///< special building rules for AI
-	DC_NO_TOWN_RATING  = 0x040, ///< town rating does not disallow you from building
-	DC_BANKRUPT        = 0x080, ///< company bankrupts, skip money check, skip vehicle on tile check in some cases
-	DC_AUTOREPLACE     = 0x100, ///< autoreplace/autorenew is in progress
+enum DoCommandFlag {
+	DC_NONE                  = 0x000, ///< no flag is set
+	DC_EXEC                  = 0x001, ///< execute the given command
+	DC_AUTO                  = 0x002, ///< don't allow building on structures
+	DC_QUERY_COST            = 0x004, ///< query cost only,  don't build.
+	DC_NO_WATER              = 0x008, ///< don't allow building on water
+	DC_NO_RAIL_OVERLAP       = 0x010, ///< don't allow overlap of rails (used in buildrail)
+	DC_NO_TEST_TOWN_RATING   = 0x020, ///< town rating does not disallow you from building
+	DC_BANKRUPT              = 0x040, ///< company bankrupts, skip money check, skip vehicle on tile check in some cases
+	DC_AUTOREPLACE           = 0x080, ///< autoreplace/autorenew is in progress, this shall disable vehicle limits when building, and ignore certain restrictions when undoing things (like vehicle attach callback)
+	DC_ALL_TILES             = 0x100, ///< allow this command also on MP_VOID tiles
+	DC_NO_MODIFY_TOWN_RATING = 0x200, ///< do not change town rating
 };
+DECLARE_ENUM_AS_BIT_SET(DoCommandFlag);
 
 /**
  * Used to combine a StringID with the command.
@@ -292,10 +334,10 @@ enum {
  * This enumeration defines some flags which are binary-or'ed on a command.
  */
 enum {
-	CMD_NO_WATER              = 0x0400, ///< dont build on water
-	CMD_NETWORK_COMMAND       = 0x0800, ///< execute the command without sending it on the network
-	CMD_NO_TEST_IF_IN_NETWORK = 0x1000, ///< When enabled, the command will bypass the no-DC_EXEC round if in network
-	CMD_SHOW_NO_ERROR         = 0x2000, ///< do not show the error message
+	CMD_NETWORK_COMMAND       = 0x0100, ///< execute the command without sending it on the network
+	CMD_NO_TEST_IF_IN_NETWORK = 0x0200, ///< When enabled, the command will bypass the no-DC_EXEC round if in network
+	CMD_FLAGS_MASK            = 0xFF00, ///< mask for all command flags
+	CMD_ID_MASK               = 0x00FF, ///< mask for the command ID
 };
 
 /**
@@ -304,9 +346,13 @@ enum {
  * This enumeration defines flags for the _command_proc_table.
  */
 enum {
-	CMD_SERVER  = 0x1, ///< the command can only be initiated by the server
-	CMD_OFFLINE = 0x2, ///< the command cannot be executed in a multiplayer game; single-player only
-	CMD_AUTO    = 0x4, ///< set the DC_AUTO flag on this command
+	CMD_SERVER    = 0x01, ///< the command can only be initiated by the server
+	CMD_SPECTATOR = 0x02, ///< the command may be initiated by a spectator
+	CMD_OFFLINE   = 0x04, ///< the command cannot be executed in a multiplayer game; single-player only
+	CMD_AUTO      = 0x08, ///< set the DC_AUTO flag on this command
+	CMD_ALL_TILES = 0x10, ///< allow this command also on MP_VOID tiles
+	CMD_NO_TEST   = 0x20, ///< the command's output may differ between test and execute due to town rating changes etc.
+	CMD_NO_WATER  = 0x40, ///< set the DC_NO_WATER flag on this command
 };
 
 /**
@@ -324,9 +370,10 @@ enum {
  * @param flags Flags for the command, from the DC_* enumeration
  * @param p1 Additional data for the command
  * @param p2 Additional data for the command
+ * @param text Additional text
  * @return The CommandCost of the command, which can be succeeded or failed.
  */
-typedef CommandCost CommandProc(TileIndex tile, uint32 flags, uint32 p1, uint32 p2);
+typedef CommandCost CommandProc(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text);
 
 /**
  * Define a command with the flags which belongs to it.
@@ -335,8 +382,9 @@ typedef CommandCost CommandProc(TileIndex tile, uint32 flags, uint32 p1, uint32
  * the #CMD_AUTO, #CMD_OFFLINE and #CMD_SERVER values.
  */
 struct Command {
-	CommandProc *proc;
-	byte flags;
+	CommandProc *proc; ///< The procedure to actually executing
+	const char *name;  ///< A human readable name for the procedure
+	byte flags;        ///< The (command) flags to that apply to this command
 };
 
 /**
@@ -346,12 +394,24 @@ struct Command {
  * are from the #CommandProc callback type. The boolean parameter indicates if the
  * command succeeded or failed.
  *
- * @param success If the command succeeded or not.
+ * @param result The result of the executed command
  * @param tile The tile of the command action
  * @param p1 Additional data of the command
  * @param p1 Additional data of the command
  * @see CommandProc
  */
-typedef void CommandCallback(bool success, TileIndex tile, uint32 p1, uint32 p2);
+typedef void CommandCallback(const CommandCost &result, TileIndex tile, uint32 p1, uint32 p2);
+
+/**
+ * Structure for buffering the build command when selecting a station to join.
+ */
+struct CommandContainer {
+	TileIndex tile;            ///< tile command being executed on
+	uint32 p1;                 ///< parameter p1
+	uint32 p2;                 ///< parameter p2
+	uint32 cmd;                ///< command being executed
+	CommandCallback *callback; ///< any callback function executed upon successful completion of the command
+	char text[80];             ///< possible text sent for name changes etc
+};
 
 #endif /* COMMAND_TYPE_H */
