@@ -1,4 +1,4 @@
-/* $Id: console_cmds.cpp 19741 2010-04-30 21:01:21Z rubidium $ */
+/* $Id: console_cmds.cpp 20520 2010-08-16 23:06:22Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -252,7 +252,18 @@ static const FiosItem *GetFiosItem(const char *file)
 	int i = strtol(file, &endptr, 10);
 	if (file == endptr || *endptr != '\0') i = -1;
 
-	return IsInsideMM(i, 0, _fios_items.Length()) ? _fios_items.Get(i) : NULL;
+	if (IsInsideMM(i, 0, _fios_items.Length())) return _fios_items.Get(i);
+
+	/* As a last effort assume it is an OpenTTD savegame and
+	 * that the ".sav" part was not given. */
+	char long_file[MAX_PATH];
+	seprintf(long_file, lastof(long_file), "%s.sav", file);
+	for (const FiosItem *item = _fios_items.Begin(); item != _fios_items.End(); item++) {
+		if (strcmp(long_file, item->name) == 0) return item;
+		if (strcmp(long_file, item->title) == 0) return item;
+	}
+
+	return NULL;
 }
 
 
@@ -391,12 +402,59 @@ DEF_CONSOLE_CMD(ConClearBuffer)
  **********************************/
 #ifdef ENABLE_NETWORK
 
+static bool ConKickOrBan(const char *argv, bool ban)
+{
+	const char *ip = argv;
+
+	if (strchr(argv, '.') == NULL && strchr(argv, ':') == NULL) { // banning with ID
+		ClientID client_id = (ClientID)atoi(argv);
+
+		if (client_id == CLIENT_ID_SERVER) {
+			IConsolePrintF(CC_ERROR, "ERROR: Silly boy, you can not %s yourself!", ban ? "ban" : "kick");
+			return true;
+		}
+
+		NetworkClientInfo *ci = NetworkFindClientInfoFromClientID(client_id);
+		if (ci == NULL) {
+			IConsoleError("Invalid client");
+			return true;
+		}
+
+		if (!ban) {
+			/* Kick only this client, not all clients with that IP */
+			NetworkServerKickClient(client_id);
+			return true;
+		}
+
+		/* When banning, kick+ban all clients with that IP */
+		ip = GetClientIP(ci);
+	}
+
+	uint n = NetworkServerKickOrBanIP(ip, ban);
+	if (n == 0) {
+		IConsolePrint(CC_DEFAULT, ban ? "Client not online, address added to banlist" : "Client not found");
+	} else {
+		IConsolePrintF(CC_DEFAULT, "%sed %u client(s)", ban ? "Bann" : "Kick", n);
+	}
+
+	return true;
+}
+
+DEF_CONSOLE_CMD(ConKick)
+{
+	if (argc == 0) {
+		IConsoleHelp("Kick a client from a network game. Usage: 'kick <ip | client-id>'");
+		IConsoleHelp("For client-id's, see the command 'clients'");
+		return true;
+	}
+
+	if (argc != 2) return false;
+
+	return ConKickOrBan(argv[1], false);
+}
+
 DEF_CONSOLE_CMD(ConBan)
 {
-	NetworkClientInfo *ci;
-	const char *banip = NULL;
-	ClientID client_id;
-
 	if (argc == 0) {
 		IConsoleHelp("Ban a client from a network game. Usage: 'ban <ip | client-id>'");
 		IConsoleHelp("For client-id's, see the command 'clients'");
@@ -406,39 +464,7 @@ DEF_CONSOLE_CMD(ConBan)
 
 	if (argc != 2) return false;
 
-	if (strchr(argv[1], '.') == NULL && strchr(argv[1], ':') == NULL) { // banning with ID
-		client_id = (ClientID)atoi(argv[1]);
-		ci = NetworkFindClientInfoFromClientID(client_id);
-	} else { // banning IP
-		ci = NetworkFindClientInfoFromIP(argv[1]);
-		if (ci == NULL) {
-			banip = argv[1];
-			client_id = (ClientID)-1;
-		} else {
-			client_id = ci->client_id;
-		}
-	}
-
-	if (client_id == CLIENT_ID_SERVER) {
-		IConsoleError("Silly boy, you can not ban yourself!");
-		return true;
-	}
-
-	if (client_id == INVALID_CLIENT_ID || (ci == NULL && client_id != (ClientID)-1)) {
-		IConsoleError("Invalid client");
-		return true;
-	}
-
-	if (ci != NULL) {
-		IConsolePrint(CC_DEFAULT, "Client banned");
-		banip = GetClientIP(ci);
-	} else {
-		IConsolePrint(CC_DEFAULT, "Client not online, banned IP");
-	}
-
-	NetworkServerBanIP(banip);
-
-	return true;
+	return ConKickOrBan(argv[1], true);
 }
 
 DEF_CONSOLE_CMD(ConUnBan)
@@ -595,46 +621,6 @@ DEF_CONSOLE_CMD(ConClientNickChange)
 	return true;
 }
 
-DEF_CONSOLE_CMD(ConKick)
-{
-	NetworkClientInfo *ci;
-	ClientID client_id;
-
-	if (argc == 0) {
-		IConsoleHelp("Kick a client from a network game. Usage: 'kick <ip | client-id>'");
-		IConsoleHelp("For client-id's, see the command 'clients'");
-		return true;
-	}
-
-	if (argc != 2) return false;
-
-	if (strchr(argv[1], '.') == NULL) {
-		client_id = (ClientID)atoi(argv[1]);
-		ci = NetworkFindClientInfoFromClientID(client_id);
-	} else {
-		ci = NetworkFindClientInfoFromIP(argv[1]);
-		client_id = (ci == NULL) ? INVALID_CLIENT_ID : ci->client_id;
-	}
-
-	if (client_id == CLIENT_ID_SERVER) {
-		IConsoleError("Silly boy, you can not kick yourself!");
-		return true;
-	}
-
-	if (client_id == INVALID_CLIENT_ID) {
-		IConsoleError("Invalid client");
-		return true;
-	}
-
-	if (ci != NULL) {
-		NetworkServerKickClient(client_id);
-	} else {
-		IConsoleError("Client not found");
-	}
-
-	return true;
-}
-
 DEF_CONSOLE_CMD(ConJoinCompany)
 {
 	if (argc < 2) {
@@ -969,7 +955,7 @@ DEF_CONSOLE_CMD(ConNewGame)
 		return true;
 	}
 
-	StartNewGameWithoutGUI((argc == 2) ? (uint)atoi(argv[1]) : GENERATE_NEW_SEED);
+	StartNewGameWithoutGUI((argc == 2) ? strtoul(argv[1], NULL, 10) : (uint)GENERATE_NEW_SEED);
 	return true;
 }
 
