@@ -1,38 +1,46 @@
-/* $Id: engine.cpp 17079 2009-08-05 23:49:30Z rubidium $ */
+/* $Id: engine.cpp 19926 2010-06-04 21:07:08Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file engine.cpp Base for all engine handling. */
 
 #include "stdafx.h"
-#include "debug.h"
 #include "company_func.h"
 #include "command_func.h"
 #include "news_func.h"
-#include "variables.h"
 #include "aircraft.h"
+#include "newgrf.h"
 #include "newgrf_engine.h"
 #include "group.h"
 #include "strings_func.h"
-#include "gfx_func.h"
 #include "core/random_func.hpp"
 #include "window_func.h"
 #include "date_func.h"
 #include "autoreplace_gui.h"
 #include "string_func.h"
-#include "oldpool_func.h"
 #include "ai/ai.hpp"
-#include "vehicle_func.h"
-#include "settings_type.h"
+#include "core/pool_func.hpp"
+#include "engine_gui.h"
+#include "engine_func.h"
+#include "engine_base.h"
+#include "company_base.h"
 
 #include "table/strings.h"
 #include "table/engines.h"
 
-DEFINE_OLD_POOL_GENERIC(Engine, Engine)
+EnginePool _engine_pool("Engine");
+INSTANTIATE_POOL_METHODS(Engine)
 
 EngineOverrideManager _engine_mngr;
 
 /** Year that engine aging stops. Engines will not reduce in reliability
  * and no more engines will be introduced */
-Year _year_engine_aging_stops;
+static Year _year_engine_aging_stops;
 
 /** Number of engines of each vehicle type in original engine data */
 const uint8 _engine_counts[4] = {
@@ -50,6 +58,8 @@ const uint8 _engine_offsets[4] = {
 	lengthof(_orig_rail_vehicle_info) + lengthof(_orig_road_vehicle_info) + lengthof(_orig_ship_vehicle_info),
 };
 
+assert_compile(lengthof(_orig_rail_vehicle_info) + lengthof(_orig_road_vehicle_info) + lengthof(_orig_ship_vehicle_info) + lengthof(_orig_aircraft_vehicle_info) == lengthof(_orig_engine_info));
+
 const uint EngineOverrideManager::NUM_DEFAULT_ENGINES = _engine_counts[VEH_TRAIN] + _engine_counts[VEH_ROAD] + _engine_counts[VEH_SHIP] + _engine_counts[VEH_AIRCRAFT];
 
 Engine::Engine() :
@@ -83,8 +93,8 @@ Engine::Engine(VehicleType type, EngineID base)
 
 		case VEH_TRAIN:
 			this->u.rail = _orig_rail_vehicle_info[base];
-			this->image_index = this->u.rail.image_index;
-			this->info.string_id = STR_8000_KIRBY_PAUL_TANK_STEAM + base;
+			this->original_image_index = this->u.rail.image_index;
+			this->info.string_id = STR_VEHICLE_NAME_TRAIN_ENGINE_RAIL_KIRBY_PAUL_TANK_STEAM + base;
 
 			/* Set the default model life of original wagons to "infinite" */
 			if (this->u.rail.railveh_type == RAILVEH_WAGON) this->info.base_life = 0xFF;
@@ -93,20 +103,20 @@ Engine::Engine(VehicleType type, EngineID base)
 
 		case VEH_ROAD:
 			this->u.road = _orig_road_vehicle_info[base];
-			this->image_index = this->u.road.image_index;
-			this->info.string_id = STR_8074_MPS_REGAL_BUS + base;
+			this->original_image_index = this->u.road.image_index;
+			this->info.string_id = STR_VEHICLE_NAME_ROAD_VEHICLE_MPS_REGAL_BUS + base;
 			break;
 
 		case VEH_SHIP:
 			this->u.ship = _orig_ship_vehicle_info[base];
-			this->image_index = this->u.ship.image_index;
-			this->info.string_id = STR_80CC_MPS_OIL_TANKER + base;
+			this->original_image_index = this->u.ship.image_index;
+			this->info.string_id = STR_VEHICLE_NAME_SHIP_MPS_OIL_TANKER + base;
 			break;
 
 		case VEH_AIRCRAFT:
 			this->u.air = _orig_aircraft_vehicle_info[base];
-			this->image_index = this->u.air.image_index;
-			this->info.string_id = STR_80D7_SAMPSON_U52 + base;
+			this->original_image_index = this->u.air.image_index;
+			this->info.string_id = STR_VEHICLE_NAME_AIRCRAFT_SAMPSON_U52 + base;
 			break;
 	}
 }
@@ -117,36 +127,6 @@ Engine::~Engine()
 	free(this->name);
 }
 
-/**
- * Determines the default cargo type of an engine.
- *
- * Usually a valid cargo is returned, even though the vehicle has zero capacity, and can therefore not carry anything. But the cargotype is still used
- * for livery selection etc..
- *
- * Vehicles with CT_INVALID as default cargo are usally not available, but it can appear as default cargo of articulated parts.
- *
- * @return The default cargo type.
- * @see CanCarryCargo
- */
-CargoID Engine::GetDefaultCargoType() const
-{
-	switch (this->type) {
-		case VEH_TRAIN:
-			return this->u.rail.cargo_type;
-
-		case VEH_ROAD:
-			return this->u.road.cargo_type;
-
-		case VEH_SHIP:
-			return this->u.ship.cargo_type;
-
-		case VEH_AIRCRAFT:
-			return FindFirstRefittableCargo(this->index);
-
-		default: NOT_REACHED();
-	}
-}
-
 /**
  * Determines whether an engine can carry something.
  * A vehicle cannot carry anything if its capacity is zero, or none of the possible cargos is available in the climate.
@@ -184,24 +164,40 @@ bool Engine::CanCarryCargo() const
  * For multiheaded engines this is the capacity of both heads.
  * For articulated engines use GetCapacityOfArticulatedParts
  *
+ * @note Keep this function consistent with GetVehicleCapacity().
+ * @param mail_capacity returns secondary cargo (mail) capacity of aircraft
  * @return The default capacity
  * @see GetDefaultCargoType
  */
-uint Engine::GetDisplayDefaultCapacity() const
+uint Engine::GetDisplayDefaultCapacity(uint16 *mail_capacity) const
 {
+	if (mail_capacity != NULL) *mail_capacity = 0;
 	if (!this->CanCarryCargo()) return 0;
 	switch (type) {
 		case VEH_TRAIN:
-			return GetEngineProperty(this->index, 0x14, this->u.rail.capacity) + (this->u.rail.railveh_type == RAILVEH_MULTIHEAD ? this->u.rail.capacity : 0);
+			return GetEngineProperty(this->index, PROP_TRAIN_CARGO_CAPACITY, this->u.rail.capacity) + (this->u.rail.railveh_type == RAILVEH_MULTIHEAD ? this->u.rail.capacity : 0);
 
 		case VEH_ROAD:
-			return GetEngineProperty(this->index, 0x0F, this->u.road.capacity);
+			return GetEngineProperty(this->index, PROP_ROADVEH_CARGO_CAPACITY, this->u.road.capacity);
 
 		case VEH_SHIP:
-			return GetEngineProperty(this->index, 0x0D, this->u.ship.capacity);
+			return GetEngineProperty(this->index, PROP_SHIP_CARGO_CAPACITY, this->u.ship.capacity);
 
-		case VEH_AIRCRAFT:
-			return AircraftDefaultCargoCapacity(this->GetDefaultCargoType(), &this->u.air);
+		case VEH_AIRCRAFT: {
+			uint capacity = GetEngineProperty(this->index, PROP_AIRCRAFT_PASSENGER_CAPACITY, this->u.air.passenger_capacity);;
+			CargoID cargo = this->GetDefaultCargoType();
+			if (IsCargoInClass(cargo, CC_PASSENGERS)) {
+				if (mail_capacity != NULL) *mail_capacity = GetEngineProperty(this->index, PROP_AIRCRAFT_MAIL_CAPACITY, this->u.air.mail_capacity);
+			} else {
+				capacity += GetEngineProperty(this->index, PROP_AIRCRAFT_MAIL_CAPACITY, this->u.air.mail_capacity);
+			}
+			switch (cargo) {
+				case CT_PASSENGERS:
+				case CT_MAIL:       return capacity;
+				case CT_GOODS:      return capacity / 2;
+				default:            return capacity / 4;
+			}
+		}
 
 		default: NOT_REACHED();
 	}
@@ -209,43 +205,71 @@ uint Engine::GetDisplayDefaultCapacity() const
 
 Money Engine::GetRunningCost() const
 {
+	Price base_price;
+	uint cost_factor;
 	switch (this->type) {
 		case VEH_ROAD:
-			return this->u.road.running_cost * GetPriceByIndex(this->u.road.running_cost_class) >> 8;
+			base_price = this->u.road.running_cost_class;
+			if (base_price == INVALID_PRICE) return 0;
+			cost_factor = GetEngineProperty(this->index, PROP_ROADVEH_RUNNING_COST_FACTOR, this->u.road.running_cost);
+			break;
 
 		case VEH_TRAIN:
-			return GetEngineProperty(this->index, 0x0D, this->u.rail.running_cost) * GetPriceByIndex(this->u.rail.running_cost_class) >> 8;
+			base_price = this->u.rail.running_cost_class;
+			if (base_price == INVALID_PRICE) return 0;
+			cost_factor = GetEngineProperty(this->index, PROP_TRAIN_RUNNING_COST_FACTOR, this->u.rail.running_cost);
+			break;
 
 		case VEH_SHIP:
-			return GetEngineProperty(this->index, 0x0F, this->u.ship.running_cost) * _price.ship_running >> 8;
+			base_price = PR_RUNNING_SHIP;
+			cost_factor = GetEngineProperty(this->index, PROP_SHIP_RUNNING_COST_FACTOR, this->u.ship.running_cost);
+			break;
 
 		case VEH_AIRCRAFT:
-			return GetEngineProperty(this->index, 0x0E, this->u.air.running_cost) * _price.aircraft_running >> 8;
+			base_price = PR_RUNNING_AIRCRAFT;
+			cost_factor = GetEngineProperty(this->index, PROP_AIRCRAFT_RUNNING_COST_FACTOR, this->u.air.running_cost);
+			break;
 
 		default: NOT_REACHED();
 	}
+
+	return GetPrice(base_price, cost_factor, this->grffile, -8);
 }
 
 Money Engine::GetCost() const
 {
+	Price base_price;
+	uint cost_factor;
 	switch (this->type) {
 		case VEH_ROAD:
-			return GetEngineProperty(this->index, 0x11, this->u.road.cost_factor) * (_price.roadveh_base >> 3) >> 5;
+			base_price = PR_BUILD_VEHICLE_ROAD;
+			cost_factor = GetEngineProperty(this->index, PROP_ROADVEH_COST_FACTOR, this->u.road.cost_factor);
+			break;
 
 		case VEH_TRAIN:
 			if (this->u.rail.railveh_type == RAILVEH_WAGON) {
-				return (GetEngineProperty(this->index, 0x17, this->u.rail.cost_factor) * _price.build_railwagon) >> 8;
+				base_price = PR_BUILD_VEHICLE_WAGON;
+				cost_factor = GetEngineProperty(this->index, PROP_TRAIN_COST_FACTOR, this->u.rail.cost_factor);
 			} else {
-				return GetEngineProperty(this->index, 0x17, this->u.rail.cost_factor) * (_price.build_railvehicle >> 3) >> 5;
+				base_price = PR_BUILD_VEHICLE_TRAIN;
+				cost_factor = GetEngineProperty(this->index, PROP_TRAIN_COST_FACTOR, this->u.rail.cost_factor);
 			}
+			break;
+
 		case VEH_SHIP:
-			return GetEngineProperty(this->index, 0x0A, this->u.ship.cost_factor) * (_price.ship_base >> 3) >> 5;
+			base_price = PR_BUILD_VEHICLE_SHIP;
+			cost_factor = GetEngineProperty(this->index, PROP_SHIP_COST_FACTOR, this->u.ship.cost_factor);
+			break;
 
 		case VEH_AIRCRAFT:
-			return GetEngineProperty(this->index, 0x0B, this->u.air.cost_factor) * (_price.aircraft_base >> 3) >> 5;
+			base_price = PR_BUILD_VEHICLE_AIRCRAFT;
+			cost_factor = GetEngineProperty(this->index, PROP_AIRCRAFT_COST_FACTOR, this->u.air.cost_factor);
+			break;
 
 		default: NOT_REACHED();
 	}
+
+	return GetPrice(base_price, cost_factor, this->grffile, -8);
 }
 
 /**
@@ -256,13 +280,13 @@ uint Engine::GetDisplayMaxSpeed() const
 {
 	switch (this->type) {
 		case VEH_TRAIN:
-			return GetEngineProperty(this->index, 0x09, this->u.rail.max_speed);
+			return GetEngineProperty(this->index, PROP_TRAIN_SPEED, this->u.rail.max_speed);
 
 		case VEH_ROAD:
 			return this->u.road.max_speed / 2;
 
 		case VEH_SHIP:
-			return GetEngineProperty(this->index, 0x0B, this->u.ship.max_speed) / 2;
+			return GetEngineProperty(this->index, PROP_SHIP_SPEED, this->u.ship.max_speed) / 2;
 
 		case VEH_AIRCRAFT:
 			return this->u.air.max_speed;
@@ -276,7 +300,7 @@ uint Engine::GetPower() const
 	/* Currently only trains have 'power' */
 	switch (this->type) {
 		case VEH_TRAIN:
-			return GetEngineProperty(this->index, 0x0B, this->u.rail.power);
+			return GetEngineProperty(this->index, PROP_TRAIN_POWER, this->u.rail.power);
 
 		default: NOT_REACHED();
 	}
@@ -292,7 +316,7 @@ uint Engine::GetDisplayWeight() const
 	/* Currently only trains have 'weight' */
 	switch (this->type) {
 		case VEH_TRAIN:
-			return GetEngineProperty(this->index, 0x16, this->u.rail.weight) << (this->u.rail.railveh_type == RAILVEH_MULTIHEAD ? 1 : 0);
+			return GetEngineProperty(this->index, PROP_TRAIN_WEIGHT, this->u.rail.weight) << (this->u.rail.railveh_type == RAILVEH_MULTIHEAD ? 1 : 0);
 
 		default: NOT_REACHED();
 	}
@@ -308,12 +332,22 @@ uint Engine::GetDisplayMaxTractiveEffort() const
 	/* Currently only trains have 'tractive effort' */
 	switch (this->type) {
 		case VEH_TRAIN:
-			return (10 * this->GetDisplayWeight() * GetEngineProperty(this->index, 0x1F, this->u.rail.tractive_effort)) / 256;
+			return (10 * this->GetDisplayWeight() * GetEngineProperty(this->index, PROP_TRAIN_TRACTIVE_EFFORT, this->u.rail.tractive_effort)) / 256;
 
 		default: NOT_REACHED();
 	}
 }
 
+/**
+ * Returns the vehicle's life length in days.
+ * @return the life length
+ */
+Date Engine::GetLifeLengthInDays() const
+{
+	/* Assume leap years; this gives the player a bit more than the given amount of years, but never less. */
+	return (this->info.lifelength + _settings_game.vehicle.extend_vehicle_life) * DAYS_IN_LEAP_YEAR;
+}
+
 /**
  * Initializes the EngineOverrideManager with the default engines.
  */
@@ -356,7 +390,7 @@ EngineID EngineOverrideManager::GetID(VehicleType type, uint16 grf_local_id, uin
  */
 void SetCachedEngineCounts()
 {
-	uint engines = GetEnginePoolSize();
+	size_t engines = Engine::GetPoolSize();
 
 	/* Set up the engine count for all companies */
 	Company *c;
@@ -374,15 +408,15 @@ void SetCachedEngineCounts()
 
 	const Vehicle *v;
 	FOR_ALL_VEHICLES(v) {
-		if (!IsEngineCountable(v)) continue;
+		if (!v->IsEngineCountable()) continue;
 
 		assert(v->engine_type < engines);
 
-		GetCompany(v->owner)->num_engines[v->engine_type]++;
+		Company::Get(v->owner)->num_engines[v->engine_type]++;
 
 		if (v->group_id == DEFAULT_GROUP) continue;
 
-		g = GetGroup(v->group_id);
+		g = Group::Get(v->group_id);
 		assert(v->type == g->vehicle_type);
 		assert(v->owner == g->owner);
 
@@ -392,8 +426,7 @@ void SetCachedEngineCounts()
 
 void SetupEngines()
 {
-	_Engine_pool.CleanPool();
-	_Engine_pool.AddBlockToPool();
+	_engine_pool.CleanPool();
 
 	assert(_engine_mngr.Length() >= _engine_mngr.NUM_DEFAULT_ENGINES);
 	const EngineIDMapping *end = _engine_mngr.End();
@@ -410,7 +443,7 @@ void ShowEnginePreviewWindow(EngineID engine);
 /* Determine if an engine type is a wagon (and not a loco) */
 static bool IsWagon(EngineID index)
 {
-	const Engine *e = GetEngine(index);
+	const Engine *e = Engine::Get(index);
 	return e->type == VEH_TRAIN && e->u.rail.railveh_type == RAILVEH_WAGON;
 }
 
@@ -447,8 +480,8 @@ static void CalcEngineReliability(Engine *e)
 		/* Kick this engine out of the lists */
 		AddRemoveEngineFromAutoreplaceAndBuildWindows(e->type);
 	}
-	InvalidateWindowClasses(WC_BUILD_VEHICLE); // Update to show the new reliability
-	InvalidateWindowClasses(WC_REPLACE_VEHICLE);
+	SetWindowClassesDirty(WC_BUILD_VEHICLE); // Update to show the new reliability
+	SetWindowClassesDirty(WC_REPLACE_VEHICLE);
 }
 
 void SetYearEngineAgingStops()
@@ -506,8 +539,6 @@ void StartupOneEngine(Engine *e, Date aging_date)
 
 	CalcEngineReliability(e);
 
-	e->lifelength = ei->lifelength + _settings_game.vehicle.extend_vehicle_life;
-
 	/* prevent certain engines from ever appearing. */
 	if (!HasBit(ei->climates, _settings_game.game_creation.landscape)) {
 		e->flags |= ENGINE_AVAILABLE;
@@ -535,17 +566,15 @@ void StartupEngines()
 
 static void AcceptEnginePreview(EngineID eid, CompanyID company)
 {
-	Engine *e = GetEngine(eid);
-	Company *c = GetCompany(company);
+	Engine *e = Engine::Get(eid);
+	Company *c = Company::Get(company);
 
 	SetBit(e->company_avail, company);
 	if (e->type == VEH_TRAIN) {
-		const RailVehicleInfo *rvi = RailVehInfo(eid);
-
-		assert(rvi->railtype < RAILTYPE_END);
-		SetBit(c->avail_railtypes, rvi->railtype);
+		assert(e->u.rail.railtype < RAILTYPE_END);
+		SetBit(c->avail_railtypes, e->u.rail.railtype);
 	} else if (e->type == VEH_ROAD) {
-		SetBit(c->avail_roadtypes, HasBit(EngInfo(eid)->misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD);
+		SetBit(c->avail_roadtypes, HasBit(e->info.misc_flags, EF_ROAD_TRAM) ? ROADTYPE_TRAM : ROADTYPE_ROAD);
 	}
 
 	e->preview_company_rank = 0xFF;
@@ -617,22 +646,19 @@ void EnginesDailyLoop()
  * @param flags operation to perfom
  * @param p1 engine-prototype offered
  * @param p2 unused
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdWantEnginePreview(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	Engine *e;
-
-	if (!IsEngineIndex(p1)) return CMD_ERROR;
-	e = GetEngine(p1);
-	if (GetBestCompany(e->preview_company_rank) != _current_company) return CMD_ERROR;
+	Engine *e = Engine::GetIfValid(p1);
+	if (e == NULL || GetBestCompany(e->preview_company_rank) != _current_company) return CMD_ERROR;
 
 	if (flags & DC_EXEC) AcceptEnginePreview(p1, _current_company);
 
 	return CommandCost();
 }
 
-StringID GetEngineCategoryName(EngineID engine);
-
 static void NewVehicleAvailable(Engine *e)
 {
 	Vehicle *v;
@@ -652,7 +678,7 @@ static void NewVehicleAvailable(Engine *e)
 
 			FOR_ALL_VEHICLES(v) {
 				if (v->type == VEH_TRAIN || v->type == VEH_ROAD || v->type == VEH_SHIP ||
-						(v->type == VEH_AIRCRAFT && IsNormalAircraft(v))) {
+						(v->type == VEH_AIRCRAFT && Aircraft::From(v)->IsNormalAircraft())) {
 					if (v->owner == c->index && v->engine_type == index) {
 						/* The user did prove me wrong, so restore old value */
 						c->block_preview = block_preview;
@@ -686,7 +712,7 @@ static void NewVehicleAvailable(Engine *e)
 
 	SetDParam(0, GetEngineCategoryName(index));
 	SetDParam(1, index);
-	AddNewsItem(STR_NEW_VEHICLE_NOW_AVAILABLE_WITH_TYPE, NS_NEW_VEHICLES, index, 0);
+	AddNewsItem(STR_NEWS_NEW_VEHICLE_NOW_AVAILABLE_WITH_TYPE, NS_NEW_VEHICLES, NR_ENGINE, index);
 }
 
 void EnginesMonthlyLoop()
@@ -695,7 +721,7 @@ void EnginesMonthlyLoop()
 		Engine *e;
 		FOR_ALL_ENGINES(e) {
 			/* Age the vehicle */
-			if (e->flags & ENGINE_AVAILABLE && e->age != 0xFFFF) {
+			if ((e->flags & ENGINE_AVAILABLE) && e->age != 0xFFFF) {
 				e->age++;
 				CalcEngineReliability(e);
 			}
@@ -731,20 +757,22 @@ static bool IsUniqueEngineName(const char *name)
  * @param flags operation to perfom
  * @param p1 engine ID to rename
  * @param p2 unused
+ * @param text the new name or an empty string when resetting to the default
+ * @return the cost of this operation or an error
  */
 CommandCost CmdRenameEngine(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!IsEngineIndex(p1)) return CMD_ERROR;
+	Engine *e = Engine::GetIfValid(p1);
+	if (e == NULL) return CMD_ERROR;
 
 	bool reset = StrEmpty(text);
 
 	if (!reset) {
 		if (strlen(text) >= MAX_LENGTH_ENGINE_NAME_BYTES) return CMD_ERROR;
-		if (!IsUniqueEngineName(text)) return_cmd_error(STR_NAME_MUST_BE_UNIQUE);
+		if (!IsUniqueEngineName(text)) return_cmd_error(STR_ERROR_NAME_MUST_BE_UNIQUE);
 	}
 
 	if (flags & DC_EXEC) {
-		Engine *e = GetEngine(p1);
 		free(e->name);
 
 		if (reset) {
@@ -769,10 +797,10 @@ CommandCost CmdRenameEngine(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
  */
 bool IsEngineBuildable(EngineID engine, VehicleType type, CompanyID company)
 {
-	/* check if it's an engine that is in the engine array */
-	if (!IsEngineIndex(engine)) return false;
+	const Engine *e = Engine::GetIfValid(engine);
 
-	const Engine *e = GetEngine(engine);
+	/* check if it's an engine that is in the engine array */
+	if (e == NULL) return false;
 
 	/* check if it's an engine of specified type */
 	if (e->type != type) return false;
@@ -780,10 +808,12 @@ bool IsEngineBuildable(EngineID engine, VehicleType type, CompanyID company)
 	/* check if it's available */
 	if (!HasBit(e->company_avail, company)) return false;
 
+	if (e->info.string_id == STR_NEWGRF_INVALID_ENGINE) return false;
+
 	if (type == VEH_TRAIN) {
 		/* Check if the rail type is available to this company */
-		const Company *c = GetCompany(company);
-		if (!HasBit(c->avail_railtypes, RailVehInfo(engine)->railtype)) return false;
+		const Company *c = Company::Get(company);
+		if (((GetRailTypeInfo(e->u.rail.railtype))->compatible_railtypes & c->avail_railtypes) == 0) return false;
 	}
 
 	return true;
@@ -797,12 +827,10 @@ bool IsEngineBuildable(EngineID engine, VehicleType type, CompanyID company)
  */
 bool IsEngineRefittable(EngineID engine)
 {
+	const Engine *e = Engine::GetIfValid(engine);
+
 	/* check if it's an engine that is in the engine array */
-	if (!IsEngineIndex(engine)) return false;
-
-	const Engine *e = GetEngine(engine);
-
-	if (e->type == VEH_SHIP && !e->u.ship.refittable) return false;
+	if (e == NULL) return false;
 
 	if (!e->CanCarryCargo()) return false;
 
@@ -811,7 +839,7 @@ bool IsEngineRefittable(EngineID engine)
 
 	/* Are there suffixes?
 	 * Note: This does not mean the suffixes are actually available for every consist at any time. */
-	if (HasBit(ei->callbackmask, CBM_VEHICLE_CARGO_SUFFIX)) return true;
+	if (HasBit(ei->callback_mask, CBM_VEHICLE_CARGO_SUFFIX)) return true;
 
 	/* Is there any cargo except the default cargo? */
 	CargoID default_cargo = e->GetDefaultCargoType();
