@@ -1,4 +1,4 @@
-/* $Id: fileio.cpp 19430 2010-03-15 22:52:39Z rubidium $ */
+/* $Id: fileio.cpp 20168 2010-07-17 11:45:42Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -44,7 +44,7 @@ struct Fio {
 	FILE *handles[MAX_FILE_SLOTS];         ///< array of file handles we can have open
 	byte buffer_start[FIO_BUFFER_SIZE];    ///< local buffer when read from file
 	const char *filenames[MAX_FILE_SLOTS]; ///< array of filenames we (should) have open
-	char *shortnames[MAX_FILE_SLOTS];///< array of short names for spriteloader's use
+	char *shortnames[MAX_FILE_SLOTS];      ///< array of short names for spriteloader's use
 #if defined(LIMITED_FDS)
 	uint open_handles;                     ///< current amount of open handles
 	uint usage_count[MAX_FILE_SLOTS];      ///< count how many times this file has been opened
@@ -53,6 +53,9 @@ struct Fio {
 
 static Fio _fio;
 
+/** Whether the working directory should be scanned. */
+static bool _do_scan_working_directory = true;
+
 /* Get current position in file */
 size_t FioGetPos()
 {
@@ -448,16 +451,21 @@ void FioCreateDirectory(const char *name)
  * It does not add the path separator to zero-sized strings.
  * @param buf    string to append the separator to
  * @param buflen the length of the buf
+ * @return true iff the operation succeeded
  */
-void AppendPathSeparator(char *buf, size_t buflen)
+bool AppendPathSeparator(char *buf, size_t buflen)
 {
 	size_t s = strlen(buf);
 
 	/* Length of string + path separator + '\0' */
-	if (s != 0 && buf[s - 1] != PATHSEPCHAR && s + 2 < buflen) {
+	if (s != 0 && buf[s - 1] != PATHSEPCHAR) {
+		if (s + 2 >= buflen) return false;
+
 		buf[s]     = PATHSEPCHAR;
 		buf[s + 1] = '\0';
 	}
+
+	return true;
 }
 
 /**
@@ -534,7 +542,18 @@ static void SimplifyFileName(char *name)
 #endif
 }
 
-bool TarListAddFile(const char *filename)
+/* static */ uint TarScanner::DoScan() {
+	DEBUG(misc, 1, "Scanning for tars");
+	TarScanner fs;
+	uint num = fs.Scan(".tar", DATA_DIR, false);
+	num += fs.Scan(".tar", AI_DIR, false);
+	num += fs.Scan(".tar", AI_LIBRARY_DIR, false);
+	num += fs.Scan(".tar", SCENARIO_DIR, false);
+	DEBUG(misc, 1, "Scan complete, found %d files", num);
+	return num;
+}
+
+bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 {
 	/* The TAR-header, repeated for every file */
 	typedef struct TarHeader {
@@ -820,66 +839,6 @@ bool ExtractTar(const char *tar_filename)
 	return true;
 }
 
-static int ScanPathForTarFiles(const char *path, size_t basepath_length)
-{
-	extern bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb);
-
-	uint num = 0;
-	struct stat sb;
-	struct dirent *dirent;
-	DIR *dir;
-
-	if (path == NULL || (dir = ttd_opendir(path)) == NULL) return 0;
-
-	while ((dirent = readdir(dir)) != NULL) {
-		const char *d_name = FS2OTTD(dirent->d_name);
-		char filename[MAX_PATH];
-
-		if (!FiosIsValidFile(path, dirent, &sb)) continue;
-
-		snprintf(filename, lengthof(filename), "%s%s", path, d_name);
-
-		if (S_ISDIR(sb.st_mode)) {
-			/* Directory */
-			if (strcmp(d_name, ".") == 0 || strcmp(d_name, "..") == 0) continue;
-			AppendPathSeparator(filename, lengthof(filename));
-			num += ScanPathForTarFiles(filename, basepath_length);
-		} else if (S_ISREG(sb.st_mode)) {
-			/* File */
-			char *ext = strrchr(filename, '.');
-
-			/* If no extension or extension isn't .tar, skip the file */
-			if (ext == NULL) continue;
-			if (strcasecmp(ext, ".tar") != 0) continue;
-
-			if (TarListAddFile(filename)) num++;
-		}
-	}
-
-	closedir(dir);
-	return num;
-}
-
-void ScanForTarFiles()
-{
-	Searchpath sp;
-	char path[MAX_PATH];
-	uint num = 0;
-
-	DEBUG(misc, 1, "Scanning for tars");
-	FOR_ALL_SEARCHPATHS(sp) {
-		FioAppendDirectory(path, MAX_PATH, sp, DATA_DIR);
-		num += ScanPathForTarFiles(path, strlen(path));
-		FioAppendDirectory(path, MAX_PATH, sp, AI_DIR);
-		num += ScanPathForTarFiles(path, strlen(path));
-		FioAppendDirectory(path, MAX_PATH, sp, AI_LIBRARY_DIR);
-		num += ScanPathForTarFiles(path, strlen(path));
-		FioAppendDirectory(path, MAX_PATH, sp, SCENARIO_DIR);
-		num += ScanPathForTarFiles(path, strlen(path));
-	}
-	DEBUG(misc, 1, "Scan complete, found %d files", num);
-}
-
 #if defined(WIN32) || defined(WINCE)
 /**
  * Determine the base (personal dir and game data dir) paths
@@ -919,6 +878,33 @@ void ChangeWorkingDirectory(const char *exe)
 #endif /* WITH_COCOA */
 }
 
+/**
+ * Whether we should scan the working directory.
+ * It should not be scanned if it's the root or
+ * the home directory as in both cases a big data
+ * directory can cause huge amounts of unrelated
+ * files scanned. Furthermore there are nearly no
+ * use cases for the home/root directory to have
+ * OpenTTD directories.
+ * @return true if it should be scanned.
+ */
+bool DoScanWorkingDirectory()
+{
+	/* No working directory, so nothing to do. */
+	if (_searchpaths[SP_WORKING_DIR] == NULL) return false;
+
+	/* Working directory is root, so do nothing. */
+	if (strcmp(_searchpaths[SP_WORKING_DIR], PATHSEP) == 0) return false;
+
+	/* No personal/home directory, so the working directory won't be that. */
+	if (_searchpaths[SP_PERSONAL_DIR] == NULL) return true;
+
+	char tmp[MAX_PATH];
+	snprintf(tmp, lengthof(tmp), "%s%s", _searchpaths[SP_WORKING_DIR], PERSONAL_DIR);
+	AppendPathSeparator(tmp, MAX_PATH);
+	return strcmp(tmp, _searchpaths[SP_PERSONAL_DIR]) != 0;
+}
+
 /**
  * Determine the base (personal dir and game data dir) paths
  * @param exe the path to the executable
@@ -964,6 +950,8 @@ void DetermineBasePaths(const char *exe)
 	_searchpaths[SP_WORKING_DIR] = strdup(tmp);
 #endif
 
+	_do_scan_working_directory = DoScanWorkingDirectory();
+
 	/* Change the working directory to that one of the executable */
 	ChangeWorkingDirectory(exe);
 	if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
@@ -1004,7 +992,10 @@ void DeterminePaths(const char *exe)
 	DetermineBasePaths(exe);
 
 	Searchpath sp;
-	FOR_ALL_SEARCHPATHS(sp) DEBUG(misc, 4, "%s added as search path", _searchpaths[sp]);
+	FOR_ALL_SEARCHPATHS(sp) {
+		if (sp == SP_WORKING_DIR && !_do_scan_working_directory) continue;
+		DEBUG(misc, 4, "%s added as search path", _searchpaths[sp]);
+	}
 
 	if (_config_file != NULL) {
 		_personal_dir = strdup(_config_file);
@@ -1079,7 +1070,7 @@ void DeterminePaths(const char *exe)
 	}
 #endif /* ENABLE_NETWORK */
 
-	ScanForTarFiles();
+	TarScanner::DoScan();
 }
 
 /**
@@ -1158,7 +1149,7 @@ static uint ScanPath(FileScanner *fs, const char *extension, const char *path, s
 			/* Directory */
 			if (!recursive) continue;
 			if (strcmp(d_name, ".") == 0 || strcmp(d_name, "..") == 0) continue;
-			AppendPathSeparator(filename, lengthof(filename));
+			if (!AppendPathSeparator(filename, lengthof(filename))) continue;
 			num += ScanPath(fs, extension, filename, basepath_length, recursive);
 		} else if (S_ISREG(sb.st_mode)) {
 			/* File */
@@ -1220,6 +1211,9 @@ uint FileScanner::Scan(const char *extension, Subdirectory sd, bool tars, bool r
 	uint num = 0;
 
 	FOR_ALL_SEARCHPATHS(sp) {
+		/* Don't search in the working directory */
+		if (sp == SP_WORKING_DIR && !_do_scan_working_directory) continue;
+
 		FioAppendDirectory(path, MAX_PATH, sp, sd);
 		num += ScanPath(this, extension, path, strlen(path), recursive);
 	}
@@ -1245,6 +1239,6 @@ uint FileScanner::Scan(const char *extension, const char *directory, bool recurs
 {
 	char path[MAX_PATH];
 	strecpy(path, directory, lastof(path));
-	AppendPathSeparator(path, lengthof(path));
+	if (!AppendPathSeparator(path, lengthof(path))) return 0;
 	return ScanPath(this, extension, path, strlen(path), recursive);
 }
