@@ -1,18 +1,27 @@
-/* $Id: fontcache.cpp 15826 2009-03-23 00:12:52Z rubidium $ */
+/* $Id: fontcache.cpp 19856 2010-05-18 21:38:09Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file fontcache.cpp Cache for characters from fonts. */
 
 #include "stdafx.h"
-#include "spritecache.h"
 #include "fontcache.h"
 #include "blitter/factory.hpp"
-#include "gfx_func.h"
-#include "core/alloc_func.hpp"
 #include "core/math_func.hpp"
 
 #include "table/sprites.h"
 #include "table/control_codes.h"
 
+static const int ASCII_LETTERSTART = 32; ///< First printable ASCII letter.
+
+/** Semi-constant for the height of the different sizes of fonts. */
+int _font_height[FS_END];
+
 #ifdef WITH_FREETYPE
 #include <ft2build.h>
 #include FT_FREETYPE_H
@@ -26,6 +35,7 @@ static FT_Library _library = NULL;
 static FT_Face _face_small = NULL;
 static FT_Face _face_medium = NULL;
 static FT_Face _face_large = NULL;
+static int _ascender[FS_END];
 
 FreeTypeSettings _freetype;
 
@@ -39,7 +49,7 @@ enum {
 #ifdef WIN32
 #include <windows.h>
 #include <shlobj.h> /* SHGetFolderPath */
-#include "win32.h"
+#include "os/windows/win32.h"
 
 /**
  * Get the short DOS 8.3 format for paths.
@@ -110,7 +120,7 @@ static FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 	font_namep = MallocT<TCHAR>(MAX_PATH);
 	MB_TO_WIDE_BUFFER(font_name, font_namep, MAX_PATH * sizeof(TCHAR));
 #else
-	font_namep = (char*)font_name; // only cast because in unicode pointer is not const
+	font_namep = const_cast<char *>(font_name); // only cast because in unicode pointer is not const
 #endif
 
 	for (index = 0;; index++) {
@@ -303,23 +313,44 @@ static int CALLBACK EnumFontCallback(const ENUMLOGFONTEX *logfont, const NEWTEXT
 		if ((fs.fsCsb[0] & info->locale.lsCsbSupported[0]) == 0 && (fs.fsCsb[1] & info->locale.lsCsbSupported[1]) == 0) return 1;
 	}
 
-	const char *english_name = GetEnglishFontName(logfont);
-	const char *font_name = WIDE_TO_MB((const TCHAR*)logfont->elfFullName);
-	DEBUG(freetype, 1, "Fallback font: %s (%s)", font_name, english_name);
+	char font_name[MAX_PATH];
+#if defined(UNICODE)
+	WIDE_TO_MB_BUFFER((const TCHAR*)logfont->elfFullName, font_name, lengthof(font_name));
+#else
+	strecpy(font_name, (const TCHAR*)logfont->elfFullName, lastof(font_name));
+#endif
 
+	/* Add english name after font name */
+	const char *english_name = GetEnglishFontName(logfont);
+	strecpy(font_name + strlen(font_name) + 1, english_name, lastof(font_name));
+
+	/* Check whether we can actually load the font. */
+	bool ft_init = _library != NULL;
+	bool found = false;
+	FT_Face face;
+	/* Init FreeType if needed. */
+	if ((ft_init || FT_Init_FreeType(&_library) == FT_Err_Ok) && GetFontByFaceName(font_name, &face) == FT_Err_Ok) {
+		FT_Done_Face(face);
+		found = true;
+	}
+	if (!ft_init) {
+		/* Uninit FreeType if we did the init. */
+		FT_Done_FreeType(_library);
+		_library = NULL;
+	}
+
+	if (!found) return 1;
+
+	DEBUG(freetype, 1, "Fallback font: %s (%s)", font_name, english_name);
 	strecpy(info->settings->small_font,  font_name, lastof(info->settings->small_font));
 	strecpy(info->settings->medium_font, font_name, lastof(info->settings->medium_font));
 	strecpy(info->settings->large_font,  font_name, lastof(info->settings->large_font));
-
-	/* Add english name after font name */
-	strecpy(info->settings->small_font + strlen(info->settings->small_font) + 1, english_name, lastof(info->settings->small_font));
-	strecpy(info->settings->medium_font + strlen(info->settings->medium_font) + 1, english_name, lastof(info->settings->medium_font));
-	strecpy(info->settings->large_font + strlen(info->settings->large_font) + 1, english_name, lastof(info->settings->large_font));
 	return 0; // stop enumerating
 }
 
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid)
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
 {
+	DEBUG(freetype, 1, "Trying fallback fonts");
 	EFCParam langInfo;
 	if (GetLocaleInfo(MAKELCID(winlangid, SORT_DEFAULT), LOCALE_FONTSIGNATURE, (LPTSTR)&langInfo.locale, sizeof(langInfo.locale) / sizeof(TCHAR)) == 0) {
 		/* Invalid langid or some other mysterious error, can't determine fallback font. */
@@ -340,6 +371,227 @@ bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, i
 	return ret == 0;
 }
 
+#elif defined(__APPLE__)
+
+#include "os/macosx/macos.h"
+#include <ApplicationServices/ApplicationServices.h>
+
+FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
+{
+	FT_Error err = FT_Err_Cannot_Open_Resource;
+
+	/* Get font reference from name. */
+	CFStringRef name = CFStringCreateWithCString(kCFAllocatorDefault, font_name, kCFStringEncodingUTF8);
+	ATSFontRef font = ATSFontFindFromName(name, kATSOptionFlagsDefault);
+	CFRelease(name);
+	if (font == kInvalidFont) return err;
+
+	/* Get a file system reference for the font. */
+	FSRef ref;
+	OSStatus os_err = -1;
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+	if (MacOSVersionIsAtLeast(10, 5, 0)) {
+		os_err = ATSFontGetFileReference(font, &ref);
+	} else
+#endif
+	{
+#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5) && !__LP64__
+		/* This type was introduced with the 10.5 SDK. */
+#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5)
+	#define ATSFSSpec FSSpec
+#endif
+		FSSpec spec;
+		os_err = ATSFontGetFileSpecification(font, (ATSFSSpec *)&spec);
+		if (os_err == noErr) os_err = FSpMakeFSRef(&spec, &ref);
+#endif
+	}
+
+	if (os_err == noErr) {
+		/* Get unix path for file. */
+		UInt8 file_path[PATH_MAX];
+		if (FSRefMakePath(&ref, file_path, sizeof(file_path)) == noErr) {
+			DEBUG(freetype, 3, "Font path for %s: %s", font_name, file_path);
+			err = FT_New_Face(_library, (const char *)file_path, 0, face);
+		}
+	}
+
+	return err;
+}
+
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
+{
+	bool result = false;
+
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+	if (MacOSVersionIsAtLeast(10, 5, 0)) {
+		/* Determine fallback font using CoreText. This uses the language isocode
+		 * to find a suitable font. CoreText is available from 10.5 onwards. */
+		char lang[16];
+		if (strcmp(language_isocode, "zh_TW") == 0) {
+			/* Traditional Chinese */
+			strecpy(lang, "zh-Hant", lastof(lang));
+		} else if (strcmp(language_isocode, "zh_CN") == 0) {
+			/* Simplified Chinese */
+			strecpy(lang, "zh-Hans", lastof(lang));
+		} else if (strncmp(language_isocode, "ur", 2) == 0) {
+			/* The urdu alphabet is variant of persian. As OS X has no default
+			 * font that advertises an urdu language code, search for persian
+			 * support instead. */
+			strecpy(lang, "fa", lastof(lang));
+		} else {
+			/* Just copy the first part of the isocode. */
+			strecpy(lang, language_isocode, lastof(lang));
+			char *sep = strchr(lang, '_');
+			if (sep != NULL) *sep = '\0';
+		}
+
+		CFStringRef lang_code;
+		lang_code = CFStringCreateWithCString(kCFAllocatorDefault, lang, kCFStringEncodingUTF8);
+
+		/* Create a font iterator and iterate over all fonts that
+		 * are available to the application. */
+		ATSFontIterator itr;
+		ATSFontRef font;
+		ATSFontIteratorCreate(kATSFontContextLocal, NULL, NULL, kATSOptionFlagsUnRestrictedScope, &itr);
+		while (!result && ATSFontIteratorNext(itr, &font) == noErr) {
+			/* Get CoreText font handle. */
+			CTFontRef font_ref = CTFontCreateWithPlatformFont(font, 0.0, NULL, NULL);
+			CFArrayRef langs = CTFontCopySupportedLanguages(font_ref);
+			if (langs != NULL) {
+				/* Font has a list of supported languages. */
+				for (CFIndex i = 0; i < CFArrayGetCount(langs); i++) {
+					CFStringRef lang = (CFStringRef)CFArrayGetValueAtIndex(langs, i);
+					if (CFStringCompare(lang, lang_code, kCFCompareAnchored) == kCFCompareEqualTo) {
+						/* Lang code is supported by font, get full font name. */
+						CFStringRef font_name = CTFontCopyFullName(font_ref);
+						char name[128];
+						CFStringGetCString(font_name, name, lengthof(name), kCFStringEncodingUTF8);
+						CFRelease(font_name);
+						/* Skip some inappropriate or ugly looking fonts that have better alternatives. */
+						if (strncmp(name, "Courier", 7) == 0 || strncmp(name, "Apple Symbols", 13) == 0 ||
+							strncmp(name, ".Aqua", 5) == 0 || strncmp(name, "LastResort", 10) == 0 ||
+							strncmp(name, "GB18030 Bitmap", 14) == 0) continue;
+
+						/* Save result. */
+						strecpy(settings->small_font,  name, lastof(settings->small_font));
+						strecpy(settings->medium_font, name, lastof(settings->medium_font));
+						strecpy(settings->large_font,  name, lastof(settings->large_font));
+						DEBUG(freetype, 2, "CT-Font for %s: %s", language_isocode, name);
+						result = true;
+						break;
+					}
+				}
+				CFRelease(langs);
+			}
+			CFRelease(font_ref);
+		}
+		ATSFontIteratorRelease(&itr);
+		CFRelease(lang_code);
+	} else
+#endif
+	{
+#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5) && !__LP64__
+		/* Determine fallback font using ATSUI. This uses a string sample with
+		 * missing characters. This is not failure-proof, but a better way like
+		 * using the isocode as in the CoreText code path is not available.
+		 * ATSUI was deprecated with 10.6 and is only partially available in
+		 * 64-bit mode. */
+
+		/* Extract a UniChar represenation of the sample string. */
+		CFStringRef cf_str = CFStringCreateWithCString(kCFAllocatorDefault, str, kCFStringEncodingUTF8);
+		if (cf_str == NULL) {
+			/* Something went wrong. Corrupt/invalid sample string? */
+			return false;
+		}
+		CFIndex str_len = CFStringGetLength(cf_str);
+		UniChar string[str_len];
+		CFStringGetCharacters(cf_str, CFRangeMake(0, str_len), string);
+
+		/* Create a default text style with the default font. */
+		ATSUStyle style;
+		ATSUCreateStyle(&style);
+
+		/* Create a text layout object from the sample string using the text style. */
+		UniCharCount run_len = kATSUToTextEnd;
+		ATSUTextLayout text_layout;
+		ATSUCreateTextLayoutWithTextPtr(string, kATSUFromTextBeginning, kATSUToTextEnd, str_len, 1, &run_len, &style, &text_layout);
+
+		/* Try to match a font for the sample text. ATSUMatchFontsToText stops after
+		 * it finds the first continous character run not renderable with the currently
+		 * selected font starting at offset. The matching needs to be repeated until
+		 * the end of the string is reached to make sure the fallback font matches for
+		 * all characters in the string and not only the first run. */
+		UniCharArrayOffset offset = kATSUFromTextBeginning;
+		OSStatus os_err;
+		do {
+			ATSUFontID font;
+			UniCharCount run_len;
+			os_err = ATSUMatchFontsToText(text_layout, offset, kATSUToTextEnd, &font, &offset, &run_len);
+			if (os_err == kATSUFontsMatched) {
+				/* Found a better fallback font. Update the text layout
+				 * object with the new font. */
+				ATSUAttributeTag tag = kATSUFontTag;
+				ByteCount size = sizeof(font);
+				ATSUAttributeValuePtr val = &font;
+				ATSUSetAttributes(style, 1, &tag, &size, &val);
+				offset += run_len;
+			}
+			/* Exit if the end of the string is reached or some other error occured. */
+		} while (os_err == kATSUFontsMatched && offset < (UniCharArrayOffset)str_len);
+
+		if (os_err == noErr || os_err == kATSUFontsMatched) {
+			/* ATSUMatchFontsToText exited normally. Extract font
+			 * out of the text layout object. */
+			ATSUFontID font;
+			ByteCount act_len;
+			ATSUGetAttribute(style, kATSUFontTag, sizeof(font), &font, &act_len);
+
+			/* Get unique font name. The result is not a c-string, we have
+			 * to leave space for a \0 and terminate it ourselves. */
+			char name[128];
+			ATSUFindFontName(font, kFontUniqueName, kFontNoPlatformCode, kFontNoScriptCode, kFontNoLanguageCode, 127, name, &act_len, NULL);
+			name[act_len > 127 ? 127 : act_len] = '\0';
+
+			/* Save Result. */
+			strecpy(settings->small_font,  name, lastof(settings->small_font));
+			strecpy(settings->medium_font, name, lastof(settings->medium_font));
+			strecpy(settings->large_font,  name, lastof(settings->large_font));
+			DEBUG(freetype, 2, "ATSUI-Font for %s: %s", language_isocode, name);
+			result = true;
+		}
+
+		ATSUDisposeTextLayout(text_layout);
+		ATSUDisposeStyle(style);
+		CFRelease(cf_str);
+#endif
+	}
+
+	if (result && strncmp(settings->medium_font, "Geeza Pro", 9) == 0) {
+		/* The font 'Geeza Pro' is often found for arabic characters, but
+		 * it has the 'tiny' problem of not having any latin characters.
+		 * 'Arial Unicode MS' on the other hand has arabic and latin glyphs,
+		 * but seems to 'forget' to inform the OS about this fact. Manually
+		 * substitute the latter for the former if it is loadable. */
+		bool ft_init = _library != NULL;
+		FT_Face face;
+		/* Init FreeType if needed. */
+		if ((ft_init || FT_Init_FreeType(&_library) == FT_Err_Ok) && GetFontByFaceName("Arial Unicode MS", &face) == FT_Err_Ok) {
+			FT_Done_Face(face);
+			strecpy(settings->small_font,  "Arial Unicode MS", lastof(settings->small_font));
+			strecpy(settings->medium_font, "Arial Unicode MS", lastof(settings->medium_font));
+			strecpy(settings->large_font,  "Arial Unicode MS", lastof(settings->large_font));
+			DEBUG(freetype, 1, "Replacing font 'Geeza Pro' with 'Arial Unicode MS'");
+		}
+		if (!ft_init) {
+			/* Uninit FreeType if we did the init. */
+			FT_Done_FreeType(_library);
+			_library = NULL;
+		}
+	 }
+
+	return result;
+}
+
 #elif defined(WITH_FONTCONFIG)
 static FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 {
@@ -407,7 +659,7 @@ static FT_Error GetFontByFaceName(const char *font_name, FT_Face *face)
 	return err;
 }
 
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid)
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str)
 {
 	if (!FcInit()) return false;
 
@@ -473,9 +725,25 @@ error_pattern:
 
 #else /* without WITH_FONTCONFIG */
 FT_Error GetFontByFaceName(const char *font_name, FT_Face *face) {return FT_Err_Cannot_Open_Resource;}
-bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid) { return false; }
+bool SetFallbackFont(FreeTypeSettings *settings, const char *language_isocode, int winlangid, const char *str) { return false; }
 #endif /* WITH_FONTCONFIG */
 
+static void SetFontGeometry(FT_Face face, FontSize size, int pixels)
+{
+	FT_Set_Pixel_Sizes(face, 0, pixels);
+
+	if (FT_IS_SCALABLE(face)) {
+		int asc = face->ascender * pixels / face->units_per_EM;
+		int dec = face->descender * pixels / face->units_per_EM;
+
+		_ascender[size] = asc;
+		_font_height[size] = asc - dec;
+	} else {
+		_ascender[size] = pixels;
+		_font_height[size] = pixels;
+	}
+}
+
 /**
  * Loads the freetype font.
  * First type to load the fontname as if it were a path. If that fails,
@@ -529,6 +797,8 @@ static void LoadFreeTypeFont(const char *font_name, FT_Face *face, const char *t
 
 void InitFreeType()
 {
+	ResetFontSizes();
+
 	if (StrEmpty(_freetype.small_font) && StrEmpty(_freetype.medium_font) && StrEmpty(_freetype.large_font)) {
 		DEBUG(freetype, 1, "No font faces specified, using sprite fonts instead");
 		return;
@@ -547,9 +817,15 @@ void InitFreeType()
 	LoadFreeTypeFont(_freetype.large_font,  &_face_large,  "large");
 
 	/* Set each font size */
-	if (_face_small  != NULL) FT_Set_Pixel_Sizes(_face_small,  0, _freetype.small_size);
-	if (_face_medium != NULL) FT_Set_Pixel_Sizes(_face_medium, 0, _freetype.medium_size);
-	if (_face_large  != NULL) FT_Set_Pixel_Sizes(_face_large,  0, _freetype.large_size);
+	if (_face_small != NULL) {
+		SetFontGeometry(_face_small, FS_SMALL, _freetype.small_size);
+	}
+	if (_face_medium != NULL) {
+		SetFontGeometry(_face_medium, FS_NORMAL, _freetype.medium_size);
+	}
+	if (_face_large != NULL) {
+		SetFontGeometry(_face_large, FS_LARGE, _freetype.large_size);
+	}
 }
 
 static void ResetGlyphCache();
@@ -571,6 +847,7 @@ static void UnloadFace(FT_Face *face)
  */
 void UninitFreeType()
 {
+	ResetFontSizes();
 	ResetGlyphCache();
 
 	UnloadFace(&_face_small);
@@ -616,14 +893,13 @@ static GlyphEntry **_glyph_ptr[FS_END];
 /** Clear the complete cache */
 static void ResetGlyphCache()
 {
-	for (int i = 0; i < FS_END; i++) {
+	for (FontSize i = FS_BEGIN; i < FS_END; i++) {
 		if (_glyph_ptr[i] == NULL) continue;
 
 		for (int j = 0; j < 256; j++) {
 			if (_glyph_ptr[i][j] == NULL) continue;
 
 			for (int k = 0; k < 256; k++) {
-				if (_glyph_ptr[i][j][k].sprite == NULL) continue;
 				free(_glyph_ptr[i][j][k].sprite);
 			}
 
@@ -660,7 +936,7 @@ static void SetGlyphPtr(FontSize size, WChar key, const GlyphEntry *glyph)
 	_glyph_ptr[size][GB(key, 8, 8)][GB(key, 0, 8)].width  = glyph->width;
 }
 
-void *AllocateFont(size_t size)
+static void *AllocateFont(size_t size)
 {
 	return MallocT<byte>(size);
 }
@@ -692,7 +968,6 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 	int height;
 	int x;
 	int y;
-	int y_adj;
 
 	assert(IsPrintable(key));
 
@@ -715,7 +990,7 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 	FT_Render_Glyph(face->glyph, aa ? FT_RENDER_MODE_NORMAL : FT_RENDER_MODE_MONO);
 
 	/* Despite requesting a normal glyph, FreeType may have returned a bitmap */
-	aa = (slot->bitmap.palette_mode == FT_PIXEL_MODE_GRAY);
+	aa = (slot->bitmap.pixel_mode == FT_PIXEL_MODE_GRAY);
 
 	/* Add 1 pixel for the shadow on the medium font. Our sprite must be at least 1x1 pixel */
 	width  = max(1, slot->bitmap.width + (size == FS_NORMAL));
@@ -726,9 +1001,7 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 	sprite.width = width;
 	sprite.height = height;
 	sprite.x_offs = slot->bitmap_left;
-	/* XXX 2 should be determined somehow... it's right for the normal face */
-	y_adj = (size == FS_NORMAL) ? 2 : 0;
-	sprite.y_offs = GetCharacterHeight(size) - slot->bitmap_top - y_adj;
+	sprite.y_offs = _ascender[size] - slot->bitmap_top;
 
 	/* Draw shadow for medium size */
 	if (size == FS_NORMAL) {
@@ -752,7 +1025,7 @@ const Sprite *GetGlyph(FontSize size, WChar key)
 	}
 
 	new_glyph.sprite = BlitterFactoryBase::GetCurrentBlitter()->Encode(&sprite, AllocateFont);
-	new_glyph.width  = (slot->advance.x >> 6) + (size != FS_NORMAL);
+	new_glyph.width  = slot->advance.x >> 6;
 
 	SetGlyphPtr(size, key, &new_glyph);
 
@@ -783,6 +1056,14 @@ uint GetGlyphWidth(FontSize size, WChar key)
 
 #endif /* WITH_FREETYPE */
 
+/** Reset the font sizes to the defaults of the sprite based fonts. */
+void ResetFontSizes()
+{
+	_font_height[FS_SMALL]  =  6;
+	_font_height[FS_NORMAL] = 10;
+	_font_height[FS_LARGE]  = 18;
+}
+
 /* Sprite based glyph mapping */
 
 #include "table/unicode.h"
@@ -819,11 +1100,11 @@ void SetUnicodeGlyph(FontSize size, uint32 key, SpriteID sprite)
 
 void InitializeUnicodeGlyphMap()
 {
-	for (FontSize size = FS_NORMAL; size != FS_END; size++) {
+	for (FontSize size = FS_BEGIN; size != FS_END; size++) {
 		/* Clear out existing glyph map if it exists */
 		if (_unicode_glyph_map[size] != NULL) {
 			for (uint i = 0; i < 256; i++) {
-				if (_unicode_glyph_map[size][i] != NULL) free(_unicode_glyph_map[size][i]);
+				free(_unicode_glyph_map[size][i]);
 			}
 			free(_unicode_glyph_map[size]);
 			_unicode_glyph_map[size] = NULL;
