@@ -1,4 +1,11 @@
-/* $Id: genworld.cpp 18300 2009-11-26 16:53:41Z rubidium $ */
+/* $Id: genworld.cpp 19938 2010-06-05 21:13:35Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file genworld.cpp Functions to generate a map. */
 
@@ -7,7 +14,7 @@
 #include "landscape.h"
 #include "company_func.h"
 #include "variables.h"
-#include "thread.h"
+#include "thread/thread.h"
 #include "command_func.h"
 #include "genworld.h"
 #include "gfxinit.h"
@@ -17,7 +24,6 @@
 #include "viewport_func.h"
 #include "gfx_func.h"
 #include "date_func.h"
-#include "core/random_func.hpp"
 #include "engine_func.h"
 #include "newgrf_storage.h"
 #include "water.h"
@@ -25,8 +31,9 @@
 #include "tilehighlight_func.h"
 #include "saveload/saveload.h"
 #include "void_map.h"
-#include "settings_type.h"
 #include "town.h"
+#include "newgrf.h"
+#include "core/random_func.hpp"
 
 #include "table/sprites.h"
 
@@ -41,11 +48,13 @@ void StartupDisasters();
 
 void InitializeGame(uint size_x, uint size_y, bool reset_date, bool reset_settings);
 
-/* Please only use this variable in genworld.h and genworld.c and
+/**
+ * Please only use this variable in genworld.h and genworld.cpp and
  *  nowhere else. For speed improvements we need it to be global, but
  *  in no way the meaning of it is to use it anywhere else besides
- *  in the genworld.h and genworld.c! -- TrueLight */
-gw_info _gw;
+ *  in the genworld.h and genworld.cpp! -- TrueLight
+ */
+GenWorldInfo _gw;
 
 /** Rights for the map generation */
 ThreadMutex *_genworld_mapgen_mutex = ThreadMutex::New();
@@ -54,6 +63,7 @@ ThreadMutex *_genworld_paint_mutex = ThreadMutex::New();
 
 /**
  * Tells if the world generation is done in a thread or not.
+ * @return the 'threaded' status
  */
 bool IsGenerateWorldThreaded()
 {
@@ -84,7 +94,7 @@ static void CleanupGeneration()
 /**
  * The internal, real, generate function.
  */
-static void _GenerateWorld(void *arg)
+static void _GenerateWorld(void *)
 {
 	try {
 		_generating_world = true;
@@ -94,14 +104,14 @@ static void _GenerateWorld(void *arg)
 		if (_settings_game.game_creation.generation_seed == GENERATE_NEW_SEED) _settings_game.game_creation.generation_seed = _settings_newgame.game_creation.generation_seed = InteractiveRandom();
 		_random.SetSeed(_settings_game.game_creation.generation_seed);
 		SetGeneratingWorldProgress(GWP_MAP_INIT, 2);
-		SetObjectToPlace(SPR_CURSOR_ZZZ, PAL_NONE, VHM_NONE, WC_MAIN_WINDOW, 0);
+		SetObjectToPlace(SPR_CURSOR_ZZZ, PAL_NONE, HT_NONE, WC_MAIN_WINDOW, 0);
 
 		IncreaseGeneratingWorldProgress(GWP_MAP_INIT);
 		/* Must start economy early because of the costs. */
 		StartupEconomy();
 
 		/* Don't generate landscape items when in the scenario editor. */
-		if (_gw.mode == GW_EMPTY) {
+		if (_gw.mode == GWM_EMPTY) {
 			SetGeneratingWorldProgress(GWP_UNMOVABLE, 1);
 
 			/* Make sure the tiles at the north border are void tiles if needed. */
@@ -143,7 +153,7 @@ static void _GenerateWorld(void *arg)
 		_generating_world = false;
 
 		/* No need to run the tile loop in the scenario editor. */
-		if (_gw.mode != GW_EMPTY) {
+		if (_gw.mode != GWM_EMPTY) {
 			uint i;
 
 			SetGeneratingWorldProgress(GWP_RUNTILELOOP, 0x500);
@@ -163,10 +173,11 @@ static void _GenerateWorld(void *arg)
 
 		CleanupGeneration();
 
+		ShowNewGRFError();
+
 		if (_network_dedicated) DEBUG(net, 0, "Map generated, starting game");
-		DEBUG(desync, 1, "new_map: %i\n", _settings_game.game_creation.generation_seed);
+		DEBUG(desync, 1, "new_map: %08x", _settings_game.game_creation.generation_seed);
 
-		if (_settings_client.gui.pause_on_newgame && _game_mode == GM_NORMAL) DoCommandP(0, 1, 0, CMD_PAUSE);
 		if (_debug_desync_level > 0) {
 			char name[MAX_PATH];
 			snprintf(name, lengthof(name), "dmp_cmds_%08x_%08x.sav", _settings_game.game_creation.generation_seed, _date);
@@ -181,25 +192,27 @@ static void _GenerateWorld(void *arg)
 
 /**
  * Set here the function, if any, that you want to be called when landscape
- *  generation is done.
+ * generation is done.
+ * @param proc callback procedure
  */
-void GenerateWorldSetCallback(gw_done_proc *proc)
+void GenerateWorldSetCallback(GWDoneProc *proc)
 {
 	_gw.proc = proc;
 }
 
 /**
  * Set here the function, if any, that you want to be called when landscape
- *  generation is aborted.
+ * generation is aborted.
+ * @param proc callback procedure
  */
-void GenerateWorldSetAbortCallback(gw_abort_proc *proc)
+void GenerateWorldSetAbortCallback(GWAbortProc *proc)
 {
 	_gw.abortp = proc;
 }
 
 /**
  * This will wait for the thread to finish up his work. It will not continue
- *  till the work is done.
+ * till the work is done.
  */
 void WaitTillGeneratedWorld()
 {
@@ -226,6 +239,7 @@ void AbortGeneratingWorld()
 
 /**
  * Is the generation being aborted?
+ * @return the 'aborted' status
  */
 bool IsGeneratingWorldAborted()
 {
@@ -252,11 +266,12 @@ void HandleGeneratingWorldAbortion()
 
 /**
  * Generate a world.
- * @param mode The mode of world generation (see GenerateWorldModes).
+ * @param mode The mode of world generation (see GenWorldMode).
  * @param size_x The X-size of the map.
  * @param size_y The Y-size of the map.
+ * @param reset_settings Whether to reset the game configuration (used for restart)
  */
-void GenerateWorld(GenerateWorldMode mode, uint size_x, uint size_y, bool reset_settings)
+void GenerateWorld(GenWorldMode mode, uint size_x, uint size_y, bool reset_settings)
 {
 	if (_gw.active) return;
 	_gw.mode   = mode;
@@ -289,7 +304,7 @@ void GenerateWorld(GenerateWorldMode mode, uint size_x, uint size_y, bool reset_
 
 	/* Create toolbars */
 	SetupColoursAndInitialWindow();
-	SetObjectToPlace(SPR_CURSOR_ZZZ, PAL_NONE, VHM_NONE, WC_MAIN_WINDOW, 0);
+	SetObjectToPlace(SPR_CURSOR_ZZZ, PAL_NONE, HT_NONE, WC_MAIN_WINDOW, 0);
 
 	if (_gw.thread != NULL) {
 		_gw.thread->Join();
@@ -298,7 +313,7 @@ void GenerateWorld(GenerateWorldMode mode, uint size_x, uint size_y, bool reset_
 	}
 
 	if (BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() == 0 ||
-	    !ThreadObject::New(&_GenerateWorld, NULL, &_gw.thread)) {
+			!ThreadObject::New(&_GenerateWorld, NULL, &_gw.thread)) {
 		DEBUG(misc, 1, "Cannot create genworld thread, reverting to single-threaded mode");
 		_gw.threaded = false;
 		_genworld_mapgen_mutex->EndCritical();
