@@ -1,4 +1,11 @@
-/* $Id: gfx_type.h 15428 2009-02-09 02:57:15Z rubidium $ */
+/* $Id: gfx_type.h 19665 2010-04-17 22:27:49Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file gfx_type.h Types related to the graphics and/or input devices. */
 
@@ -6,18 +13,18 @@
 #define GFX_TYPE_H
 
 #include "core/endian_type.hpp"
-#include "core/enum_type.hpp"
 #include "core/geometry_type.hpp"
 #include "zoom_type.h"
 
-typedef uint32 SpriteID;      ///< The number of a sprite, without mapping bits and colourtables
+typedef uint32 SpriteID;  ///< The number of a sprite, without mapping bits and colourtables
+typedef uint32 PaletteID; ///< The number of the palette
+typedef uint32 CursorID;  ///< The number of the cursor (sprite)
 
 /** Combination of a palette sprite and a 'real' sprite */
 struct PalSpriteID {
 	SpriteID sprite;  ///< The 'real' sprite
-	SpriteID pal;     ///< The palette (use \c PAL_NONE) if not needed)
+	PaletteID pal;    ///< The palette (use \c PAL_NONE) if not needed)
 };
-typedef int32 CursorID;
 
 enum WindowKeyCodes {
 	WKC_SHIFT = 0x8000,
@@ -107,8 +114,8 @@ struct CursorVars {
 	Point pos, size, offs, delta; ///< position, size, offset from top-left, and movement
 	Point draw_pos, draw_size;    ///< position and size bounding-box for drawing
 	int short_vehicle_offset;     ///< offset of the X for short vehicles
-	SpriteID sprite; ///< current image of cursor
-	SpriteID pal;
+	CursorID sprite; ///< current image of cursor
+	PaletteID pal;
 
 	int wheel;       ///< mouse wheel movement
 
@@ -129,6 +136,7 @@ struct CursorVars {
 	bool vehchain;   ///< vehicle chain is dragged
 };
 
+/** Data about how and where to blit pixels. */
 struct DrawPixelInfo {
 	void *dst_ptr;
 	int left, top, width, height;
@@ -136,22 +144,26 @@ struct DrawPixelInfo {
 	ZoomLevel zoom;
 };
 
-struct Colour {
+/** Structure to access the alpha, red, green, and blue channels from a 32 bit number. */
+union Colour {
+	uint32 data; ///< Conversion of the channel information to a 32 bit number.
+	struct {
 #if TTD_ENDIAN == TTD_BIG_ENDIAN
-	uint8 a, r, g, b; ///< colour channels in BE order
+		uint8 a, r, g, b; ///< colour channels in BE order
 #else
-	uint8 b, g, r, a; ///< colour channels in LE order
+		uint8 b, g, r, a; ///< colour channels in LE order
 #endif /* TTD_ENDIAN == TTD_BIG_ENDIAN */
-
-	operator uint32 () const { return *(uint32 *)this; }
+	};
 };
 
 /** Available font sizes */
 enum FontSize {
-	FS_NORMAL,
-	FS_SMALL,
-	FS_LARGE,
+	FS_NORMAL, ///< Index of the normal font in the font tables.
+	FS_SMALL,  ///< Index of the small font in the font tables.
+	FS_LARGE,  ///< Index of the large font in the font tables.
 	FS_END,
+
+	FS_BEGIN = FS_NORMAL, ///< First font.
 };
 DECLARE_POSTFIX_INCREMENT(FontSize);
 
@@ -184,6 +196,7 @@ enum Colours {
 	COLOUR_END,
 	INVALID_COLOUR = 0xFF,
 };
+template <> struct EnumPropsT<Colours> : MakeEnumPropsT<Colours, byte, COLOUR_DARK_BLUE, COLOUR_END, INVALID_COLOUR, 4> {};
 
 /** Colour of the strings, see _string_colourmap in table/palettes.h or docs/ottd-colourtext-palette.png */
 enum TextColour {
