@@ -1,4 +1,4 @@
-/* $Id: group_gui.cpp 19439 2010-03-16 20:54:26Z rubidium $ */
+/* $Id: group_gui.cpp 20429 2010-08-09 21:38:18Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -127,7 +127,7 @@ static const NWidgetPart _nested_group_widgets[] = {
 			EndContainer(),
 			NWidget(NWID_HORIZONTAL),
 				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, GRP_WIDGET_AVAILABLE_VEHICLES), SetMinimalSize(106, 12), SetFill(0, 1),
-						SetDataTip(0x0, STR_VEHICLE_LIST_AVAILABLE_ENGINES_TOOLTIP),
+						SetDataTip(STR_BLACK_STRING, STR_VEHICLE_LIST_AVAILABLE_ENGINES_TOOLTIP),
 				NWidget(WWT_DROPDOWN, COLOUR_GREY, GRP_WIDGET_MANAGE_VEHICLES_DROPDOWN), SetMinimalSize(118, 12), SetFill(0, 1),
 						SetDataTip(STR_VEHICLE_LIST_MANAGE_LIST, STR_VEHICLE_LIST_MANAGE_LIST_TOOLTIP),
 				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, GRP_WIDGET_STOP_ALL), SetMinimalSize(12, 12), SetFill(0, 1),
@@ -226,7 +226,6 @@ public:
 
 		this->GetWidget<NWidgetCore>(GRP_WIDGET_CAPTION)->widget_data = STR_VEHICLE_LIST_TRAIN_CAPTION + this->vehicle_type;
 		this->GetWidget<NWidgetCore>(GRP_WIDGET_LIST_VEHICLE)->tool_tip = STR_VEHICLE_LIST_TRAIN_LIST_TOOLTIP + this->vehicle_type;
-		this->GetWidget<NWidgetCore>(GRP_WIDGET_AVAILABLE_VEHICLES)->widget_data = STR_VEHICLE_LIST_AVAILABLE_TRAINS + this->vehicle_type;
 
 		this->GetWidget<NWidgetCore>(GRP_WIDGET_CREATE_GROUP)->widget_data += this->vehicle_type;
 		this->GetWidget<NWidgetCore>(GRP_WIDGET_RENAME_GROUP)->widget_data += this->vehicle_type;
@@ -301,22 +300,28 @@ public:
 
 	virtual void SetStringParameters(int widget) const
 	{
-		if (widget != GRP_WIDGET_CAPTION) return;
+		switch (widget) {
+			case GRP_WIDGET_AVAILABLE_VEHICLES:
+				SetDParam(0, STR_VEHICLE_LIST_AVAILABLE_TRAINS + this->vehicle_type);
+				break;
 
-		/* If selected_group == DEFAULT_GROUP || ALL_GROUP, draw the standard caption
-		 * We list all vehicles or ungrouped vehicles */
-		if (IsDefaultGroupID(this->group_sel) || IsAllGroupID(this->group_sel)) {
-			SetDParam(0, STR_COMPANY_NAME);
-			SetDParam(1, GB(this->window_number, 0, 8));
-			SetDParam(2, this->vehicles.Length());
-			SetDParam(3, this->vehicles.Length());
-		} else {
-			const Group *g = Group::Get(this->group_sel);
+			case GRP_WIDGET_CAPTION:
+				/* If selected_group == DEFAULT_GROUP || ALL_GROUP, draw the standard caption
+				 * We list all vehicles or ungrouped vehicles */
+				if (IsDefaultGroupID(this->group_sel) || IsAllGroupID(this->group_sel)) {
+					SetDParam(0, STR_COMPANY_NAME);
+					SetDParam(1, GB(this->window_number, 0, 8));
+					SetDParam(2, this->vehicles.Length());
+					SetDParam(3, this->vehicles.Length());
+				} else {
+					const Group *g = Group::Get(this->group_sel);
 
-			SetDParam(0, STR_GROUP_NAME);
-			SetDParam(1, g->index);
-			SetDParam(2, g->num_vehicle);
-			SetDParam(3, g->num_vehicle);
+					SetDParam(0, STR_GROUP_NAME);
+					SetDParam(1, g->index);
+					SetDParam(2, g->num_vehicle);
+					SetDParam(3, g->num_vehicle);
+				}
+				break;
 		}
 	}
 
@@ -381,12 +386,12 @@ public:
 	{
 		switch (widget) {
 			case GRP_WIDGET_ALL_VEHICLES:
-				DrawString(r.left + WD_FRAMERECT_LEFT + 8, r.right - WD_FRAMERECT_RIGHT - 8, r.top + WD_MATRIX_TOP,
+				DrawString(r.left + WD_FRAMERECT_LEFT + 8, r.right - WD_FRAMERECT_RIGHT - 8, r.top + WD_FRAMERECT_TOP + 1,
 						STR_GROUP_ALL_TRAINS + this->vehicle_type, IsAllGroupID(this->group_sel) ? TC_WHITE : TC_BLACK);
 				break;
 
 			case GRP_WIDGET_DEFAULT_VEHICLES:
-				DrawString(r.left + WD_FRAMERECT_LEFT + 8, r.right - WD_FRAMERECT_RIGHT - 8, r.top + WD_MATRIX_TOP,
+				DrawString(r.left + WD_FRAMERECT_LEFT + 8, r.right - WD_FRAMERECT_RIGHT - 8, r.top + WD_FRAMERECT_TOP + 1,
 						STR_GROUP_DEFAULT_TRAINS + this->vehicle_type, IsDefaultGroupID(this->group_sel) ? TC_WHITE : TC_BLACK);
 				break;
 
