@@ -1,4 +1,4 @@
-/* $Id: industry_gui.cpp 19855 2010-05-18 21:30:56Z rubidium $ */
+/* $Id: industry_gui.cpp 20430 2010-08-09 21:48:19Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -206,6 +206,7 @@ public:
 		this->callback_timer = DAY_TICKS;
 
 		this->InitNested(&_build_industry_desc, 0);
+		this->SetWidgetDisabledState(DPIW_FUND_WIDGET, !this->enabled[this->selected_index]);
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
@@ -374,15 +375,13 @@ public:
 					uint16 callback_res = GetIndustryCallback(CBID_INDUSTRY_FUND_MORE_TEXT, 0, 0, NULL, this->selected_type, INVALID_TILE);
 					if (callback_res != CALLBACK_FAILED) {  // Did it fail?
 						str = GetGRFStringID(indsp->grf_prop.grffile->grfid, 0xD000 + callback_res);  // No. here's the new string
+						if (str != STR_UNDEFINED) {
+							PrepareTextRefStackUsage(6);
+							DrawStringMultiLine(left, right, y, bottom, str);
+							StopTextRefStackUsage();
+						}
 					}
 				}
-
-				/* Draw the Additional purchase text, provided by newgrf callback, if any.
-				 * Otherwhise, will print Nothing */
-				if (str != STR_NULL && str != STR_UNDEFINED) {
-					SetDParam(0, str);
-					DrawStringMultiLine(left, right, y, bottom, STR_JUST_STRING);
-				}
 			} break;
 		}
 	}
