@@ -1,4 +1,11 @@
-/* $Id: network_gamelist.cpp 16120 2009-04-22 08:48:08Z rubidium $ */
+/* $Id: network_gamelist.cpp 18746 2010-01-06 20:49:24Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /**
  * @file network_gamelist.cpp This file handles the GameList
@@ -9,17 +16,15 @@
 
 #include "../stdafx.h"
 #include "../debug.h"
-#include "../core/alloc_func.hpp"
-#include "../thread.h"
-#include "../string_func.h"
+#include "../thread/thread.h"
 #include "network_internal.h"
 #include "network_udp.h"
 #include "network_gamelist.h"
 
 NetworkGameList *_network_game_list = NULL;
 
-ThreadMutex *_network_game_list_mutex = ThreadMutex::New();
-NetworkGameList *_network_game_delayed_insertion_list = NULL;
+static ThreadMutex *_network_game_list_mutex = ThreadMutex::New();
+static NetworkGameList *_network_game_delayed_insertion_list = NULL;
 
 /** Add a new item to the linked gamelist, but do it delayed in the next tick
  * or so to prevent race conditions.
@@ -41,17 +46,18 @@ static void NetworkGameListHandleDelayedInsert()
 		NetworkGameList *ins_item = _network_game_delayed_insertion_list;
 		_network_game_delayed_insertion_list = ins_item->next;
 
-		NetworkGameList *item = NetworkGameListAddItem(ins_item->ip, ins_item->port);
+		NetworkGameList *item = NetworkGameListAddItem(ins_item->address);
 
 		if (item != NULL) {
-			ClearGRFConfigList(&item->info.grfconfig);
 			if (StrEmpty(item->info.server_name)) {
+				ClearGRFConfigList(&item->info.grfconfig);
 				memset(&item->info, 0, sizeof(item->info));
 				strecpy(item->info.server_name, ins_item->info.server_name, lastof(item->info.server_name));
 				strecpy(item->info.hostname, ins_item->info.hostname, lastof(item->info.hostname));
 				item->online = false;
 			}
-			item->manually = ins_item->manually;
+			item->manually |= ins_item->manually;
+			if (item->manually) NetworkRebuildHostList();
 			UpdateNetworkGameWindow(false);
 		}
 		free(ins_item);
@@ -61,25 +67,31 @@ static void NetworkGameListHandleDelayedInsert()
 
 /** Add a new item to the linked gamelist. If the IP and Port match
  * return the existing item instead of adding it again
- * @param ip the IP-address (inet_addr) of the to-be added item
+ * @param address the address of the to-be added item
  * @param port the port the server is running on
  * @return a point to the newly added or already existing item */
-NetworkGameList *NetworkGameListAddItem(uint32 ip, uint16 port)
+NetworkGameList *NetworkGameListAddItem(NetworkAddress address)
 {
-	if (ip == 0) return NULL;
+	const char *hostname = address.GetHostname();
+
+	/* Do not query the 'any' address. */
+	if (StrEmpty(hostname) ||
+			strcmp(hostname, "0.0.0.0") == 0 ||
+			strcmp(hostname, "::") == 0) {
+		return NULL;
+	}
 
 	NetworkGameList *item, *prev_item;
 
 	prev_item = NULL;
 	for (item = _network_game_list; item != NULL; item = item->next) {
-		if (item->ip == ip && item->port == port) return item;
+		if (item->address == address) return item;
 		prev_item = item;
 	}
 
 	item = CallocT<NetworkGameList>(1);
 	item->next = NULL;
-	item->ip = ip;
-	item->port = port;
+	item->address = address;
 
 	if (prev_item == NULL) {
 		_network_game_list = item;
@@ -97,10 +109,8 @@ NetworkGameList *NetworkGameListAddItem(uint32 ip, uint16 port)
  * @param remove pointer to the item to be removed */
 void NetworkGameListRemoveItem(NetworkGameList *remove)
 {
-	NetworkGameList *item, *prev_item;
-
-	prev_item = NULL;
-	for (item = _network_game_list; item != NULL; item = item->next) {
+	NetworkGameList *prev_item = NULL;
+	for (NetworkGameList *item = _network_game_list; item != NULL; item = item->next) {
 		if (remove == item) {
 			if (prev_item == NULL) {
 				_network_game_list = remove->next;
@@ -114,6 +124,7 @@ void NetworkGameListRemoveItem(NetworkGameList *remove)
 			remove = NULL;
 
 			DEBUG(net, 4, "[gamelist] removed server from list");
+			NetworkRebuildHostList();
 			UpdateNetworkGameWindow(false);
 			return;
 		}
@@ -122,7 +133,7 @@ void NetworkGameListRemoveItem(NetworkGameList *remove)
 }
 
 enum {
-	MAX_GAME_LIST_REQUERY_COUNT  =  5, ///< How often do we requery in number of times per server?
+	MAX_GAME_LIST_REQUERY_COUNT  = 10, ///< How often do we requery in number of times per server?
 	REQUERY_EVERY_X_GAMELOOPS    = 60, ///< How often do we requery in time?
 	REFRESH_GAMEINFO_X_REQUERIES = 50, ///< Refresh the game info itself after REFRESH_GAMEINFO_X_REQUERIES * REQUERY_EVERY_X_GAMELOOPS game loops
 };
@@ -143,9 +154,42 @@ void NetworkGameListRequery()
 
 		/* item gets mostly zeroed by NetworkUDPQueryServer */
 		uint8 retries = item->retries;
-		NetworkUDPQueryServer(NetworkAddress(item->ip, item->port));
+		NetworkUDPQueryServer(NetworkAddress(item->address));
 		item->retries = (retries >= REFRESH_GAMEINFO_X_REQUERIES) ? 0 : retries;
 	}
 }
 
+/**
+ * Rebuild the GRFConfig's of the servers in the game list as we did
+ * a rescan and might have found new NewGRFs.
+ */
+void NetworkAfterNewGRFScan()
+{
+	for (NetworkGameList *item = _network_game_list; item != NULL; item = item->next) {
+		/* Reset compatability state */
+		item->info.compatible = item->info.version_compatible;
+
+		for (GRFConfig *c = item->info.grfconfig; c != NULL; c = c->next) {
+			assert(HasBit(c->flags, GCF_COPY));
+
+			const GRFConfig *f = FindGRFConfig(c->grfid, c->md5sum);
+			if (f == NULL) {
+				/* Don't know the GRF, so mark game incompatible and the (possibly)
+				 * already resolved name for this GRF (another server has sent the
+				 * name of the GRF already */
+				c->name   = FindUnknownGRFName(c->grfid, c->md5sum, true);
+				c->status = GCS_NOT_FOUND;
+
+				/* If we miss a file, we're obviously incompatible */
+				item->info.compatible = false;
+			} else {
+				c->filename  = f->filename;
+				c->name      = f->name;
+				c->info      = f->info;
+				c->status    = GCS_UNKNOWN;
+			}
+		}
+	}
+}
+
 #endif /* ENABLE_NETWORK */
