@@ -1,4 +1,11 @@
-/* $Id: network_udp.cpp 16060 2009-04-14 21:13:07Z rubidium $ */
+/* $Id: network_udp.cpp 19094 2010-02-11 14:58:51Z yexo $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /**
  * @file network_udp.cpp This file handles the UDP related communication.
@@ -12,21 +19,22 @@
 #include "../stdafx.h"
 #include "../date_func.h"
 #include "../map_func.h"
+#include "../debug.h"
 #include "network_gamelist.h"
 #include "network_internal.h"
 #include "network_udp.h"
-#include "core/host.h"
+#include "network.h"
 #include "../core/endian_func.hpp"
-#include "../core/alloc_func.hpp"
-#include "../string_func.h"
 #include "../company_base.h"
-#include "../settings_type.h"
-#include "../thread.h"
+#include "../thread/thread.h"
 #include "../rev.h"
 
 #include "core/udp.h"
 
-ThreadMutex *_network_udp_mutex = ThreadMutex::New();
+static ThreadMutex *_network_udp_mutex = ThreadMutex::New();
+
+/** Session key to register ourselves to the master server */
+static uint64 _session_key = 0;
 
 enum {
 	ADVERTISE_NORMAL_INTERVAL = 30000, // interval between advertising in ticks (15 minutes)
@@ -43,19 +51,27 @@ NetworkUDPSocketHandler *_udp_master_socket = NULL; ///< udp master socket
 class MasterNetworkUDPSocketHandler : public NetworkUDPSocketHandler {
 protected:
 	DECLARE_UDP_RECEIVE_COMMAND(PACKET_UDP_MASTER_ACK_REGISTER);
+	DECLARE_UDP_RECEIVE_COMMAND(PACKET_UDP_MASTER_SESSION_KEY);
 public:
+	MasterNetworkUDPSocketHandler(NetworkAddressList *addresses) : NetworkUDPSocketHandler(addresses) {}
 	virtual ~MasterNetworkUDPSocketHandler() {}
 };
 
 DEF_UDP_RECEIVE_COMMAND(Master, PACKET_UDP_MASTER_ACK_REGISTER)
 {
 	_network_advertise_retries = 0;
-	DEBUG(net, 2, "[udp] advertising on master server successful");
+	DEBUG(net, 2, "[udp] advertising on master server successful (%s)", NetworkAddress::AddressFamilyAsString(client_addr->GetAddress()->ss_family));
 
 	/* We are advertised, but we don't want to! */
 	if (!_settings_client.network.server_advertise) NetworkUDPRemoveAdvertise(false);
 }
 
+DEF_UDP_RECEIVE_COMMAND(Master, PACKET_UDP_MASTER_SESSION_KEY)
+{
+	_session_key = p->Recv_uint64();
+	DEBUG(net, 2, "[udp] received new session key from master server (%s)", NetworkAddress::AddressFamilyAsString(client_addr->GetAddress()->ss_family));
+}
+
 ///*** Communication with clients (we are server) ***/
 
 class ServerNetworkUDPSocketHandler : public NetworkUDPSocketHandler {
@@ -64,6 +80,7 @@ protected:
 	DECLARE_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_DETAIL_INFO);
 	DECLARE_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_GET_NEWGRFS);
 public:
+	ServerNetworkUDPSocketHandler(NetworkAddressList *addresses) : NetworkUDPSocketHandler(addresses) {}
 	virtual ~ServerNetworkUDPSocketHandler() {}
 };
 
@@ -78,12 +95,12 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_FIND_SERVER)
 
 	/* Update some game_info */
 	ngi.clients_on     = _network_game_info.clients_on;
-	ngi.start_date     = _network_game_info.start_date;
+	ngi.start_date     = ConvertYMDToDate(_settings_game.game_creation.starting_year, 0, 1);
 
 	ngi.server_lang    = _settings_client.network.server_lang;
 	ngi.use_password   = !StrEmpty(_settings_client.network.server_password);
 	ngi.clients_max    = _settings_client.network.max_clients;
-	ngi.companies_on   = ActiveCompanyCount();
+	ngi.companies_on   = (byte)Company::GetNumItems();
 	ngi.companies_max  = _settings_client.network.max_companies;
 	ngi.spectators_on  = NetworkSpectatorCount();
 	ngi.spectators_max = _settings_client.network.max_spectators;
@@ -104,7 +121,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_FIND_SERVER)
 	/* Let the client know that we are here */
 	this->SendPacket(&packet, client_addr);
 
-	DEBUG(net, 2, "[udp] queried from '%s'", inet_ntoa(client_addr->sin_addr));
+	DEBUG(net, 2, "[udp] queried from %s", client_addr->GetHostname());
 }
 
 DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_DETAIL_INFO)
@@ -116,7 +133,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_DETAIL_INFO)
 
 	/* Send the amount of active companies */
 	packet.Send_uint8 (NETWORK_COMPANY_INFO_VERSION);
-	packet.Send_uint8 (ActiveCompanyCount());
+	packet.Send_uint8 ((uint8)Company::GetNumItems());
 
 	/* Fetch the latest version of the stats */
 	NetworkCompanyStats company_stats[MAX_COMPANIES];
@@ -154,7 +171,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 	uint8 in_reply_count = 0;
 	size_t packet_len = 0;
 
-	DEBUG(net, 6, "[udp] newgrf data request from %s:%d", inet_ntoa(client_addr->sin_addr), ntohs(client_addr->sin_port));
+	DEBUG(net, 6, "[udp] newgrf data request from %s", client_addr->GetAddressAsString());
 
 	num_grfs = p->Recv_uint8 ();
 	if (num_grfs > NETWORK_MAX_GRF_COUNT) return;
@@ -173,7 +190,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 		 * the current list and do not send the other data.
 		 * The name could be an empty string, if so take the filename. */
 		packet_len += sizeof(c.grfid) + sizeof(c.md5sum) +
-				min(strlen((f->name != NULL && !StrEmpty(f->name)) ? f->name : f->filename) + 1, (size_t)NETWORK_GRF_NAME_LENGTH);
+				min(strlen((!StrEmpty(f->name)) ? f->name : f->filename) + 1, (size_t)NETWORK_GRF_NAME_LENGTH);
 		if (packet_len > SEND_MTU - 4) { // 4 is 3 byte header + grf count in reply
 			break;
 		}
@@ -189,8 +206,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 		char name[NETWORK_GRF_NAME_LENGTH];
 
 		/* The name could be an empty string, if so take the filename */
-		strecpy(name, (in_reply[i]->name != NULL && !StrEmpty(in_reply[i]->name)) ?
-				in_reply[i]->name : in_reply[i]->filename, lastof(name));
+		strecpy(name, (!StrEmpty(in_reply[i]->name)) ? in_reply[i]->name : in_reply[i]->filename, lastof(name));
 		this->Send_GRFIdentifier(&packet, in_reply[i]);
 		packet.Send_string(name);
 	}
@@ -217,11 +233,12 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 	/* Just a fail-safe.. should never happen */
 	if (_network_udp_server) return;
 
-	DEBUG(net, 4, "[udp] server response from %s:%d", inet_ntoa(client_addr->sin_addr), ntohs(client_addr->sin_port));
+	DEBUG(net, 4, "[udp] server response from %s", client_addr->GetAddressAsString());
 
 	/* Find next item */
-	item = NetworkGameListAddItem(inet_addr(inet_ntoa(client_addr->sin_addr)), ntohs(client_addr->sin_port));
+	item = NetworkGameListAddItem(*client_addr);
 
+	ClearGRFConfigList(&item->info.grfconfig);
 	this->Recv_NetworkGameInfo(p, &item->info);
 
 	item->info.compatible = true;
@@ -236,7 +253,6 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 		const GRFConfig *in_request[NETWORK_MAX_GRF_COUNT];
 		const GRFConfig *c;
 		uint in_request_count = 0;
-		struct sockaddr_in out_addr;
 
 		for (c = item->info.grfconfig; c != NULL; c = c->next) {
 			if (c->status == GCS_NOT_FOUND) item->info.compatible = false;
@@ -255,15 +271,17 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 				this->Send_GRFIdentifier(&packet, in_request[i]);
 			}
 
-			out_addr.sin_family      = AF_INET;
-			out_addr.sin_port        = htons(item->port);
-			out_addr.sin_addr.s_addr = item->ip;
-			this->SendPacket(&packet, &out_addr);
+			this->SendPacket(&packet, &item->address);
 		}
 	}
 
-	if (item->info.hostname[0] == '\0')
-		snprintf(item->info.hostname, sizeof(item->info.hostname), "%s", inet_ntoa(client_addr->sin_addr));
+	if (item->info.hostname[0] == '\0') {
+		snprintf(item->info.hostname, sizeof(item->info.hostname), "%s", client_addr->GetHostname());
+	}
+
+	if (client_addr->GetAddress()->ss_family == AF_INET6) {
+		strecat(item->info.server_name, " (IPv6)", lastof(item->info.server_name));
+	}
 
 	/* Check if we are allowed on this server based on the revision-match */
 	item->info.version_compatible = IsNetworkCompatibleVersion(item->info.server_revision);
@@ -279,19 +297,32 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_MASTER_RESPONSE_LIST)
 	/* packet begins with the protocol version (uint8)
 	 * then an uint16 which indicates how many
 	 * ip:port pairs are in this packet, after that
-	 * an uint32 (ip) and an uint16 (port) for each pair
+	 * an uint32 (ip) and an uint16 (port) for each pair.
 	 */
 
-	uint8 ver = p->Recv_uint8();
+	ServerListType type = (ServerListType)(p->Recv_uint8() - 1);
 
-	if (ver == 1) {
+	if (type < SLT_END) {
 		for (int i = p->Recv_uint16(); i != 0 ; i--) {
-			uint32 ip = TO_LE32(p->Recv_uint32());
-			uint16 port = p->Recv_uint16();
+			sockaddr_storage addr_storage;
+			memset(&addr_storage, 0, sizeof(addr_storage));
+
+			if (type == SLT_IPv4) {
+				addr_storage.ss_family = AF_INET;
+				((sockaddr_in*)&addr_storage)->sin_addr.s_addr = TO_LE32(p->Recv_uint32());
+			} else {
+				assert(type == SLT_IPv6);
+				addr_storage.ss_family = AF_INET6;
+				byte *addr = (byte*)&((sockaddr_in6*)&addr_storage)->sin6_addr;
+				for (uint i = 0; i < sizeof(in6_addr); i++) *addr++ = p->Recv_uint8();
+			}
+			NetworkAddress addr(addr_storage, type == SLT_IPv4 ? sizeof(sockaddr_in) : sizeof(sockaddr_in6));
+			addr.SetPort(p->Recv_uint16());
 
 			/* Somehow we reached the end of the packet */
 			if (this->HasClientQuit()) return;
-			NetworkUDPQueryServer(NetworkAddress(ip, port));
+
+			NetworkUDPQueryServer(addr);
 		}
 	}
 }
@@ -302,7 +333,7 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_NEWGRFS)
 	uint8 num_grfs;
 	uint i;
 
-	DEBUG(net, 6, "[udp] newgrf data reply from %s:%d", inet_ntoa(client_addr->sin_addr), ntohs(client_addr->sin_port));
+	DEBUG(net, 6, "[udp] newgrf data reply from %s", client_addr->GetAddressAsString());
 
 	num_grfs = p->Recv_uint8 ();
 	if (num_grfs > NETWORK_MAX_GRF_COUNT) return;
@@ -347,37 +378,15 @@ void ClientNetworkUDPSocketHandler::HandleIncomingNetworkGameInfoGRFConfig(GRFCo
 	SetBit(config->flags, GCF_COPY);
 }
 
-/* Close UDP connection */
-void NetworkUDPCloseAll()
-{
-	DEBUG(net, 1, "[udp] closed listeners");
-
-	_network_udp_mutex->BeginCritical();
-	_udp_server_socket->Close();
-	_udp_master_socket->Close();
-	_udp_client_socket->Close();
-	_network_udp_mutex->EndCritical();
-
-	_network_udp_server = false;
-	_network_udp_broadcast = 0;
-}
-
 /* Broadcast to all ips */
 static void NetworkUDPBroadCast(NetworkUDPSocketHandler *socket)
 {
-	uint i;
-
-	for (i = 0; _broadcast_list[i] != 0; i++) {
+	for (NetworkAddress *addr = _broadcast_list.Begin(); addr != _broadcast_list.End(); addr++) {
 		Packet p(PACKET_UDP_CLIENT_FIND_SERVER);
-		struct sockaddr_in out_addr;
 
-		out_addr.sin_family = AF_INET;
-		out_addr.sin_port = htons(_settings_client.network.server_port);
-		out_addr.sin_addr.s_addr = _broadcast_list[i];
+		DEBUG(net, 4, "[udp] broadcasting to %s", addr->GetHostname());
 
-		DEBUG(net, 4, "[udp] broadcasting to %s", inet_ntoa(out_addr.sin_addr));
-
-		socket->SendPacket(&p, &out_addr);
+		socket->SendPacket(&p, addr, true, true);
 	}
 }
 
@@ -385,24 +394,16 @@ static void NetworkUDPBroadCast(NetworkUDPSocketHandler *socket)
 /* Request the the server-list from the master server */
 void NetworkUDPQueryMasterServer()
 {
-	struct sockaddr_in out_addr;
-
-	if (!_udp_client_socket->IsConnected()) {
-		if (!_udp_client_socket->Listen(0, 0, true)) return;
-	}
-
 	Packet p(PACKET_UDP_CLIENT_GET_LIST);
-
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
-	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
+	NetworkAddress out_addr(NETWORK_MASTER_SERVER_HOST, NETWORK_MASTER_SERVER_PORT);
 
 	/* packet only contains protocol version */
 	p.Send_uint8(NETWORK_MASTER_SERVER_VERSION);
+	p.Send_uint8(SLT_AUTODETECT);
 
-	_udp_client_socket->SendPacket(&p, &out_addr);
+	_udp_client_socket->SendPacket(&p, &out_addr, true);
 
-	DEBUG(net, 2, "[udp] master server queried at %s:%d", inet_ntoa(out_addr.sin_addr), ntohs(out_addr.sin_port));
+	DEBUG(net, 2, "[udp] master server queried at %s", out_addr.GetAddressAsString());
 }
 
 /* Find all servers */
@@ -411,11 +412,6 @@ void NetworkUDPSearchGame()
 	/* We are still searching.. */
 	if (_network_udp_broadcast > 0) return;
 
-	/* No UDP-socket yet.. */
-	if (!_udp_client_socket->IsConnected()) {
-		if (!_udp_client_socket->Listen(0, 0, true)) return;
-	}
-
 	DEBUG(net, 0, "[udp] searching server");
 
 	NetworkUDPBroadCast(_udp_client_socket);
@@ -436,20 +432,14 @@ struct NetworkUDPQueryServerInfo : NetworkAddress {
  * Threaded part for resolving the IP of a server and querying it.
  * @param pntr the NetworkUDPQueryServerInfo.
  */
-void NetworkUDPQueryServerThread(void *pntr)
+static void NetworkUDPQueryServerThread(void *pntr)
 {
 	NetworkUDPQueryServerInfo *info = (NetworkUDPQueryServerInfo*)pntr;
 
-	struct sockaddr_in out_addr;
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(info->GetPort());
-	out_addr.sin_addr.s_addr = info->GetIP();
-
 	/* Clear item in gamelist */
 	NetworkGameList *item = CallocT<NetworkGameList>(1);
-	item->ip = info->GetIP();
-	item->port = info->GetPort();
-	strecpy(item->info.server_name, info->GetHostname(), lastof(item->info.server_name));
+	item->address = *info;
+	info->GetAddressAsString(item->info.server_name, lastof(item->info.server_name));
 	strecpy(item->info.hostname, info->GetHostname(), lastof(item->info.hostname));
 	item->manually = info->manually;
 	NetworkGameListAddItemDelayed(item);
@@ -457,7 +447,7 @@ void NetworkUDPQueryServerThread(void *pntr)
 	_network_udp_mutex->BeginCritical();
 	/* Init the packet */
 	Packet p(PACKET_UDP_CLIENT_FIND_SERVER);
-	if (_udp_client_socket != NULL) _udp_client_socket->SendPacket(&p, &out_addr);
+	if (_udp_client_socket != NULL) _udp_client_socket->SendPacket(&p, info);
 	_network_udp_mutex->EndCritical();
 
 	delete info;
@@ -465,26 +455,18 @@ void NetworkUDPQueryServerThread(void *pntr)
 
 void NetworkUDPQueryServer(NetworkAddress address, bool manually)
 {
-	/* No UDP-socket yet.. */
-	if (!_udp_client_socket->IsConnected()) {
-		if (!_udp_client_socket->Listen(0, 0, true)) return;
-	}
-
 	NetworkUDPQueryServerInfo *info = new NetworkUDPQueryServerInfo(address, manually);
 	if (address.IsResolved() || !ThreadObject::New(NetworkUDPQueryServerThread, info)) {
 		NetworkUDPQueryServerThread(info);
 	}
 }
 
-void NetworkUDPRemoveAdvertiseThread(void *pntr)
+static void NetworkUDPRemoveAdvertiseThread(void *pntr)
 {
 	DEBUG(net, 1, "[udp] removing advertise from master server");
 
 	/* Find somewhere to send */
-	struct sockaddr_in out_addr;
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
-	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
+	NetworkAddress out_addr(NETWORK_MASTER_SERVER_HOST, NETWORK_MASTER_SERVER_PORT);
 
 	/* Send the packet */
 	Packet p(PACKET_UDP_SERVER_UNREGISTER);
@@ -493,7 +475,7 @@ void NetworkUDPRemoveAdvertiseThread(void *pntr)
 	p.Send_uint16(_settings_client.network.server_port);
 
 	_network_udp_mutex->BeginCritical();
-	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr);
+	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr, true);
 	_network_udp_mutex->EndCritical();
 }
 
@@ -506,51 +488,55 @@ void NetworkUDPRemoveAdvertise(bool blocking)
 	/* Check if we are advertising */
 	if (!_networking || !_network_server || !_network_udp_server) return;
 
-	/* check for socket */
-	if (!_udp_master_socket->IsConnected()) {
-		if (!_udp_master_socket->Listen(_network_server_bind_ip, 0, false)) return;
-	}
-
 	if (blocking || !ThreadObject::New(NetworkUDPRemoveAdvertiseThread, NULL)) {
 		NetworkUDPRemoveAdvertiseThread(NULL);
 	}
 }
 
-void NetworkUDPAdvertiseThread(void *pntr)
+static void NetworkUDPAdvertiseThread(void *pntr)
 {
 	/* Find somewhere to send */
-	struct sockaddr_in out_addr;
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
-	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
+	NetworkAddress out_addr(NETWORK_MASTER_SERVER_HOST, NETWORK_MASTER_SERVER_PORT);
 
 	DEBUG(net, 1, "[udp] advertising to master server");
 
+	/* Add a bit more messaging when we cannot get a session key */
+	static byte session_key_retries = 0;
+	if (_session_key == 0 && session_key_retries++ == 2) {
+		DEBUG(net, 0, "[udp] advertising to the master server is failing");
+		DEBUG(net, 0, "[udp]   we are not receiving the session key from the server");
+		DEBUG(net, 0, "[udp]   please allow udp packets from %s to you to be delivered", out_addr.GetAddressAsString(false));
+		DEBUG(net, 0, "[udp]   please allow udp packets from you to %s to be delivered", out_addr.GetAddressAsString(false));
+	}
+	if (_session_key != 0 && _network_advertise_retries == 0) {
+		DEBUG(net, 0, "[udp] advertising to the master server is failing");
+		DEBUG(net, 0, "[udp]   we are not receiving the acknowledgement from the server");
+		DEBUG(net, 0, "[udp]   this usually means that the master server cannot reach us");
+		DEBUG(net, 0, "[udp]   please allow udp and tcp packets to port %u to be delivered", _settings_client.network.server_port);
+		DEBUG(net, 0, "[udp]   please allow udp and tcp packets from port %u to be delivered", _settings_client.network.server_port);
+	}
+
 	/* Send the packet */
 	Packet p(PACKET_UDP_SERVER_REGISTER);
 	/* Packet is: WELCOME_MESSAGE, Version, server_port */
 	p.Send_string(NETWORK_MASTER_SERVER_WELCOME_MESSAGE);
 	p.Send_uint8 (NETWORK_MASTER_SERVER_VERSION);
 	p.Send_uint16(_settings_client.network.server_port);
+	p.Send_uint64(_session_key);
 
 	_network_udp_mutex->BeginCritical();
-	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr);
+	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr, true);
 	_network_udp_mutex->EndCritical();
 }
 
 /* Register us to the master server
-     This function checks if it needs to send an advertise */
+ *   This function checks if it needs to send an advertise */
 void NetworkUDPAdvertise()
 {
 	/* Check if we should send an advertise */
 	if (!_networking || !_network_server || !_network_udp_server || !_settings_client.network.server_advertise)
 		return;
 
-	/* check for socket */
-	if (!_udp_master_socket->IsConnected()) {
-		if (!_udp_master_socket->Listen(_network_server_bind_ip, 0, false)) return;
-	}
-
 	if (_network_need_advertise) {
 		_network_need_advertise = false;
 		_network_advertise_retries = ADVERTISE_RETRY_TIMES;
@@ -576,23 +562,35 @@ void NetworkUDPAdvertise()
 
 void NetworkUDPInitialize()
 {
+	/* If not closed, then do it. */
+	if (_udp_server_socket != NULL) NetworkUDPClose();
+
+	DEBUG(net, 1, "[udp] initializing listeners");
 	assert(_udp_client_socket == NULL && _udp_server_socket == NULL && _udp_master_socket == NULL);
 
 	_network_udp_mutex->BeginCritical();
+
 	_udp_client_socket = new ClientNetworkUDPSocketHandler();
-	_udp_server_socket = new ServerNetworkUDPSocketHandler();
-	_udp_master_socket = new MasterNetworkUDPSocketHandler();
+
+	NetworkAddressList server;
+	GetBindAddresses(&server, _settings_client.network.server_port);
+	_udp_server_socket = new ServerNetworkUDPSocketHandler(&server);
+
+	server.Clear();
+	GetBindAddresses(&server, 0);
+	_udp_master_socket = new MasterNetworkUDPSocketHandler(&server);
 
 	_network_udp_server = false;
 	_network_udp_broadcast = 0;
 	_network_udp_mutex->EndCritical();
 }
 
-void NetworkUDPShutdown()
+void NetworkUDPClose()
 {
-	NetworkUDPCloseAll();
-
 	_network_udp_mutex->BeginCritical();
+	_udp_server_socket->Close();
+	_udp_master_socket->Close();
+	_udp_client_socket->Close();
 	delete _udp_client_socket;
 	delete _udp_server_socket;
 	delete _udp_master_socket;
@@ -600,6 +598,10 @@ void NetworkUDPShutdown()
 	_udp_server_socket = NULL;
 	_udp_master_socket = NULL;
 	_network_udp_mutex->EndCritical();
+
+	_network_udp_server = false;
+	_network_udp_broadcast = 0;
+	DEBUG(net, 1, "[udp] closed listeners");
 }
 
 #endif /* ENABLE_NETWORK */
