@@ -1,14 +1,19 @@
-/* $Id: newgrf_cargo.cpp 13871 2008-07-29 22:37:54Z rubidium $ */
+/* $Id: newgrf_cargo.cpp 19132 2010-02-14 16:31:35Z alberth $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/** @file newgrf_cargo.cpp Implementation of NewGRF cargoes. */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "debug.h"
-#include "cargotype.h"
 #include "newgrf.h"
-#include "newgrf_callbacks.h"
 #include "newgrf_spritegroup.h"
-#include "newgrf_cargo.h"
-
+#include "core/bitmath_func.hpp"
 
 static uint32 CargoGetRandomBits(const ResolverObject *object)
 {
@@ -33,16 +38,16 @@ static uint32 CargoGetVariable(const ResolverObject *object, byte variable, byte
 	DEBUG(grf, 1, "Unhandled cargo property 0x%X", variable);
 
 	*available = false;
-	return 0;
+	return UINT_MAX;
 }
 
 
-static const SpriteGroup *CargoResolveReal(const ResolverObject *object, const SpriteGroup *group)
+static const SpriteGroup *CargoResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
 {
 	/* Cargo action 2s should always have only 1 "loaded" state, but some
 	 * times things don't follow the spec... */
-	if (group->g.real.num_loaded > 0) return group->g.real.loaded[0];
-	if (group->g.real.num_loading > 0) return group->g.real.loading[0];
+	if (group->num_loaded > 0) return group->loaded[0];
+	if (group->num_loading > 0) return group->loading[0];
 
 	return NULL;
 }
@@ -64,6 +69,8 @@ static void NewCargoResolver(ResolverObject *res, const CargoSpec *cs)
 	res->last_value      = 0;
 	res->trigger         = 0;
 	res->reseed          = 0;
+	res->count           = 0;
+	res->grffile         = cs->grffile;
 }
 
 
@@ -74,10 +81,10 @@ SpriteID GetCustomCargoSprite(const CargoSpec *cs)
 
 	NewCargoResolver(&object, cs);
 
-	group = Resolve(cs->group, &object);
-	if (group == NULL || group->type != SGT_RESULT) return 0;
+	group = SpriteGroup::Resolve(cs->group, &object);
+	if (group == NULL) return 0;
 
-	return group->g.result.sprite;
+	return group->GetResult();
 }
 
 
@@ -91,10 +98,10 @@ uint16 GetCargoCallback(CallbackID callback, uint32 param1, uint32 param2, const
 	object.callback_param1 = param1;
 	object.callback_param2 = param2;
 
-	group = Resolve(cs->group, &object);
-	if (group == NULL || group->type != SGT_CALLBACK) return CALLBACK_FAILED;
+	group = SpriteGroup::Resolve(cs->group, &object);
+	if (group == NULL) return CALLBACK_FAILED;
 
-	return group->g.callback.result;
+	return group->GetCallbackResult();
 }
 
 
@@ -122,7 +129,7 @@ CargoID GetCargoTranslation(uint8 cargo, const GRFFile *grffile, bool usebit)
 uint8 GetReverseCargoTranslation(CargoID cargo, const GRFFile *grffile)
 {
 	/* Note: All grf versions use CargoBit here. Pre-version 7 do NOT use the 'climate dependent' ID. */
-	const CargoSpec *cs = GetCargo(cargo);
+	const CargoSpec *cs = CargoSpec::Get(cargo);
 
 	/* If the GRF contains a translation table (and the cargo is in the table)
 	 * then get the cargo ID for the label */
@@ -131,5 +138,5 @@ uint8 GetReverseCargoTranslation(CargoID cargo, const GRFFile *grffile)
 	}
 
 	/* No matching label was found, so we return the 'climate independent' 'bitnum' */
-	return cs->bitnum;;
+	return cs->bitnum;
 }
