@@ -1,4 +1,11 @@
-/* $Id: newgrf_commons.cpp 15583 2009-02-25 21:29:50Z frosch $ */
+/* $Id: newgrf_commons.cpp 20432 2010-08-09 21:58:52Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file newgrf_commons.cpp Implementation of the class OverrideManagerBase
  * and its descendance, present and futur
@@ -6,12 +13,16 @@
 
 #include "stdafx.h"
 #include "landscape.h"
-#include "town.h"
-#include "industry.h"
+#include "house.h"
+#include "industrytype.h"
 #include "newgrf.h"
 #include "newgrf_commons.h"
+#include "clear_map.h"
 #include "station_map.h"
 #include "tree_map.h"
+#include "tunnelbridge_map.h"
+#include "variables.h"
+#include "core/mem_func.hpp"
 
 /** Constructor of generic class
  * @param offset end of original data for this entity. i.e: houses = 110
@@ -128,7 +139,7 @@ uint16 OverrideManagerBase::AddEntityID(byte grf_local_id, uint32 grfid, byte su
  * @param entity_id of the entity being queried
  * @return mapped id
  */
-uint16 OverrideManagerBase::GetSubstituteID(byte entity_id)
+uint16 OverrideManagerBase::GetSubstituteID(uint16 entity_id)
 {
 	return mapping_ID[entity_id].substitute_id;
 }
@@ -146,11 +157,11 @@ void HouseOverrideManager::SetEntitySpec(const HouseSpec *hs)
 		return;
 	}
 
-	memcpy(&_house_specs[house_id], hs, sizeof(*hs));
+	MemCpyT(HouseSpec::Get(house_id), hs);
 
 	/* Now add the overrides. */
 	for (int i = 0; i != max_offset; i++) {
-		HouseSpec *overridden_hs = GetHouseSpecs(i);
+		HouseSpec *overridden_hs = HouseSpec::Get(i);
 
 		if (entity_overrides[i] != hs->local_id || grfid_overrides[i] != hs->grffile->grfid) continue;
 
@@ -270,13 +281,72 @@ void IndustryTileOverrideManager::SetEntitySpec(const IndustryTileSpec *its)
 /** Function used by houses (and soon industries) to get information
  * on type of "terrain" the tile it is queries sits on.
  * @param tile TileIndex of the tile been queried
+ * @param upper_halftile If true, query upper halftile in case of rail tiles.
  * @return value corresponding to the grf expected format:
  *         Terrain type: 0 normal, 1 desert, 2 rainforest, 4 on or above snowline */
-uint32 GetTerrainType(TileIndex tile)
+uint32 GetTerrainType(TileIndex tile, TileContext context)
 {
 	switch (_settings_game.game_creation.landscape) {
 		case LT_TROPIC: return GetTropicZone(tile);
-		case LT_ARCTIC: return GetTileZ(tile) > GetSnowLine() ? 4 : 0;
+		case LT_ARCTIC: {
+			bool has_snow;
+			switch (GetTileType(tile)) {
+				case MP_CLEAR:
+					/* During map generation the snowstate may not be valid yet, as the tileloop may not have run yet. */
+					if (_generating_world) goto genworld;
+					has_snow = IsSnowTile(tile) && GetClearDensity(tile) >= 2;
+					break;
+
+				case MP_RAILWAY: {
+					/* During map generation the snowstate may not be valid yet, as the tileloop may not have run yet. */
+					if (_generating_world) goto genworld; // we do not care about foundations here
+					RailGroundType ground = GetRailGroundType(tile);
+					has_snow = (ground == RAIL_GROUND_ICE_DESERT || (context == TCX_UPPER_HALFTILE && ground == RAIL_GROUND_HALF_SNOW));
+					break;
+				}
+
+				case MP_ROAD:
+					/* During map generation the snowstate may not be valid yet, as the tileloop may not have run yet. */
+					if (_generating_world) goto genworld; // we do not care about foundations here
+					has_snow = IsOnSnow(tile);
+					break;
+
+				case MP_TREES: {
+					/* During map generation the snowstate may not be valid yet, as the tileloop may not have run yet. */
+					if (_generating_world) goto genworld;
+					TreeGround ground = GetTreeGround(tile);
+					has_snow = (ground == TREE_GROUND_SNOW_DESERT || ground == TREE_GROUND_ROUGH_SNOW) && GetTreeDensity(tile) >= 2;
+					break;
+				}
+
+				case MP_TUNNELBRIDGE:
+					if (context == TCX_ON_BRIDGE) {
+						has_snow = (GetBridgeHeight(tile) > GetSnowLine());
+					} else {
+						/* During map generation the snowstate may not be valid yet, as the tileloop may not have run yet. */
+						if (_generating_world) goto genworld; // we do not care about foundations here
+						has_snow = HasTunnelBridgeSnowOrDesert(tile);
+					}
+					break;
+
+				case MP_STATION:
+				case MP_HOUSE:
+				case MP_INDUSTRY:
+				case MP_UNMOVABLE:
+					/* These tiles usually have a levelling foundation. So use max Z */
+					has_snow = (GetTileMaxZ(tile) > GetSnowLine());
+					break;
+
+				case MP_VOID:
+				case MP_WATER:
+				genworld:
+					has_snow = (GetTileZ(tile) > GetSnowLine());
+					break;
+
+				default: NOT_REACHED();
+			}
+			return has_snow ? 4 : 0;
+		}
 		default:        return 0;
 	}
 }
@@ -290,14 +360,14 @@ TileIndex GetNearbyTile(byte parameter, TileIndex tile)
 	if (y >= 8) y -= 16;
 
 	/* Swap width and height depending on axis for railway stations */
-	if (IsRailwayStationTile(tile) && GetRailStationAxis(tile) == AXIS_Y) Swap(x, y);
+	if (HasStationTileRail(tile) && GetRailStationAxis(tile) == AXIS_Y) Swap(x, y);
 
 	/* Make sure we never roam outside of the map, better wrap in that case */
 	return TILE_MASK(tile + TileDiffXY(x, y));
 }
 
 /**
- * Common part of station var 0x67 , house var 0x62, indtile var 0x60, industry var 0x62.
+ * Common part of station var 0x67, house var 0x62, indtile var 0x60, industry var 0x62.
  *
  * @param tile the tile of interest.
  * @return 0czzbbss: c = TileType; zz = TileZ; bb: 7-3 zero, 4-2 TerrainType, 1 water/shore, 0 zero; ss = TileSlope
