@@ -1,4 +1,11 @@
-/* $Id: newgrf_commons.h 15428 2009-02-09 02:57:15Z rubidium $ */
+/* $Id: newgrf_commons.h 20432 2010-08-09 21:58:52Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file newgrf_commons.h This file simplyfies and embeds a common mechanism of
  * loading/saving and mapping of grf entities.
@@ -7,9 +14,14 @@
 #ifndef NEWGRF_COMMONS_H
 #define NEWGRF_COMMONS_H
 
-#include "core/bitmath_func.hpp"
+#include "tile_cmd.h"
 
-#include "table/sprites.h"
+/** Contextx for tile accesses */
+enum TileContext {
+	TCX_NORMAL,         ///< Nothing special.
+	TCX_UPPER_HALFTILE, ///< Querying information about the upper part of a tile with halftile foundation.
+	TCX_ON_BRIDGE,      ///< Querying information about stuff on the bridge (via some bridgehead).
+};
 
 /**
  * Maps an entity id stored on the map to a GRF file.
@@ -52,7 +64,7 @@ public:
 	void Add(uint8 local_id, uint32 grfid, uint entity_type);
 	virtual uint16 AddEntityID(byte grf_local_id, uint32 grfid, byte substitute_id);
 
-	uint16 GetSubstituteID(byte entity_id);
+	uint16 GetSubstituteID(uint16 entity_id);
 	virtual uint16 GetID(uint8 grf_local_id, uint32 grfid);
 
 	inline uint16 GetMaxMapping() { return max_new_entities; }
@@ -96,47 +108,8 @@ extern HouseOverrideManager _house_mngr;
 extern IndustryOverrideManager _industry_mngr;
 extern IndustryTileOverrideManager _industile_mngr;
 
-uint32 GetTerrainType(TileIndex tile);
+uint32 GetTerrainType(TileIndex tile, TileContext context = TCX_NORMAL);
 TileIndex GetNearbyTile(byte parameter, TileIndex tile);
 uint32 GetNearbyTileInformation(TileIndex tile);
 
-/**
- * Applies PALETTE_MODIFIER_TRANSPARENT and PALETTE_MODIFIER_COLOUR to a palette entry of a sprite layout entry
- * @Note for ground sprites use #GroundSpritePaletteTransform
- * @Note Not useable for OTTD internal spritelayouts from table/xxx_land.h as PALETTE_MODIFIER_TRANSPARENT is only set
- *       when to use the default palette.
- *
- * @param image The sprite to draw
- * @param pal The palette from the sprite layout
- * @param default_pal The default recolour sprite to use (typically company colour resp. random industry/house colour)
- * @return The palette to use
- */
-static inline SpriteID SpriteLayoutPaletteTransform(SpriteID image, SpriteID pal, SpriteID default_pal)
-{
-	if (HasBit(image, PALETTE_MODIFIER_TRANSPARENT) || HasBit(image, PALETTE_MODIFIER_COLOUR)) {
-		return (pal != 0 ? pal : default_pal);
-	} else {
-		return PAL_NONE;
-	}
-}
-
-/**
- * Applies PALETTE_MODIFIER_COLOUR to a palette entry of a ground sprite
- * @Note Not useable for OTTD internal spritelayouts from table/xxx_land.h as PALETTE_MODIFIER_TRANSPARENT is only set
- *       when to use the default palette.
- *
- * @param image The sprite to draw
- * @param pal The palette from the sprite layout
- * @param default_pal The default recolour sprite to use (typically company colour resp. random industry/house colour)
- * @return The palette to use
- */
-static inline SpriteID GroundSpritePaletteTransform(SpriteID image, SpriteID pal, SpriteID default_pal)
-{
-	if (HasBit(image, PALETTE_MODIFIER_COLOUR)) {
-		return (pal != 0 ? pal : default_pal);
-	} else {
-		return PAL_NONE;
-	}
-}
-
 #endif /* NEWGRF_COMMONS_H */
