@@ -1,27 +1,29 @@
-/* $Id: newgrf_industries.cpp 14426 2008-10-01 11:48:57Z rubidium $ */
+/* $Id: newgrf_industries.cpp 19938 2010-06-05 21:13:35Z rubidium $ */
 
-/** @file newgrf_industries.cpp */
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/** @file newgrf_industries.cpp Handling of NewGRF industries. */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "debug.h"
-#include "variables.h"
-#include "landscape.h"
 #include "industry.h"
-#include "industry_map.h"
 #include "newgrf.h"
-#include "newgrf_callbacks.h"
-#include "newgrf_spritegroup.h"
 #include "newgrf_industries.h"
-#include "newgrf_industrytiles.h"
 #include "newgrf_commons.h"
 #include "newgrf_text.h"
 #include "newgrf_town.h"
 #include "window_func.h"
 #include "town.h"
-#include "player_func.h"
-#include "player_base.h"
+#include "company_base.h"
 #include "command_func.h"
+#include "gui.h"
+#include "strings_func.h"
+#include "core/random_func.hpp"
 
 #include "table/strings.h"
 
@@ -41,76 +43,15 @@ IndustryType MapNewGRFIndustryType(IndustryType grf_type, uint32 grf_id)
 	return _industry_mngr.GetID(GB(grf_type, 0, 6), grf_id);
 }
 
-static uint32 GetGRFParameter(IndustryType ind_id, byte parameter)
-{
-	const IndustrySpec *indspec = GetIndustrySpec(ind_id);
-	const GRFFile *file = indspec->grf_prop.grffile;
-
-	if (parameter >= file->param_end) return 0;
-	return file->param[parameter];
-}
-
 /**
- * Finds the distance for the closest tile with water/land given a tile
- * @param tile  the tile to find the distance too
- * @param water whether to find water or land
- * @return distance to nearest water (max 0x7F) / land (max 0x1FF; 0x200 if there is no land)
- * @note FAILS when an industry should be seen as water
- */
-static uint GetClosestWaterDistance(TileIndex tile, bool water)
-{
-	if (IsTileType(tile, MP_WATER) == water) return 0;
-
-	uint max_dist = water ? 0x7F : 0x200;
-
-	int x = TileX(tile);
-	int y = TileY(tile);
-
-	uint max_x = MapMaxX();
-	uint max_y = MapMaxY();
-
-	/* go in a 'spiral' with increasing manhattan distance in each iteration */
-	for (uint dist = 1; dist < max_dist; dist++) {
-		/* next 'diameter' */
-		y--;
-
-		/* going counter-clockwise around this square */
-		for (DiagDirection dir = DIAGDIR_BEGIN; dir < DIAGDIR_END; dir++) {
-			static const int8 ddx[DIAGDIR_END] = { -1,  1,  1, -1};
-			static const int8 ddy[DIAGDIR_END] = {  1,  1, -1, -1};
-
-			int dx = ddx[dir];
-			int dy = ddy[dir];
-
-			/* each side of this square has length 'dist' */
-			for (uint a = 0; a < dist; a++) {
-				/* MP_VOID tiles are not checked (interval is [0; max) for IsInsideMM())*/
-				if (IsInsideMM(x, 0, max_x) && IsInsideMM(y, 0, max_y)) {
-					TileIndex t = TileXY(x, y);
-					if (IsTileType(t, MP_WATER) == water) return dist;
-				}
-				x += dx;
-				y += dy;
-			}
-		}
-	}
-
-	if (!water) {
-		/* no land found - is this a water-only map? */
-		for (TileIndex t = 0; t < MapSize(); t++) {
-			if (!IsTileType(t, MP_VOID) && !IsTileType(t, MP_WATER)) return 0x1FF;
-		}
-	}
-
-	return max_dist;
-}
-
-/** Make an analysis of a tile and check for its belonging to the same
+ * Make an analysis of a tile and check for its belonging to the same
  * industry, and/or the same grf file
  * @param tile TileIndex of the tile to query
  * @param i Industry to which to compare the tile to
- * @return value encoded as per NFO specs */
-uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i)
+ * @param cur_grfid GRFID of the current callback chain
+ * @return value encoded as per NFO specs
+ */
+uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i, uint32 cur_grfid)
 {
 	if (!IsTileType(tile, MP_INDUSTRY) || GetIndustryIndex(tile) != i->index) {
 		/* No industry and/or the tile does not have the same industry as the one we match it with */
@@ -119,17 +60,16 @@ uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i)
 
 	IndustryGfx gfx = GetCleanIndustryGfx(tile);
 	const IndustryTileSpec *indtsp = GetIndustryTileSpec(gfx);
-	const IndustrySpec *indold = GetIndustrySpec(i->type);
 
 	if (gfx < NEW_INDUSTRYOFFSET) { // Does it belongs to an old type?
 		/* It is an old tile.  We have to see if it's been overriden */
 		if (indtsp->grf_prop.override == INVALID_INDUSTRYTILE) { // has it been overridden?
 			return 0xFF << 8 | gfx; // no. Tag FF + the gfx id of that tile
 		}
-		/* Not overriden */
+		/* Overriden */
 		const IndustryTileSpec *tile_ovr = GetIndustryTileSpec(indtsp->grf_prop.override);
 
-		if (tile_ovr->grf_prop.grffile->grfid == indold->grf_prop.grffile->grfid) {
+		if (tile_ovr->grf_prop.grffile->grfid == cur_grfid) {
 			return tile_ovr->grf_prop.local_id; // same grf file
 		} else {
 			return 0xFFFE; // not the same grf file
@@ -137,7 +77,7 @@ uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i)
 	}
 	/* Not an 'old type' tile */
 	if (indtsp->grf_prop.spritegroup != NULL) { // tile has a spritegroup ?
-		if (indtsp->grf_prop.grffile->grfid == indold->grf_prop.grffile->grfid) { // same industry, same grf ?
+		if (indtsp->grf_prop.grffile->grfid == cur_grfid) { // same industry, same grf ?
 			return indtsp->grf_prop.local_id;
 		} else {
 			return 0xFFFE; // Defined in another grf file
@@ -149,12 +89,12 @@ uint32 GetIndustryIDAtOffset(TileIndex tile, const Industry *i)
 
 static uint32 GetClosestIndustry(TileIndex tile, IndustryType type, const Industry *current)
 {
-	uint32 best_dist = MAX_UVALUE(uint32);
+	uint32 best_dist = UINT32_MAX;
 	const Industry *i;
 	FOR_ALL_INDUSTRIES(i) {
 		if (i->type != type || i == current) continue;
 
-		best_dist = min(best_dist, DistanceManhattan(tile, i->xy));
+		best_dist = min(best_dist, DistanceManhattan(tile, i->location.tile));
 	}
 
 	return best_dist;
@@ -172,7 +112,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 {
 	uint32 GrfID = GetRegister(0x100);  ///< Get the GRFID of the definition to look for in register 100h
 	IndustryType ind_index;
-	uint32 closest_dist = MAX_UVALUE(uint32);
+	uint32 closest_dist = UINT32_MAX;
 	byte count = 0;
 
 	/* Determine what will be the industry type to look for */
@@ -185,16 +125,19 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 			GrfID = GetIndustrySpec(current->type)->grf_prop.grffile->grfid;
 			/* Fall through */
 
-		default: //use the grfid specified in register 100h
+		default: // use the grfid specified in register 100h
 			SetBit(param_setID, 7); // bit 7 means it is not an old type
 			ind_index = MapNewGRFIndustryType(param_setID, GrfID);
 			break;
 	}
 
+	/* If the industry type is invalid, there is none and the closest is far away. */
+	if (ind_index >= NUM_INDUSTRYTYPES) return 0 | 0xFFFF;
+
 	if (layout_filter == 0) {
 		/* If the filter is 0, it could be because none was specified as well as being really a 0.
 		 * In either case, just do the regular var67 */
-		closest_dist = GetClosestIndustry(current->xy, ind_index, current);
+		closest_dist = GetClosestIndustry(current->location.tile, ind_index, current);
 		count = GetIndustryTypeCount(ind_index);
 	} else {
 		/* Count only those who match the same industry type and layout filter
@@ -202,7 +145,7 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 		const Industry *i;
 		FOR_ALL_INDUSTRIES(i) {
 			if (i->type == ind_index && i != current && i->selected_layout == layout_filter) {
-				closest_dist = min(closest_dist, DistanceManhattan(current->xy, i->xy));
+				closest_dist = min(closest_dist, DistanceManhattan(current->location.tile, i->location.tile));
 				count++;
 			}
 		}
@@ -212,10 +155,10 @@ static uint32 GetCountAndDistanceOfClosestInstance(byte param_setID, byte layout
 }
 
 /** This function implements the industries variables that newGRF defines.
+ * @param object the object that we want to query
  * @param variable that is queried
  * @param parameter unused
  * @param available will return false if ever the variable asked for does not exist
- * @param ind is of course the industry we are inquiring
  * @return the value stored in the corresponding variable*/
 uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
 {
@@ -242,14 +185,6 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 	}
 
 	if (industry == NULL) {
-		/* industry does not exist, only use those variables that are "safe" */
-		switch (variable) {
-			/* Read GRF parameter */
-			case 0x7F: return GetGRFParameter(type, parameter);
-			/* Manhattan distance of closes dry/water tile */
-			case 0x43: return GetClosestWaterDistance(tile, (indspec->behaviour & INDUSTRYBEH_BUILT_ONWATER) == 0);
-		}
-
 		DEBUG(grf, 1, "Unhandled property 0x%X (no available industry) in callback 0x%x", variable, object->callback);
 
 		*available = false;
@@ -260,9 +195,10 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		case 0x40:
 		case 0x41:
 		case 0x42: { // waiting cargo, but only if those two callback flags are set
-			uint16 callback = indspec->callback_flags;
+			uint16 callback = indspec->callback_mask;
 			if (HasBit(callback, CBM_IND_PRODUCTION_CARGO_ARRIVAL) || HasBit(callback, CBM_IND_PRODUCTION_256_TICKS)) {
 				if ((indspec->behaviour & INDUSTRYBEH_PROD_MULTI_HNDLING) != 0) {
+					if (industry->prod_level == 0) return 0;
 					return min(industry->incoming_cargo_waiting[variable - 0x40] / industry->prod_level, (uint16)0xFFFF);
 				} else {
 					return min(industry->incoming_cargo_waiting[variable - 0x40], (uint16)0xFFFF);
@@ -278,31 +214,31 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		/* Layout number */
 		case 0x44: return industry->selected_layout;
 
-		/* player info */
+		/* Company info */
 		case 0x45: {
-			byte colours;
+			byte colours = 0;
 			bool is_ai = false;
 
-			if (IsValidPlayer(industry->founder)) {
-				const Player *p = GetPlayer(industry->founder);
-				const Livery *l = &p->livery[LS_DEFAULT];
+			const Company *c = Company::GetIfValid(industry->founder);
+			if (c != NULL) {
+				const Livery *l = &c->livery[LS_DEFAULT];
 
-				is_ai = p->is_ai;
+				is_ai = c->is_ai;
 				colours = l->colour1 + l->colour2 * 16;
-			} else {
-				colours = GB(Random(), 0, 8);
 			}
 
 			return industry->founder | (is_ai ? 0x10000 : 0) | (colours << 24);
 		}
 
+		case 0x46: return industry->construction_date; // Date when built - long format - (in days)
+
 		/* Get industry ID at offset param */
-		case 0x60: return GetIndustryIDAtOffset(GetNearbyTile(parameter, industry->xy), industry);
+		case 0x60: return GetIndustryIDAtOffset(GetNearbyTile(parameter, industry->location.tile), industry, object->grffile->grfid);
 
 		/* Get random tile bits at offset param */
 		case 0x61:
 			tile = GetNearbyTile(parameter, tile);
-			return (IsTileType(tile, MP_INDUSTRY) && GetIndustryByTile(tile) == industry) ? GetIndustryRandomBits(tile) : 0;
+			return (IsTileType(tile, MP_INDUSTRY) && Industry::GetByTile(tile) == industry) ? GetIndustryRandomBits(tile) : 0;
 
 		/* Land info of nearby tiles */
 		case 0x62: return GetNearbyIndustryTileInformation(parameter, tile, INVALID_INDUSTRY);
@@ -310,7 +246,7 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		/* Animation stage of nearby tiles */
 		case 0x63:
 			tile = GetNearbyTile(parameter, tile);
-			if (IsTileType(tile, MP_INDUSTRY) && GetIndustryByTile(tile) == industry) {
+			if (IsTileType(tile, MP_INDUSTRY) && Industry::GetByTile(tile) == industry) {
 				return GetIndustryAnimationState(tile);
 			}
 			return 0xFFFFFFFF;
@@ -318,7 +254,7 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		/* Distance of nearest industry of given type */
 		case 0x64: return GetClosestIndustry(tile, MapNewGRFIndustryType(parameter, indspec->grf_prop.grffile->grfid), industry);
 		/* Get town zone and Manhattan distance of closest town */
- 		case 0x65: return GetTownRadiusGroup(industry->town, tile) << 16 | min(DistanceManhattan(tile, industry->town->xy), 0xFFFF);
+		case 0x65: return GetTownRadiusGroup(industry->town, tile) << 16 | min(DistanceManhattan(tile, industry->town->xy), 0xFFFF);
 		/* Get square of Euclidian distance of closes town */
 		case 0x66: return GetTownRadiusGroup(industry->town, tile) << 16 | min(DistanceSquare(tile, industry->town->xy), 0xFFFF);
 
@@ -330,20 +266,17 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		/* Get a variable from the persistent storage */
 		case 0x7C: return industry->psa.Get(parameter);
 
-		/* Read GRF parameter */
-		case 0x7F: return GetGRFParameter(type, parameter);
-
 		/* Industry structure access*/
-		case 0x80: return industry->xy;
-		case 0x81: return GB(industry->xy, 8, 8);
+		case 0x80: return industry->location.tile;
+		case 0x81: return GB(industry->location.tile, 8, 8);
 		/* Pointer to the town the industry is associated with */
 		case 0x82: return industry->town->index;
 		case 0x83:
 		case 0x84:
 		case 0x85: DEBUG(grf, 0, "NewGRFs shouldn't be doing pointer magic"); break; // not supported
-		case 0x86: return industry->width;
-		case 0x87: return industry->height;// xy dimensions
-		/*  */
+		case 0x86: return industry->location.w;
+		case 0x87: return industry->location.h;// xy dimensions
+
 		case 0x88:
 		case 0x89: return industry->produced_cargo[variable - 0x88];
 		case 0x8A: return industry->produced_cargo_waiting[0];
@@ -365,7 +298,7 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		case 0x98: return industry->this_month_transported[0];
 		case 0x99: return GB(industry->this_month_transported[0], 8, 8);
 		case 0x9A: return industry->this_month_transported[1];
-		case 0x9B: return GB(industry->this_month_transported[0], 8, 8);
+		case 0x9B: return GB(industry->this_month_transported[1], 8, 8);
 		/* fraction of cargo transported LAST month. */
 		case 0x9C:
 		case 0x9D: return industry->last_month_pct_transported[variable - 0x9C];
@@ -378,11 +311,11 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 		case 0xA2: return industry->last_month_transported[0];
 		case 0xA3: return GB(industry->last_month_transported[0], 8, 8);
 		case 0xA4: return industry->last_month_transported[1];
-		case 0xA5: return GB(industry->last_month_transported[0], 8, 8);
+		case 0xA5: return GB(industry->last_month_transported[1], 8, 8);
 
 		case 0xA6: return industry->type;
 		case 0xA7: return industry->founder;
-		case 0xA8: return industry->random_color;
+		case 0xA8: return industry->random_colour;
 		case 0xA9: return Clamp(industry->last_prod_year - ORIGINAL_BASE_YEAR, 0, 255);
 		case 0xAA: return industry->counter;
 		case 0xAB: return GB(industry->counter, 8, 8);
@@ -396,10 +329,10 @@ uint32 IndustryGetVariable(const ResolverObject *object, byte variable, byte par
 	DEBUG(grf, 1, "Unhandled industry property 0x%X", variable);
 
 	*available = false;
-	return (uint32)-1;
+	return UINT_MAX;
 }
 
-static const SpriteGroup *IndustryResolveReal(const ResolverObject *object, const SpriteGroup *group)
+static const SpriteGroup *IndustryResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
 {
 	/* IndustryTile do not have 'real' groups */
 	return NULL;
@@ -407,18 +340,21 @@ static const SpriteGroup *IndustryResolveReal(const ResolverObject *object, cons
 
 static uint32 IndustryGetRandomBits(const ResolverObject *object)
 {
-	return object->u.industry.ind == NULL ? 0 : object->u.industry.ind->random;
+	const Industry *ind = object->u.industry.ind;
+	return ind != NULL ? ind->random: 0;
 }
 
 static uint32 IndustryGetTriggers(const ResolverObject *object)
 {
-	return object->u.industry.ind == NULL ? 0 : object->u.industry.ind->random_triggers;
+	const Industry *ind = object->u.industry.ind;
+	return ind != NULL ? ind->random_triggers : 0;
 }
 
 static void IndustrySetTriggers(const ResolverObject *object, int triggers)
 {
-	if (object->u.industry.ind == NULL) return;
-	object->u.industry.ind->random_triggers = triggers;
+	Industry *ind = object->u.industry.ind;
+	assert(ind != NULL && ind->index != INVALID_INDUSTRY);
+	ind->random_triggers = triggers;
 }
 
 static void NewIndustryResolver(ResolverObject *res, TileIndex tile, Industry *indus, IndustryType type)
@@ -441,6 +377,10 @@ static void NewIndustryResolver(ResolverObject *res, TileIndex tile, Industry *i
 	res->last_value      = 0;
 	res->trigger         = 0;
 	res->reseed          = 0;
+	res->count           = 0;
+
+	const IndustrySpec *indspec = GetIndustrySpec(type);
+	res->grffile         = (indspec != NULL ? indspec->grf_prop.grffile : NULL);
 }
 
 uint16 GetIndustryCallback(CallbackID callback, uint32 param1, uint32 param2, Industry *industry, IndustryType type, TileIndex tile)
@@ -453,10 +393,10 @@ uint16 GetIndustryCallback(CallbackID callback, uint32 param1, uint32 param2, In
 	object.callback_param1 = param1;
 	object.callback_param2 = param2;
 
-	group = Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
-	if (group == NULL || group->type != SGT_CALLBACK) return CALLBACK_FAILED;
+	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
+	if (group == NULL) return CALLBACK_FAILED;
 
-	return group->g.callback.result;
+	return group->GetCallbackResult();
 }
 
 uint32 IndustryLocationGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
@@ -516,34 +456,35 @@ bool CheckIfCallBackAllowsCreation(TileIndex tile, IndustryType type, uint itspe
 
 	Industry ind;
 	ind.index = INVALID_INDUSTRY;
-	ind.xy = tile;
-	ind.width = 0;
+	ind.location.tile = tile;
+	ind.location.w = 0;
 	ind.type = type;
 	ind.selected_layout = itspec_index;
-	ind.town = ClosestTownFromTile(tile, (uint)-1);
+	ind.town = ClosestTownFromTile(tile, UINT_MAX);
 
 	NewIndustryResolver(&object, tile, &ind, type);
 	object.GetVariable = IndustryLocationGetVariable;
 	object.callback = CBID_INDUSTRY_LOCATION;
 	_industry_creation_random_bits = seed;
 
-	group = Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
+	group = SpriteGroup::Resolve(GetIndustrySpec(type)->grf_prop.spritegroup, &object);
 
 	/* Unlike the "normal" cases, not having a valid result means we allow
 	 * the building of the industry, as that's how it's done in TTDP. */
-	if (group == NULL || group->type != SGT_CALLBACK) return true;
+	if (group == NULL) return true;
+	uint16 result = group->GetCallbackResult();
+	if (result == 0x400 || result == CALLBACK_FAILED) return true;
 
 	/* Copy some parameters from the registers to the error message text ref. stack */
 	SwitchToErrorRefStack();
 	PrepareTextRefStackUsage(4);
 	SwitchToNormalRefStack();
 
-	switch (group->g.callback.result) {
-		case 0x400: return true;
-		case 0x401: _error_message = STR_0239_SITE_UNSUITABLE; break;
-		case 0x402: _error_message = STR_0317_CAN_ONLY_BE_BUILT_IN_RAINFOREST; break;
-		case 0x403: _error_message = STR_0318_CAN_ONLY_BE_BUILT_IN_DESERT; break;
-		default: _error_message = GetGRFStringID(indspec->grf_prop.grffile->grfid, 0xD000 + group->g.callback.result); break;
+	switch (result) {
+		case 0x401: _error_message = STR_ERROR_SITE_UNSUITABLE; break;
+		case 0x402: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_RAINFOREST; break;
+		case 0x403: _error_message = STR_ERROR_CAN_ONLY_BE_BUILT_IN_DESERT; break;
+		default: _error_message = GetGRFStringID(indspec->grf_prop.grffile->grfid, 0xD000 + result); break;
 	}
 
 	return false;
@@ -553,7 +494,7 @@ bool CheckIfCallBackAllowsAvailability(IndustryType type, IndustryAvailabilityCa
 {
 	const IndustrySpec *indspec = GetIndustrySpec(type);
 
-	if (HasBit(indspec->callback_flags, CBM_IND_AVAILABLE)) {
+	if (HasBit(indspec->callback_mask, CBM_IND_AVAILABLE)) {
 		uint16 res = GetIndustryCallback(CBID_INDUSTRY_AVAILABLE, 0, creation_type, NULL, type, INVALID_TILE);
 		if (res != CALLBACK_FAILED) {
 			return (res == 0);
@@ -562,7 +503,7 @@ bool CheckIfCallBackAllowsAvailability(IndustryType type, IndustryAvailabilityCa
 	return true;
 }
 
-static int32 DerefIndProd(uint field, bool use_register)
+static int32 DerefIndProd(int field, bool use_register)
 {
 	return use_register ? (int32)GetRegister(field) : field;
 }
@@ -576,31 +517,44 @@ void IndustryProductionCallback(Industry *ind, int reason)
 {
 	const IndustrySpec *spec = GetIndustrySpec(ind->type);
 	ResolverObject object;
-	NewIndustryResolver(&object, ind->xy, ind, ind->type);
+	NewIndustryResolver(&object, ind->location.tile, ind, ind->type);
 	if ((spec->behaviour & INDUSTRYBEH_PRODCALLBACK_RANDOM) != 0) object.callback_param1 = Random();
 	int multiplier = 1;
 	if ((spec->behaviour & INDUSTRYBEH_PROD_MULTI_HNDLING) != 0) multiplier = ind->prod_level;
 	object.callback_param2 = reason;
 
 	for (uint loop = 0;; loop++) {
-		SB(object.callback_param2, 8, 16, loop);
-		const SpriteGroup *group = Resolve(spec->grf_prop.spritegroup, &object);
-		if (group == NULL || group->type != SGT_INDUSTRY_PRODUCTION) break;
+		/* limit the number of calls to break infinite loops.
+		 * 'loop' is provided as 16 bits to the newgrf, so abort when those are exceeded. */
+		if (loop >= 0x10000) {
+			/* display error message */
+			SetDParamStr(0, spec->grf_prop.grffile->filename);
+			SetDParam(1, spec->name);
+			ShowErrorMessage(STR_NEWGRF_BUGGY, STR_NEWGRF_BUGGY_ENDLESS_PRODUCTION_CALLBACK, 0, 0);
 
-		bool deref = (group->g.indprod.version == 1);
+			/* abort the function early, this error isn't critical and will allow the game to continue to run */
+			break;
+		}
+
+		SB(object.callback_param2, 8, 16, loop);
+		const SpriteGroup *tgroup = SpriteGroup::Resolve(spec->grf_prop.spritegroup, &object);
+		if (tgroup == NULL || tgroup->type != SGT_INDUSTRY_PRODUCTION) break;
+		const IndustryProductionSpriteGroup *group = (const IndustryProductionSpriteGroup *)tgroup;
+
+		bool deref = (group->version == 1);
 
 		for (uint i = 0; i < 3; i++) {
-			ind->incoming_cargo_waiting[i] = Clamp(ind->incoming_cargo_waiting[i] - DerefIndProd(group->g.indprod.substract_input[i], deref) * multiplier, 0, 0xFFFF);
+			ind->incoming_cargo_waiting[i] = Clamp(ind->incoming_cargo_waiting[i] - DerefIndProd(group->subtract_input[i], deref) * multiplier, 0, 0xFFFF);
 		}
 		for (uint i = 0; i < 2; i++) {
-			ind->produced_cargo_waiting[i] = Clamp(ind->produced_cargo_waiting[i] + max(DerefIndProd(group->g.indprod.add_output[i], deref), 0) * multiplier, 0, 0xFFFF);
+			ind->produced_cargo_waiting[i] = Clamp(ind->produced_cargo_waiting[i] + max(DerefIndProd(group->add_output[i], deref), 0) * multiplier, 0, 0xFFFF);
 		}
 
-		int32 again = DerefIndProd(group->g.indprod.again, deref);
+		int32 again = DerefIndProd(group->again, deref);
 		if (again == 0) break;
 
 		SB(object.callback_param2, 24, 8, again);
 	}
 
-	InvalidateWindow(WC_INDUSTRY_VIEW, ind->index);
+	SetWindowDirty(WC_INDUSTRY_VIEW, ind->index);
 }
