@@ -1,10 +1,22 @@
-/* $Id: newgrf_industrytiles.h 11410 2007-11-11 17:56:37Z rubidium $ */
+/* $Id: newgrf_industrytiles.h 18809 2010-01-15 16:41:15Z rubidium $ */
 
-/** @file newgrf_industrytiles.h */
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/** @file newgrf_industrytiles.h NewGRF handling of industry tiles. */
 
 #ifndef NEWGRF_INDUSTRYTILES_H
 #define NEWGRF_INDUSTRYTILES_H
 
+#include "tile_cmd.h"
+#include "industry_type.h"
+#include "newgrf_callbacks.h"
+#include "core/random_func.hpp"
+
 enum IndustryAnimationTrigger {
 	IAT_CONSTRUCTION_STATE_CHANGE,
 	IAT_TILELOOP,
