@@ -1,30 +1,36 @@
-/* $Id: newgrf_station.cpp 13871 2008-07-29 22:37:54Z rubidium $ */
+/* $Id: newgrf_station.cpp 19300 2010-03-02 00:38:01Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file newgrf_station.cpp Functions for dealing with station classes and custom stations. */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "variables.h"
-#include "tile_cmd.h"
-#include "landscape.h"
 #include "debug.h"
-#include "sprite.h"
-#include "station.h"
-#include "station_map.h"
-#include "newgrf.h"
-#include "newgrf_callbacks.h"
+#include "station_base.h"
+#include "waypoint_base.h"
+#include "roadstop_base.h"
+#include "newgrf_cargo.h"
 #include "newgrf_commons.h"
 #include "newgrf_station.h"
 #include "newgrf_spritegroup.h"
-#include "cargotype.h"
+#include "newgrf_sound.h"
+#include "newgrf_railtype.h"
 #include "town.h"
 #include "newgrf_town.h"
-#include "gfx_func.h"
 #include "date_func.h"
-#include "player_func.h"
+#include "company_func.h"
+#include "animated_tile_func.h"
+#include "functions.h"
 #include "tunnelbridge_map.h"
+#include "newgrf.h"
+#include "core/random_func.hpp"
 
-#include "table/sprites.h"
 #include "table/strings.h"
 
 static StationClass _station_classes[STAT_CLASS_MAX];
@@ -33,6 +39,46 @@ enum {
 	MAX_SPECLIST = 255,
 };
 
+enum TriggerArea {
+	TA_TILE,
+	TA_PLATFORM,
+	TA_WHOLE,
+};
+
+struct ETileArea : TileArea {
+	ETileArea(const BaseStation *st, TileIndex tile, TriggerArea ta)
+	{
+		switch (ta) {
+			default: NOT_REACHED();
+
+			case TA_TILE:
+				this->tile = tile;
+				this->w    = 1;
+				this->h    = 1;
+				break;
+
+			case TA_PLATFORM: {
+				TileIndex start, end;
+				Axis axis = GetRailStationAxis(tile);
+				TileIndexDiff delta = TileOffsByDiagDir(AxisToDiagDir(axis));
+
+				for (end = tile; IsRailStationTile(end + delta) && IsCompatibleTrainStationTile(tile, end + delta); end += delta) { /* Nothing */ }
+				for (start = tile; IsRailStationTile(start - delta) && IsCompatibleTrainStationTile(tile, start - delta); start -= delta) { /* Nothing */ }
+
+				this->tile = start;
+				this->w = TileX(end) - TileX(start) + 1;
+				this->h = TileY(end) - TileY(start) + 1;
+				break;
+			}
+
+			case TA_WHOLE:
+				st->GetTileArea(this, Station::IsExpected(st) ? STATION_RAIL : STATION_WAYPOINT);
+				break;
+		}
+	}
+};
+
+
 /**
  * Reset station classes to their default state.
  * This includes initialising the Default and Waypoint classes with an empty
@@ -51,13 +97,13 @@ void ResetStationClasses()
 
 	/* Set up initial data */
 	_station_classes[0].id = 'DFLT';
-	_station_classes[0].name = STR_STAT_CLASS_DFLT;
+	_station_classes[0].name = STR_STATION_CLASS_DFLT;
 	_station_classes[0].stations = 1;
 	_station_classes[0].spec = MallocT<StationSpec*>(1);
 	_station_classes[0].spec[0] = NULL;
 
 	_station_classes[1].id = 'WAYP';
-	_station_classes[1].name = STR_STAT_CLASS_WAYP;
+	_station_classes[1].name = STR_STATION_CLASS_WAYP;
 	_station_classes[1].stations = 1;
 	_station_classes[1].spec = MallocT<StationSpec*>(1);
 	_station_classes[1].spec[0] = NULL;
@@ -160,8 +206,14 @@ const StationSpec *GetCustomStationSpec(StationClassID sclass, uint station)
 	return NULL;
 }
 
-
-const StationSpec *GetCustomStationSpecByGrf(uint32 grfid, byte localidx)
+/**
+ * Retrieve a station spec by GRF location.
+ * @param grfid    GRF ID of station spec.
+ * @param localidx Index within GRF file of station spec.
+ * @param index    Pointer to return the index of the station spec in its station class. If NULL then not used.
+ * @return The station spec.
+ */
+const StationSpec *GetCustomStationSpecByGrf(uint32 grfid, byte localidx, int *index)
 {
 	uint j;
 
@@ -169,7 +221,10 @@ const StationSpec *GetCustomStationSpecByGrf(uint32 grfid, byte localidx)
 		for (j = 0; j < _station_classes[i].stations; j++) {
 			const StationSpec *statspec = _station_classes[i].spec[j];
 			if (statspec == NULL) continue;
-			if (statspec->grffile->grfid == grfid && statspec->localidx == localidx) return statspec;
+			if (statspec->grffile->grfid == grfid && statspec->localidx == localidx) {
+				if (index != NULL) *index = j;
+				return statspec;
+			}
 		}
 	}
 
@@ -223,31 +278,20 @@ uint32 GetPlatformInfo(Axis axis, byte tile, int platforms, int length, int x, i
  */
 static TileIndex FindRailStationEnd(TileIndex tile, TileIndexDiff delta, bool check_type, bool check_axis)
 {
-	bool waypoint;
 	byte orig_type = 0;
 	Axis orig_axis = AXIS_X;
+	StationID sid = GetStationIndex(tile);
 
-	waypoint = IsTileType(tile, MP_RAILWAY);
-
-	if (waypoint) {
-		if (check_axis) orig_axis = GetWaypointAxis(tile);
-	} else {
-		if (check_type) orig_type = GetCustomStationSpecIndex(tile);
-		if (check_axis) orig_axis = GetRailStationAxis(tile);
-	}
+	if (check_type) orig_type = GetCustomStationSpecIndex(tile);
+	if (check_axis) orig_axis = GetRailStationAxis(tile);
 
 	while (true) {
 		TileIndex new_tile = TILE_ADD(tile, delta);
 
-		if (waypoint) {
-			if (!IsTileType(new_tile, MP_RAILWAY)) break;
-			if (!IsRailWaypoint(new_tile)) break;
-			if (check_axis && GetWaypointAxis(new_tile) != orig_axis) break;
-		} else {
-			if (!IsRailwayStationTile(new_tile)) break;
-			if (check_type && GetCustomStationSpecIndex(new_tile) != orig_type) break;
-			if (check_axis && GetRailStationAxis(new_tile) != orig_axis) break;
-		}
+		if (!IsTileType(new_tile, MP_STATION) || GetStationIndex(new_tile) != sid) break;
+		if (!HasStationRail(new_tile)) break;
+		if (check_type && GetCustomStationSpecIndex(new_tile) != orig_type) break;
+		if (check_axis && GetRailStationAxis(new_tile) != orig_axis) break;
 
 		tile = new_tile;
 	}
@@ -263,12 +307,11 @@ static uint32 GetPlatformInfoHelper(TileIndex tile, bool check_type, bool check_
 	int sy = TileY(FindRailStationEnd(tile, TileDiffXY( 0, -1), check_type, check_axis));
 	int ex = TileX(FindRailStationEnd(tile, TileDiffXY( 1,  0), check_type, check_axis)) + 1;
 	int ey = TileY(FindRailStationEnd(tile, TileDiffXY( 0,  1), check_type, check_axis)) + 1;
-	Axis axis = IsTileType(tile, MP_RAILWAY) ? GetWaypointAxis(tile) : GetRailStationAxis(tile);
 
 	tx -= sx; ex -= sx;
 	ty -= sy; ey -= sy;
 
-	return GetPlatformInfo(axis, IsTileType(tile, MP_RAILWAY) ? 2 : GetStationGfx(tile), ex, ey, tx, ty, centred);
+	return GetPlatformInfo(GetRailStationAxis(tile), GetStationGfx(tile), ex, ey, tx, ty, centred);
 }
 
 
@@ -282,7 +325,7 @@ static uint32 GetRailContinuationInfo(TileIndex tile)
 	static const Direction y_dir[8] = { DIR_SE, DIR_NW, DIR_SW, DIR_NE, DIR_S, DIR_W, DIR_E, DIR_N };
 	static const DiagDirection y_exits[8] = { DIAGDIR_SE, DIAGDIR_NW, DIAGDIR_SW, DIAGDIR_NE, DIAGDIR_SE, DIAGDIR_NW, DIAGDIR_SE, DIAGDIR_NW };
 
-	Axis axis = IsTileType(tile, MP_RAILWAY) ? GetWaypointAxis(tile) : GetRailStationAxis(tile);
+	Axis axis = GetRailStationAxis(tile);
 
 	/* Choose appropriate lookup table to use */
 	const Direction *dir = axis == AXIS_X ? x_dir : y_dir;
@@ -316,7 +359,7 @@ static uint32 GetRailContinuationInfo(TileIndex tile)
 /* Station Resolver Functions */
 static uint32 StationGetRandomBits(const ResolverObject *object)
 {
-	const Station *st = object->u.station.st;
+	const BaseStation *st = object->u.station.st;
 	const TileIndex tile = object->u.station.tile;
 	return (st == NULL ? 0 : st->random_bits) | (tile == INVALID_TILE ? 0 : GetStationTileRandomBits(tile) << 16);
 }
@@ -324,14 +367,14 @@ static uint32 StationGetRandomBits(const ResolverObject *object)
 
 static uint32 StationGetTriggers(const ResolverObject *object)
 {
-	const Station *st = object->u.station.st;
+	const BaseStation *st = object->u.station.st;
 	return st == NULL ? 0 : st->waiting_triggers;
 }
 
 
 static void StationSetTriggers(const ResolverObject *object, int triggers)
 {
-	Station *st = (Station*)object->u.station.st;
+	BaseStation *st = const_cast<BaseStation *>(object->u.station.st);
 	assert(st != NULL);
 	st->waiting_triggers = triggers;
 }
@@ -353,7 +396,7 @@ static struct {
 
 static uint32 StationGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
 {
-	const Station *st = object->u.station.st;
+	const BaseStation *st = object->u.station.st;
 	TileIndex tile = object->u.station.tile;
 
 	if (object->scope == VSG_SCOPE_PARENT) {
@@ -379,10 +422,10 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 			case 0x41:
 			case 0x46:
 			case 0x47:
-			case 0x49: return 0x2110000;       // Platforms, tracks & position
-			case 0x42: return 0;               // Rail type (XXX Get current type from GUI?)
-			case 0x43: return _current_player; // Station owner
-			case 0x44: return 2;               // PBS status
+			case 0x49: return 0x2110000;        // Platforms, tracks & position
+			case 0x42: return 0;                // Rail type (XXX Get current type from GUI?)
+			case 0x43: return _current_company; // Station owner
+			case 0x44: return 2;                // PBS status
 			case 0xFA: return Clamp(_date - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 65535); // Build date, clamped to a 16 bit value
 		}
 
@@ -400,9 +443,9 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 			if (!HasBit(_svc.valid, 1)) { _svc.v41 = GetPlatformInfoHelper(tile, true,  false, false); SetBit(_svc.valid, 1); }
 			return _svc.v41;
 
-		case 0x42: return GetTerrainType(tile) | (GetRailType(tile) << 8);
+		case 0x42: return GetTerrainType(tile) | (GetReverseRailTypeTranslation(GetRailType(tile), object->u.station.statspec->grffile) << 8);
 		case 0x43: return st->owner; // Station owner
-		case 0x44: return 2;         // PBS status
+		case 0x44: return HasStationReservation(tile) ? 7 : 4; // PBS status
 		case 0x45:
 			if (!HasBit(_svc.valid, 2)) { _svc.v45 = GetRailContinuationInfo(tile); SetBit(_svc.valid, 2); }
 			return _svc.v45;
@@ -415,28 +458,26 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 			if (!HasBit(_svc.valid, 4)) { _svc.v47 = GetPlatformInfoHelper(tile, true,  false, true); SetBit(_svc.valid, 4); }
 			return _svc.v47;
 
-		case 0x48: { // Accepted cargo types
-			CargoID cargo_type;
-			uint32 value = 0;
-
-			for (cargo_type = 0; cargo_type < NUM_CARGO; cargo_type++) {
-				if (HasBit(st->goods[cargo_type].acceptance_pickup, GoodsEntry::PICKUP)) SetBit(value, cargo_type);
-			}
-			return value;
-		}
 		case 0x49:
 			if (!HasBit(_svc.valid, 5)) { _svc.v49 = GetPlatformInfoHelper(tile, false, true, false); SetBit(_svc.valid, 5); }
 			return _svc.v49;
 
+		case 0x4A: // Animation frame of tile
+			return GetStationAnimationFrame(tile);
+
 		/* Variables which use the parameter */
 		/* Variables 0x60 to 0x65 are handled separately below */
-		case 0x67: { // Land info of nearby tiles
+		case 0x66: // Animation frame of nearby tile
+			if (parameter != 0) tile = GetNearbyTile(parameter, tile);
+			return st->TileBelongsToRailStation(tile) ? GetStationAnimationFrame(tile) : UINT_MAX;
+
+		case 0x67: { // Land info of nearby tile
 			Axis axis = GetRailStationAxis(tile);
 
 			if (parameter != 0) tile = GetNearbyTile(parameter, tile); // only perform if it is required
 
 			Slope tileh = GetTileSlope(tile, NULL);
-			bool swap = (axis == AXIS_Y && HasBit(tileh, SLOPE_W) != HasBit(tileh, SLOPE_E));
+			bool swap = (axis == AXIS_Y && HasBit(tileh, CORNER_W) != HasBit(tileh, CORNER_E));
 
 			return GetNearbyTileInformation(tile) ^ (swap ? SLOPE_EW : 0);
 		}
@@ -444,7 +485,7 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 		case 0x68: { // Station info of nearby tiles
 			TileIndex nearby_tile = GetNearbyTile(parameter, tile);
 
-			if (!IsRailwayStationTile(nearby_tile)) return 0xFFFFFFFF;
+			if (!HasStationTileRail(nearby_tile)) return 0xFFFFFFFF;
 
 			uint32 grfid = st->speclist[GetCustomStationSpecIndex(tile)].grfid;
 			bool perpendicular = GetRailStationAxis(tile) != GetRailStationAxis(nearby_tile);
@@ -452,32 +493,52 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 			uint32 res = GB(GetStationGfx(nearby_tile), 1, 2) << 12 | !!perpendicular << 11 | !!same_station << 10;
 
 			if (IsCustomStationSpecIndex(nearby_tile)) {
-				const StationSpecList ssl = GetStationByTile(nearby_tile)->speclist[GetCustomStationSpecIndex(nearby_tile)];
+				const StationSpecList ssl = BaseStation::GetByTile(nearby_tile)->speclist[GetCustomStationSpecIndex(nearby_tile)];
 				res |= 1 << (ssl.grfid != grfid ? 9 : 8) | ssl.localidx;
 			}
 			return res;
 		}
 
-		/* General station properties */
+		/* General station variables */
 		case 0x82: return 50;
 		case 0x84: return st->string_id;
 		case 0x86: return 0;
-		case 0x8A: return st->had_vehicle_of_type;
 		case 0xF0: return st->facilities;
-		case 0xF1: return st->airport_type;
-		case 0xF2: return st->truck_stops->status;
-		case 0xF3: return st->bus_stops->status;
-		case 0xF6: return st->airport_flags;
-		case 0xF7: return GB(st->airport_flags, 8, 8);
 		case 0xFA: return Clamp(st->build_date - DAYS_TILL_ORIGINAL_BASE_YEAR, 0, 65535);
 	}
 
+	return st->GetNewGRFVariable(object, variable, parameter, available);
+}
+
+byte MapAirportTypeToTTDType(byte ottd_type);
+
+uint32 Station::GetNewGRFVariable(const ResolverObject *object, byte variable, byte parameter, bool *available) const
+{
+	switch (variable) {
+		case 0x48: { // Accepted cargo types
+			CargoID cargo_type;
+			uint32 value = 0;
+
+			for (cargo_type = 0; cargo_type < NUM_CARGO; cargo_type++) {
+				if (HasBit(this->goods[cargo_type].acceptance_pickup, GoodsEntry::PICKUP)) SetBit(value, cargo_type);
+			}
+			return value;
+		}
+
+		case 0x8A: return this->had_vehicle_of_type;
+		case 0xF1: return MapAirportTypeToTTDType(this->airport_type);
+		case 0xF2: return (this->truck_stops != NULL) ? this->truck_stops->status : 0;
+		case 0xF3: return (this->bus_stops != NULL)   ? this->bus_stops->status   : 0;
+		case 0xF6: return this->airport_flags;
+		case 0xF7: return GB(this->airport_flags, 8, 8);
+	}
+
 	/* Handle cargo variables with parameter, 0x60 to 0x65 */
 	if (variable >= 0x60 && variable <= 0x65) {
 		CargoID c = GetCargoTranslation(parameter, object->u.station.statspec->grffile);
 
 		if (c == CT_INVALID) return 0;
-		const GoodsEntry *ge = &st->goods[c];
+		const GoodsEntry *ge = &this->goods[c];
 
 		switch (variable) {
 			case 0x60: return min(ge->cargo.Count(), 4095);
@@ -491,7 +552,7 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 
 	/* Handle cargo variables (deprecated) */
 	if (variable >= 0x8C && variable <= 0xEC) {
-		const GoodsEntry *g = &st->goods[GB(variable - 0x8C, 3, 4)];
+		const GoodsEntry *g = &this->goods[GB(variable - 0x8C, 3, 4)];
 		switch (GB(variable - 0x8C, 0, 3)) {
 			case 0: return g->cargo.Count();
 			case 1: return GB(min(g->cargo.Count(), 4095), 0, 4) | (GB(g->acceptance_pickup, GoodsEntry::ACCEPTANCE, 1) << 7);
@@ -504,26 +565,59 @@ static uint32 StationGetVariable(const ResolverObject *object, byte variable, by
 		}
 	}
 
-	DEBUG(grf, 1, "Unhandled station property 0x%X", variable);
+	DEBUG(grf, 1, "Unhandled station variable 0x%X", variable);
 
 	*available = false;
 	return UINT_MAX;
 }
 
-
-static const SpriteGroup *StationResolveReal(const ResolverObject *object, const SpriteGroup *group)
+uint32 Waypoint::GetNewGRFVariable(const ResolverObject *object, byte variable, byte parameter, bool *available) const
 {
-	const Station *st = object->u.station.st;
+	switch (variable) {
+		case 0x48: return 0; // Accepted cargo types
+		case 0x8A: return HVOT_WAYPOINT;
+		case 0xF1: return 0; // airport type
+		case 0xF2: return 0; // truck stop status
+		case 0xF3: return 0; // bus stop status
+		case 0xF6: return 0; // airport flags
+		case 0xF7: return 0; // airport flags cont.
+	}
+
+	/* Handle cargo variables with parameter, 0x60 to 0x65 */
+	if (variable >= 0x60 && variable <= 0x65) {
+		return 0;
+	}
+
+	/* Handle cargo variables (deprecated) */
+	if (variable >= 0x8C && variable <= 0xEC) {
+		switch (GB(variable - 0x8C, 0, 3)) {
+			case 3: return INITIAL_STATION_RATING;
+			case 4: return INVALID_STATION;
+			default: return 0;
+		}
+	}
+
+	DEBUG(grf, 1, "Unhandled station variable 0x%X", variable);
+
+	*available = false;
+	return UINT_MAX;
+}
+
+static const SpriteGroup *StationResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
+{
+	const BaseStation *bst = object->u.station.st;
 	const StationSpec *statspec = object->u.station.statspec;
 	uint set;
 
 	uint cargo = 0;
 	CargoID cargo_type = object->u.station.cargo_type;
 
-	if (st == NULL || statspec->sclass == STAT_CLASS_WAYP) {
-		return group->g.real.loading[0];
+	if (bst == NULL || statspec->sclass == STAT_CLASS_WAYP) {
+		return group->loading[0];
 	}
 
+	const Station *st = Station::From(bst);
+
 	switch (cargo_type) {
 		case CT_INVALID:
 		case CT_DEFAULT_NA:
@@ -542,26 +636,26 @@ static const SpriteGroup *StationResolveReal(const ResolverObject *object, const
 			break;
 	}
 
-	if (HasBit(statspec->flags, 1)) cargo /= (st->trainst_w + st->trainst_h);
+	if (HasBit(statspec->flags, SSF_DIV_BY_STATION_SIZE)) cargo /= (st->train_station.w + st->train_station.h);
 	cargo = min(0xfff, cargo);
 
 	if (cargo > statspec->cargo_threshold) {
-		if (group->g.real.num_loading > 0) {
-			set = ((cargo - statspec->cargo_threshold) * group->g.real.num_loading) / (4096 - statspec->cargo_threshold);
-			return group->g.real.loading[set];
+		if (group->num_loading > 0) {
+			set = ((cargo - statspec->cargo_threshold) * group->num_loading) / (4096 - statspec->cargo_threshold);
+			return group->loading[set];
 		}
 	} else {
-		if (group->g.real.num_loaded > 0) {
-			set = (cargo * group->g.real.num_loaded) / (statspec->cargo_threshold + 1);
-			return group->g.real.loaded[set];
+		if (group->num_loaded > 0) {
+			set = (cargo * group->num_loaded) / (statspec->cargo_threshold + 1);
+			return group->loaded[set];
 		}
 	}
 
-	return group->g.real.loading[0];
+	return group->loading[0];
 }
 
 
-static void NewStationResolver(ResolverObject *res, const StationSpec *statspec, const Station *st, TileIndex tile)
+static void NewStationResolver(ResolverObject *res, const StationSpec *statspec, const BaseStation *st, TileIndex tile)
 {
 	res->GetRandomBits = StationGetRandomBits;
 	res->GetTriggers   = StationGetTriggers;
@@ -579,6 +673,8 @@ static void NewStationResolver(ResolverObject *res, const StationSpec *statspec,
 	res->last_value      = 0;
 	res->trigger         = 0;
 	res->reseed          = 0;
+	res->count           = 0;
+	res->grffile         = (statspec != NULL ? statspec->grffile : NULL);
 }
 
 static const SpriteGroup *ResolveStation(ResolverObject *object)
@@ -589,13 +685,14 @@ static const SpriteGroup *ResolveStation(ResolverObject *object)
 	if (object->u.station.st == NULL) {
 		/* No station, so we are in a purchase list */
 		ctype = CT_PURCHASE;
-	} else {
+	} else if (Station::IsExpected(object->u.station.st)) {
+		const Station *st = Station::From(object->u.station.st);
 		/* Pick the first cargo that we have waiting */
-		for (CargoID cargo = 0; cargo < NUM_CARGO; cargo++) {
-			const CargoSpec *cs = GetCargo(cargo);
-			if (cs->IsValid() && object->u.station.statspec->spritegroup[cargo] != NULL &&
-					!object->u.station.st->goods[cargo].cargo.Empty()) {
-				ctype = cargo;
+		const CargoSpec *cs;
+		FOR_ALL_CARGOSPECS(cs) {
+			if (object->u.station.statspec->spritegroup[cs->Index()] != NULL &&
+					!st->goods[cs->Index()].cargo.Empty()) {
+				ctype = cs->Index();
 				break;
 			}
 		}
@@ -615,10 +712,10 @@ static const SpriteGroup *ResolveStation(ResolverObject *object)
 	/* Invalidate all cached vars */
 	_svc.valid = 0;
 
-	return Resolve(group, object);
+	return SpriteGroup::Resolve(group, object);
 }
 
-SpriteID GetCustomStationRelocation(const StationSpec *statspec, const Station *st, TileIndex tile)
+SpriteID GetCustomStationRelocation(const StationSpec *statspec, const BaseStation *st, TileIndex tile)
 {
 	const SpriteGroup *group;
 	ResolverObject object;
@@ -627,25 +724,41 @@ SpriteID GetCustomStationRelocation(const StationSpec *statspec, const Station *
 
 	group = ResolveStation(&object);
 	if (group == NULL || group->type != SGT_RESULT) return 0;
-	return group->g.result.sprite - 0x42D;
+	return group->GetResult() - 0x42D;
 }
 
 
-SpriteID GetCustomStationGroundRelocation(const StationSpec *statspec, const Station *st, TileIndex tile)
+SpriteID GetCustomStationGroundRelocation(const StationSpec *statspec, const BaseStation *st, TileIndex tile)
 {
 	const SpriteGroup *group;
 	ResolverObject object;
 
 	NewStationResolver(&object, statspec, st, tile);
-	object.callback_param1 = 1; // Indicate we are resolving the ground sprite
+	if (HasBit(statspec->flags, SSF_SEPARATE_GROUND)) {
+		object.callback_param1 = 1; // Indicate we are resolving the ground sprite
+	}
 
 	group = ResolveStation(&object);
 	if (group == NULL || group->type != SGT_RESULT) return 0;
-	return group->g.result.sprite - 0x42D;
+	return group->GetResult() - 0x42D;
 }
 
 
-uint16 GetStationCallback(CallbackID callback, uint32 param1, uint32 param2, const StationSpec *statspec, const Station *st, TileIndex tile)
+SpriteID GetCustomStationFoundationRelocation(const StationSpec *statspec, const BaseStation *st, TileIndex tile)
+{
+	const SpriteGroup *group;
+	ResolverObject object;
+
+	NewStationResolver(&object, statspec, st, tile);
+	object.callback_param1 = 2; // Indicate we are resolving the foundation sprites
+
+	group = ResolveStation(&object);
+	if (group == NULL || group->type != SGT_RESULT) return 0;
+	return group->GetResult() + GetRegister(0x100);
+}
+
+
+uint16 GetStationCallback(CallbackID callback, uint32 param1, uint32 param2, const StationSpec *statspec, const BaseStation *st, TileIndex tile)
 {
 	const SpriteGroup *group;
 	ResolverObject object;
@@ -657,8 +770,8 @@ uint16 GetStationCallback(CallbackID callback, uint32 param1, uint32 param2, con
 	object.callback_param2 = param2;
 
 	group = ResolveStation(&object);
-	if (group == NULL || group->type != SGT_CALLBACK) return CALLBACK_FAILED;
-	return group->g.callback.result;
+	if (group == NULL) return CALLBACK_FAILED;
+	return group->GetCallbackResult();
 }
 
 
@@ -669,22 +782,28 @@ uint16 GetStationCallback(CallbackID callback, uint32 param1, uint32 param2, con
  * @param exec Whether to actually allocate the spec.
  * @return Index within the Station's spec list, or -1 if the allocation failed.
  */
-int AllocateSpecToStation(const StationSpec *statspec, Station *st, bool exec)
+int AllocateSpecToStation(const StationSpec *statspec, BaseStation *st, bool exec)
 {
 	uint i;
 
 	if (statspec == NULL || st == NULL) return 0;
 
-	/* Check if this spec has already been allocated */
-	for (i = 1; i < st->num_specs && i < MAX_SPECLIST; i++) {
-		if (st->speclist[i].spec == statspec) return i;
-	}
-
 	for (i = 1; i < st->num_specs && i < MAX_SPECLIST; i++) {
 		if (st->speclist[i].spec == NULL && st->speclist[i].grfid == 0) break;
 	}
 
-	if (i == MAX_SPECLIST) return -1;
+	if (i == MAX_SPECLIST) {
+		/* As final effort when the spec list is already full...
+		 * try to find the same spec and return that one. This might
+		 * result in slighty "wrong" (as per specs) looking stations,
+		 * but it's fairly unlikely that one reaches the limit anyways.
+		 */
+		for (i = 1; i < st->num_specs && i < MAX_SPECLIST; i++) {
+			if (st->speclist[i].spec == statspec) return i;
+		}
+
+		return -1;
+	}
 
 	if (exec) {
 		if (i >= st->num_specs) {
@@ -713,17 +832,18 @@ int AllocateSpecToStation(const StationSpec *statspec, Station *st, bool exec)
  * @param specindex Index of the custom station within the Station's spec list.
  * @return Indicates whether the StationSpec was deallocated.
  */
-void DeallocateSpecFromStation(Station* st, byte specindex)
+void DeallocateSpecFromStation(BaseStation *st, byte specindex)
 {
 	/* specindex of 0 (default) is never freeable */
 	if (specindex == 0) return;
 
+	ETileArea area = ETileArea(st, INVALID_TILE, TA_WHOLE);
 	/* Check all tiles over the station to check if the specindex is still in use */
-	BEGIN_TILE_LOOP(tile, st->trainst_w, st->trainst_h, st->train_tile) {
-		if (IsTileType(tile, MP_STATION) && GetStationIndex(tile) == st->index && IsRailwayStation(tile) && GetCustomStationSpecIndex(tile) == specindex) {
+	TILE_AREA_LOOP(tile, area) {
+		if (st->TileBelongsToRailStation(tile) && GetCustomStationSpecIndex(tile) == specindex) {
 			return;
 		}
-	} END_TILE_LOOP(tile, st->trainst_w, st->trainst_h, st->train_tile)
+	}
 
 	/* This specindex is no longer in use, so deallocate it */
 	st->speclist[specindex].spec     = NULL;
@@ -740,8 +860,12 @@ void DeallocateSpecFromStation(Station* st, byte specindex)
 			free(st->speclist);
 			st->num_specs = 0;
 			st->speclist  = NULL;
+			st->cached_anim_triggers = 0;
+			return;
 		}
 	}
+
+	StationUpdateAnimTriggers(st);
 }
 
 /** Draw representation of a station tile for GUI purposes.
@@ -757,19 +881,16 @@ bool DrawStationTile(int x, int y, RailType railtype, Axis axis, StationClassID
 {
 	const StationSpec *statspec;
 	const DrawTileSprites *sprites;
-	const DrawTileSeqStruct *seq;
 	const RailtypeInfo *rti = GetRailTypeInfo(railtype);
-	SpriteID relocation;
-	SpriteID image;
-	SpriteID palette = PLAYER_SPRITE_COLOR(_local_player);
+	PaletteID palette = COMPANY_SPRITE_COLOUR(_local_company);
 	uint tile = 2;
 
 	statspec = GetCustomStationSpec(sclass, station);
 	if (statspec == NULL) return false;
 
-	relocation = GetCustomStationRelocation(statspec, NULL, INVALID_TILE);
+	uint relocation = GetCustomStationRelocation(statspec, NULL, INVALID_TILE);
 
-	if (HasBit(statspec->callbackmask, CBM_STATION_SPRITE_LAYOUT)) {
+	if (HasBit(statspec->callback_mask, CBM_STATION_SPRITE_LAYOUT)) {
 		uint16 callback = GetStationCallback(CBID_STATION_SPRITE_LAYOUT, 0x2110000, 0, statspec, NULL, INVALID_TILE);
 		if (callback != CALLBACK_FAILED) tile = callback;
 	}
@@ -780,55 +901,29 @@ bool DrawStationTile(int x, int y, RailType railtype, Axis axis, StationClassID
 		sprites = &statspec->renderdata[(tile < statspec->tiles) ? tile + axis : (uint)axis];
 	}
 
-	image = sprites->ground.sprite;
-	if (HasBit(image, SPRITE_MODIFIER_USE_OFFSET)) {
+	SpriteID image = sprites->ground.sprite;
+	PaletteID pal = sprites->ground.pal;
+	if (HasBit(image, SPRITE_MODIFIER_CUSTOM_SPRITE)) {
 		image += GetCustomStationGroundRelocation(statspec, NULL, INVALID_TILE);
 		image += rti->custom_ground_offset;
 	} else {
 		image += rti->total_offset;
 	}
 
-	DrawSprite(image, PAL_NONE, x, y);
+	DrawSprite(image, GroundSpritePaletteTransform(image, pal, palette), x, y);
 
-	foreach_draw_tile_seq(seq, sprites->seq) {
-		Point pt;
-		image = seq->image.sprite;
-		if (HasBit(image, SPRITE_MODIFIER_USE_OFFSET)) {
-			image += rti->total_offset;
-		} else {
-			image += relocation;
-		}
-
-		SpriteID pal;
-		if (HasBit(image, PALETTE_MODIFIER_TRANSPARENT) || HasBit(image, PALETTE_MODIFIER_COLOR)) {
-			if (seq->image.pal > 0) {
-				pal = seq->image.pal;
-			} else {
-				pal = palette;
-			}
-		} else {
-			pal = PAL_NONE;
-		}
-
-		if ((byte)seq->delta_z != 0x80) {
-			pt = RemapCoords(seq->delta_x, seq->delta_y, seq->delta_z);
-			DrawSprite(image, pal, x + pt.x, y + pt.y);
-		}
-	}
+	DrawRailTileSeqInGUI(x, y, sprites, rti->total_offset, relocation, palette);
 
 	return true;
 }
 
 
-static const StationSpec* GetStationSpec(TileIndex t)
+const StationSpec *GetStationSpec(TileIndex t)
 {
-	const Station* st;
-	uint specindex;
-
 	if (!IsCustomStationSpecIndex(t)) return NULL;
 
-	st = GetStationByTile(t);
-	specindex = GetCustomStationSpecIndex(t);
+	const BaseStation *st = BaseStation::GetByTile(t);
+	uint specindex = GetCustomStationSpecIndex(t);
 	return specindex < st->num_specs ? st->speclist[specindex].spec : NULL;
 }
 
@@ -837,7 +932,7 @@ static const StationSpec* GetStationSpec(TileIndex t)
  * XXX This could be cached (during build) in the map array to save on all the dereferencing */
 bool IsStationTileBlocked(TileIndex tile)
 {
-	const StationSpec* statspec = GetStationSpec(tile);
+	const StationSpec *statspec = GetStationSpec(tile);
 
 	return statspec != NULL && HasBit(statspec->blocked, GetStationGfx(tile));
 }
@@ -846,10 +941,145 @@ bool IsStationTileBlocked(TileIndex tile)
  * XXX This could be cached (during build) in the map array to save on all the dereferencing */
 bool IsStationTileElectrifiable(TileIndex tile)
 {
-	const StationSpec* statspec = GetStationSpec(tile);
+	const StationSpec *statspec = GetStationSpec(tile);
 
 	return
 		statspec == NULL ||
 		HasBit(statspec->pylons, GetStationGfx(tile)) ||
 		!HasBit(statspec->wires, GetStationGfx(tile));
 }
+
+void AnimateStationTile(TileIndex tile)
+{
+	const StationSpec *ss = GetStationSpec(tile);
+	if (ss == NULL) return;
+
+	const BaseStation *st = BaseStation::GetByTile(tile);
+
+	uint8 animation_speed = ss->anim_speed;
+
+	if (HasBit(ss->callback_mask, CBM_STATION_ANIMATION_SPEED)) {
+		uint16 callback = GetStationCallback(CBID_STATION_ANIMATION_SPEED, 0, 0, ss, st, tile);
+		if (callback != CALLBACK_FAILED) animation_speed = Clamp(callback & 0xFF, 0, 16);
+	}
+
+	if (_tick_counter % (1 << animation_speed) != 0) return;
+
+	uint8 frame      = GetStationAnimationFrame(tile);
+	uint8 num_frames = ss->anim_frames;
+
+	bool frame_set_by_callback = false;
+
+	if (HasBit(ss->callback_mask, CBM_STATION_ANIMATION_NEXT_FRAME)) {
+		uint32 param = HasBit(ss->flags, SSF_CB141_RANDOM_BITS) ? Random() : 0;
+		uint16 callback = GetStationCallback(CBID_STATION_ANIM_NEXT_FRAME, param, 0, ss, st, tile);
+
+		if (callback != CALLBACK_FAILED) {
+			frame_set_by_callback = true;
+
+			switch (callback & 0xFF) {
+				case 0xFF:
+					DeleteAnimatedTile(tile);
+					break;
+
+				case 0xFE:
+					frame_set_by_callback = false;
+					break;
+
+				default:
+					frame = callback & 0xFF;
+					break;
+			}
+
+			/* If the lower 7 bits of the upper byte of the callback
+			 * result are not empty, it is a sound effect. */
+			if (GB(callback, 8, 7) != 0) PlayTileSound(ss->grffile, GB(callback, 8, 7), tile);
+		}
+	}
+
+	if (!frame_set_by_callback) {
+		if (frame < num_frames) {
+			frame++;
+		} else if (frame == num_frames && HasBit(ss->anim_status, 0)) {
+			/* This animation loops, so start again from the beginning */
+			frame = 0;
+		} else {
+			/* This animation doesn't loop, so stay here */
+			DeleteAnimatedTile(tile);
+		}
+	}
+
+	SetStationAnimationFrame(tile, frame);
+	MarkTileDirtyByTile(tile);
+}
+
+
+static void ChangeStationAnimationFrame(const StationSpec *ss, const BaseStation *st, TileIndex tile, uint16 random_bits, StatAnimTrigger trigger, CargoID cargo_type)
+{
+	uint16 callback = GetStationCallback(CBID_STATION_ANIM_START_STOP, (random_bits << 16) | Random(), (uint8)trigger | (cargo_type << 8), ss, st, tile);
+	if (callback == CALLBACK_FAILED) return;
+
+	switch (callback & 0xFF) {
+		case 0xFD: /* Do nothing. */         break;
+		case 0xFE: AddAnimatedTile(tile);    break;
+		case 0xFF: DeleteAnimatedTile(tile); break;
+		default:
+			SetStationAnimationFrame(tile, callback);
+			AddAnimatedTile(tile);
+			break;
+	}
+
+	/* If the lower 7 bits of the upper byte of the callback
+	 * result are not empty, it is a sound effect. */
+	if (GB(callback, 8, 7) != 0) PlayTileSound(ss->grffile, GB(callback, 8, 7), tile);
+}
+
+void StationAnimationTrigger(const BaseStation *st, TileIndex tile, StatAnimTrigger trigger, CargoID cargo_type)
+{
+	/* List of coverage areas for each animation trigger */
+	static const TriggerArea tas[] = {
+		TA_TILE, TA_WHOLE, TA_WHOLE, TA_PLATFORM, TA_PLATFORM, TA_PLATFORM, TA_WHOLE
+	};
+
+	/* Get Station if it wasn't supplied */
+	if (st == NULL) st = BaseStation::GetByTile(tile);
+
+	/* Check the cached animation trigger bitmask to see if we need
+	 * to bother with any further processing. */
+	if (!HasBit(st->cached_anim_triggers, trigger)) return;
+
+	uint16 random_bits = Random();
+	ETileArea area = ETileArea(st, tile, tas[trigger]);
+
+	/* Check all tiles over the station to check if the specindex is still in use */
+	TILE_AREA_LOOP(tile, area) {
+		if (st->TileBelongsToRailStation(tile)) {
+			const StationSpec *ss = GetStationSpec(tile);
+			if (ss != NULL && HasBit(ss->anim_triggers, trigger)) {
+				CargoID cargo;
+				if (cargo_type == CT_INVALID) {
+					cargo = CT_INVALID;
+				} else {
+					cargo = GetReverseCargoTranslation(cargo_type, ss->grffile);
+				}
+				ChangeStationAnimationFrame(ss, st, tile, random_bits, trigger, cargo);
+			}
+		}
+	}
+}
+
+/**
+ * Update the cached animation trigger bitmask for a station.
+ * @param st Station to update.
+ */
+void StationUpdateAnimTriggers(BaseStation *st)
+{
+	st->cached_anim_triggers = 0;
+
+	/* Combine animation trigger bitmask for all station specs
+	 * of this station. */
+	for (uint i = 0; i < st->num_specs; i++) {
+		const StationSpec *ss = st->speclist[i].spec;
+		if (ss != NULL) st->cached_anim_triggers |= ss->anim_triggers;
+	}
+}
