@@ -1,4 +1,11 @@
-/* $Id: newgrf_text.cpp 14267 2008-09-07 21:41:47Z rubidium $ */
+/* $Id: newgrf_text.cpp 20092 2010-07-08 19:44:00Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file newgrf_text.cpp
  * Implementation of  Action 04 "universal holder" structure and functions.
@@ -10,15 +17,12 @@
  */
 
 #include "stdafx.h"
-#include "debug.h"
-#include "openttd.h"
-#include "variables.h"
 #include "newgrf.h"
-#include "newgrf_text.h"
 #include "strings_func.h"
-#include "core/alloc_func.hpp"
 #include "newgrf_storage.h"
 #include "string_func.h"
+#include "date_type.h"
+#include "debug.h"
 
 #include "table/strings.h"
 #include "table/control_codes.h"
@@ -27,11 +31,71 @@
 #define TABSIZE 11
 
 /**
- * Explains the newgrf shift bit positionning.
+ * Perform a mapping from TTDPatch's string IDs to OpenTTD's
+ * string IDs, but only for the ones we are aware off; the rest
+ * like likely unused and will show a warning.
+ * @param str the string ID to convert
+ * @return the converted string ID
+ */
+StringID TTDPStringIDToOTTDStringIDMapping(StringID str)
+{
+	/* StringID table for TextIDs 0x4E->0x6D */
+	static const StringID units_volume[] = {
+		STR_NOTHING,    STR_PASSENGERS, STR_TONS,       STR_BAGS,
+		STR_LITERS,     STR_ITEMS,      STR_CRATES,     STR_TONS,
+		STR_TONS,       STR_TONS,       STR_TONS,       STR_BAGS,
+		STR_TONS,       STR_TONS,       STR_TONS,       STR_BAGS,
+		STR_TONS,       STR_TONS,       STR_BAGS,       STR_LITERS,
+		STR_TONS,       STR_LITERS,     STR_TONS,       STR_NOTHING,
+		STR_BAGS,       STR_LITERS,     STR_TONS,       STR_NOTHING,
+		STR_TONS,       STR_NOTHING,    STR_LITERS,     STR_NOTHING
+	};
+
+	/* A string straight from a NewGRF; no need to remap this as it's already mapped. */
+	if (IsInsideMM(str, 0xD000, 0xD7FF) || IsInsideMM(str, 0xDC00, 0xDCFF)) return str;
+
+#define TEXTID_TO_STRINGID(begin, end, stringid) if (str >= begin && str <= end) return str + (stringid - begin)
+	/* We have some changes in our cargo strings, resulting in some missing. */
+	TEXTID_TO_STRINGID(0x000E, 0x002D, STR_CARGO_PLURAL_NOTHING);
+	TEXTID_TO_STRINGID(0x002E, 0x004D, STR_CARGO_SINGULAR_NOTHING);
+	if (str >= 0x004E && str <= 0x006D) return units_volume[str - 0x004E];
+	TEXTID_TO_STRINGID(0x006E, 0x008D, STR_QUANTITY_NOTHING);
+	TEXTID_TO_STRINGID(0x008E, 0x00AD, STR_ABBREV_NOTHING);
+
+	/* Map building names according to our lang file changes. There are several
+	 * ranges of house ids, all of which need to be remapped to allow newgrfs
+	 * to use original house names. */
+	TEXTID_TO_STRINGID(0x200F, 0x201F, STR_TOWN_BUILDING_NAME_TALL_OFFICE_BLOCK_1);
+	TEXTID_TO_STRINGID(0x2036, 0x2041, STR_TOWN_BUILDING_NAME_COTTAGES_1);
+	TEXTID_TO_STRINGID(0x2059, 0x205C, STR_TOWN_BUILDING_NAME_IGLOO_1);
+
+	/* Same thing for industries */
+	TEXTID_TO_STRINGID(0x4802, 0x4826, STR_INDUSTRY_NAME_COAL_MINE);
+	TEXTID_TO_STRINGID(0x482D, 0x482E, STR_NEWS_INDUSTRY_CONSTRUCTION);
+	TEXTID_TO_STRINGID(0x4832, 0x4834, STR_NEWS_INDUSTRY_CLOSURE_GENERAL);
+	TEXTID_TO_STRINGID(0x4835, 0x4838, STR_NEWS_INDUSTRY_PRODUCTION_INCREASE_GENERAL);
+	TEXTID_TO_STRINGID(0x4839, 0x483A, STR_NEWS_INDUSTRY_PRODUCTION_DECREASE_GENERAL);
+
+	switch (str) {
+		case 0x4830: return STR_ERROR_CAN_T_CONSTRUCT_THIS_INDUSTRY;
+		case 0x4831: return STR_ERROR_FOREST_CAN_ONLY_BE_PLANTED;
+		case 0x483B: return STR_ERROR_CAN_ONLY_BE_POSITIONED;
+	}
+#undef TEXTID_TO_STRINGID
+
+	if (str == STR_NULL) return STR_EMPTY;
+
+	DEBUG(grf, 0, "Unknown StringID 0x%04X remapped to STR_EMPTY. Please open a Feature Request if you need it", str);
+
+	return STR_EMPTY;
+}
+
+/**
+ * Explains the newgrf shift bit positioning.
  * the grf base will not be used in order to find the string, but rather for
  * jumping from standard langID scheme to the new one.
  */
-enum grf_base_languages {
+enum GRFBaseLanguages {
 	GRFLB_AMERICAN    = 0x01,
 	GRFLB_ENGLISH     = 0x02,
 	GRFLB_GERMAN      = 0x04,
@@ -40,137 +104,66 @@ enum grf_base_languages {
 	GRFLB_GENERIC     = 0x80,
 };
 
-enum grf_extended_languages {
+enum GRFExtendedLanguages {
 	GRFLX_AMERICAN    = 0x00,
 	GRFLX_ENGLISH     = 0x01,
 	GRFLX_GERMAN      = 0x02,
 	GRFLX_FRENCH      = 0x03,
 	GRFLX_SPANISH     = 0x04,
-	GRFLX_ESPERANTO   = 0x05,
-	GRFLX_RUSSIAN     = 0x07,
-	GRFLX_CZECH       = 0x15,
-	GRFLX_SLOVAK      = 0x16,
-	GRFLX_BULGARIAN   = 0x18,
-	GRFLX_AFRIKAANS   = 0x1B,
-	GRFLX_GREEK       = 0x1E,
-	GRFLX_DUTCH       = 0x1F,
-	GRFLX_CATALAN     = 0x22,
-	GRFLX_HUNGARIAN   = 0x24,
-	GRFLX_ITALIAN     = 0x27,
-	GRFLX_ROMANIAN    = 0x28,
-	GRFLX_ICELANDIC   = 0x29,
-	GRFLX_LATVIAN     = 0x2A,
-	GRFLX_LITHUANIAN  = 0x2B,
-	GRFLX_SLOVENIAN   = 0x2C,
-	GRFLX_DANISH      = 0x2D,
-	GRFLX_SWEDISH     = 0x2E,
-	GRFLX_NORWEGIAN   = 0x2F,
-	GRFLX_POLISH      = 0x30,
-	GRFLX_GALICIAN    = 0x31,
-	GRFLX_FRISIAN     = 0x32,
-	GRFLX_UKRAINIAN   = 0x33,
-	GRFLX_ESTONIAN    = 0x34,
-	GRFLX_FINNISH     = 0x35,
-	GRFLX_PORTUGUESE  = 0x36,
-	GRFLX_BRAZILIAN   = 0x37,
-	GRFLX_CROATIAN    = 0x38,
-	GRFLX_JAPANESE    = 0x39,
-	GRFLX_KOREAN      = 0x3A,
-	GRFLX_TURKISH     = 0x3E,
 	GRFLX_UNSPECIFIED = 0x7F,
 };
 
-
-struct iso_grf {
-	char code[6];
-	byte grfLangID;
-};
-
-/**
- * ISO code VS NewGrf langID conversion array.
- * This array is used in two ways:
- * 1-its ISO part is matching OpenTTD dynamic language id
- *   with newgrf bit positionning language id
- * 2-its shift part is used to know what is the shift to
- *   watch for when inserting new strings, hence analysing newgrf langid
- */
-const iso_grf iso_codes[] = {
-	{"en_US", GRFLX_AMERICAN},
-	{"en_GB", GRFLX_ENGLISH},
-	{"de_DE", GRFLX_GERMAN},
-	{"fr_FR", GRFLX_FRENCH},
-	{"es_ES", GRFLX_SPANISH},
-	{"af_ZA", GRFLX_AFRIKAANS},
-	{"hr_HR", GRFLX_CROATIAN},
-	{"cs_CZ", GRFLX_CZECH},
-	{"ca_ES", GRFLX_CATALAN},
-	{"da_DA", GRFLX_DANISH},
-	{"nl_NL", GRFLX_DUTCH},
-	{"et_ET", GRFLX_ESTONIAN},
-	{"fi_FI", GRFLX_FINNISH},
-	{"fy_NL", GRFLX_FRISIAN},
-	{"gl_ES", GRFLX_GALICIAN},
-	{"el_GR", GRFLX_GREEK},
-	{"hu_HU", GRFLX_HUNGARIAN},
-	{"is_IS", GRFLX_ICELANDIC},
-	{"it_IT", GRFLX_ITALIAN},
-	{"lv_LV", GRFLX_LATVIAN},
-	{"lt_LT", GRFLX_LITHUANIAN},
-	{"nb_NO", GRFLX_NORWEGIAN},
-	{"pl_PL", GRFLX_POLISH},
-	{"pt_PT", GRFLX_PORTUGUESE},
-	{"pt_BR", GRFLX_BRAZILIAN},
-	{"ro_RO", GRFLX_ROMANIAN},
-	{"ru_RU", GRFLX_RUSSIAN},
-	{"sk_SK", GRFLX_SLOVAK},
-	{"sl_SL", GRFLX_SLOVENIAN},
-	{"sv_SE", GRFLX_SWEDISH},
-	{"tr_TR", GRFLX_TURKISH},
-	{"uk_UA", GRFLX_UKRAINIAN},
-	{"eo_EO", GRFLX_ESPERANTO},
-	{"bg_BG", GRFLX_BULGARIAN},
-	{"ja_JP", GRFLX_JAPANESE},
-	{"ko_KR", GRFLX_KOREAN},
-	{"gen",   GRFLB_GENERIC}   ///< this is not iso code, but there has to be something...
-};
-
-
 /**
  * Element of the linked list.
  * Each of those elements represent the string,
  * but according to a different lang.
  */
 struct GRFText {
-	public:
-		static GRFText* New(byte langid, const char* text)
-		{
-			return new(strlen(text) + 1) GRFText(langid, text);
-		}
+public:
+	static GRFText *New(byte langid, const char *text)
+	{
+		return new (strlen(text) + 1) GRFText(langid, text);
+	}
 
-	private:
-		GRFText(byte langid_, const char* text_) : next(NULL), langid(langid_)
-		{
-			strcpy(text, text_);
-		}
+	/**
+	 * Helper allocation function to disallow something.
+	 * Don't allow simple 'news'; they wouldn't have enough memory.
+	 * @param size the amount of space not to allocate
+	 */
+	void *operator new(size_t size)
+	{
+		NOT_REACHED();
+	}
 
-		void* operator new(size_t size, size_t extra)
-		{
-			return ::operator new(size + extra);
-		}
+	/**
+	 * Free the memory we allocated
+	 * @param p memory to free
+	 */
+	void operator delete(void *p)
+	{
+		free(p);
+	}
+private:
+	GRFText(byte langid_, const char *text_) : next(NULL), langid(langid_)
+	{
+		strcpy(text, text_);
+	}
+
+	/**
+	 * Allocate memory for this class.
+	 * @param size the size of the instance
+	 * @param extra the extra memory for the text
+	 * @return the requested amount of memory for both the instance and the text
+	 */
+	void *operator new(size_t size, size_t extra)
+	{
+		return MallocT<byte>(size + extra);
+	}
 
 public:
-		/* dummy operator delete to silence VC8:
-		 * 'void *GRFText::operator new(size_t,size_t)' : no matching operator delete found;
-		 *     memory will not be freed if initialization throws an exception */
-		void operator delete(void *p, size_t extra)
-		{
-			return ::operator delete(p);
-		}
-
-	public:
-		GRFText *next;
-		byte langid;
-		char text[VARARRAY_SIZE];
+	GRFText *next;
+	byte langid;
+	char text[];
 };
 
 
@@ -210,7 +203,13 @@ char *TranslateTTDPatchCodes(uint32 grfid, const char *str)
 		if (unicode && Utf8EncodedCharLen(*str) != 0) {
 			c = Utf8Consume(&str);
 			/* 'Magic' range of control codes. */
-			if (GB(c, 8, 8) == 0xE0) c = GB(c, 0, 8);
+			if (GB(c, 8, 8) == 0xE0) {
+				c = GB(c, 0, 8);
+			} else if (c >= 0x20) {
+				if (!IsValidChar(c, CS_ALPHANUMERAL)) c = '?';
+				d += Utf8Encode(d, c);
+				continue;
+			}
 		} else {
 			c = (byte)*str++;
 		}
@@ -246,7 +245,7 @@ char *TranslateTTDPatchCodes(uint32 grfid, const char *str)
 			}
 			case 0x82:
 			case 0x83:
-			case 0x84: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_SPEED + c - 0x82); break;
+			case 0x84: d += Utf8Encode(d, SCC_NEWGRF_PRINT_DATE + c - 0x82); break;
 			case 0x85: d += Utf8Encode(d, SCC_NEWGRF_DISCARD_WORD);       break;
 			case 0x86: d += Utf8Encode(d, SCC_NEWGRF_ROTATE_TOP_4_WORDS); break;
 			case 0x87: d += Utf8Encode(d, SCC_NEWGRF_PRINT_WORD_LITRES);  break;
@@ -269,7 +268,7 @@ char *TranslateTTDPatchCodes(uint32 grfid, const char *str)
 			case 0x98: d += Utf8Encode(d, SCC_BLACK);   break;
 			case 0x9A:
 				switch (*str++) {
-					case 0: /* FALL THROUGH */
+					case 0: // FALL THROUGH
 					case 1:
 						d += Utf8Encode(d, SCC_NEWGRF_PRINT_QWORD_CURRENCY);
 						break;
@@ -283,6 +282,19 @@ char *TranslateTTDPatchCodes(uint32 grfid, const char *str)
 						d += Utf8Encode(d, SCC_NEWGRF_UNPRINT);
 						d += Utf8Encode(d, *str++);
 						break;
+					case 6:
+						d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_BYTE);
+						break;
+					case 7:
+						d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_WORD);
+						break;
+					case 8:
+						d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_DWORD);
+						break;
+					case 0x0B:
+						d += Utf8Encode(d, SCC_NEWGRF_PRINT_HEX_QWORD);
+						break;
+
 					default:
 						grfmsg(1, "missing handler for extended format code");
 						break;
@@ -333,7 +345,7 @@ StringID AddGRFString(uint32 grfid, uint16 stringid, byte langid_to_add, bool ne
 	 * actually translated.
 	 */
 	if (!new_scheme) {
-		if (HASBITS(langid_to_add, GRFLB_AMERICAN | GRFLB_ENGLISH)) {
+		if (langid_to_add & (GRFLB_AMERICAN | GRFLB_ENGLISH)) {
 			langid_to_add = GRFLX_ENGLISH;
 		} else {
 			StringID ret = STR_EMPTY;
@@ -423,7 +435,7 @@ const char *GetGRFStringPtr(uint16 stringid)
 	/* Remember this grfid in case the string has included text */
 	_last_grfid = _grf_text[stringid].grfid;
 
-	/*Search the list of lang-strings of this stringid for current lang */
+	/* Search the list of lang-strings of this stringid for current lang */
 	for (search_text = _grf_text[stringid].textholder; search_text != NULL; search_text = search_text->next) {
 		if (search_text->langid == _currentLangID) {
 			return search_text->text;
@@ -447,26 +459,13 @@ const char *GetGRFStringPtr(uint16 stringid)
  * Equivalence Setter function between game and newgrf langID.
  * This function will adjust _currentLangID as to what is the LangID
  * of the current language set by the user.
- * The array iso_codes will be used to find that match.
- * If not found, it will have to be standard english
  * This function is called after the user changed language,
  * from strings.cpp:ReadLanguagePack
- * @param iso_name iso code of current selection
+ * @param language_id iso code of current selection
  */
-void SetCurrentGrfLangID(const char *iso_name)
+void SetCurrentGrfLangID(byte language_id)
 {
-	/* Use English by default, if we can't match up the iso_code. */
-	byte ret = GRFLX_ENGLISH;
-	byte i;
-
-	for (i=0; i < lengthof(iso_codes); i++) {
-		if (strncmp(iso_codes[i].code, iso_name, strlen(iso_codes[i].code)) == 0) {
-			/* We found a match, so let's use it. */
-			ret = iso_codes[i].grfLangID;
-			break;
-		}
-	}
-	_currentLangID = ret;
+	_currentLangID = language_id;
 }
 
 bool CheckGrfLangID(byte lang_id, byte grf_version)
@@ -575,7 +574,7 @@ static TextRefStack *_newgrf_textrefstack = &_newgrf_normal_textrefstack;
  */
 void PrepareTextRefStackUsage(byte numEntries)
 {
-	extern TemporaryStorageArray<uint32, 0x110> _temp_store;
+	extern TemporaryStorageArray<int32, 0x110> _temp_store;
 
 	_newgrf_textrefstack->ResetStack();
 
@@ -609,10 +608,12 @@ void RewindTextRefStack()
 /**
  * FormatString for NewGRF specific "magic" string control codes
  * @param scc   the string control code that has been read
- * @param stack the current "stack"
+ * @param buff  the buffer we're writing to
+ * @param str   the string that we need to write
+ * @param argv  the OpenTTD stack of values
  * @return the string control code to "execute" now
  */
-uint RemapNewGRFStringControlCode(uint scc, char **buff, const char **str, int64 *argv)
+uint RemapNewGRFStringControlCode(uint scc, char *buf_start, char **buff, const char **str, int64 *argv)
 {
 	if (_newgrf_textrefstack->used) {
 		switch (scc) {
@@ -624,6 +625,11 @@ uint RemapNewGRFStringControlCode(uint scc, char **buff, const char **str, int64
 			case SCC_NEWGRF_PRINT_DWORD_CURRENCY:
 			case SCC_NEWGRF_PRINT_DWORD:          *argv = _newgrf_textrefstack->PopSignedDWord();   break;
 
+			case SCC_NEWGRF_PRINT_HEX_BYTE:       *argv = _newgrf_textrefstack->PopUnsignedByte();  break;
+			case SCC_NEWGRF_PRINT_HEX_DWORD:      *argv = _newgrf_textrefstack->PopUnsignedDWord(); break;
+			case SCC_NEWGRF_PRINT_HEX_QWORD:      *argv = _newgrf_textrefstack->PopSignedQWord(); break;
+
+			case SCC_NEWGRF_PRINT_HEX_WORD:
 			case SCC_NEWGRF_PRINT_WORD_SPEED:
 			case SCC_NEWGRF_PRINT_WORD_LITRES:
 			case SCC_NEWGRF_PRINT_UNSIGNED_WORD:  *argv = _newgrf_textrefstack->PopUnsignedWord();  break;
@@ -635,11 +641,10 @@ uint RemapNewGRFStringControlCode(uint scc, char **buff, const char **str, int64
 
 			case SCC_NEWGRF_ROTATE_TOP_4_WORDS:   _newgrf_textrefstack->RotateTop4Words(); break;
 			case SCC_NEWGRF_PUSH_WORD:            _newgrf_textrefstack->PushWord(Utf8Consume(str)); break;
-			case SCC_NEWGRF_UNPRINT:              *buff -= Utf8Consume(str); break;
+			case SCC_NEWGRF_UNPRINT:              *buff = max(*buff - Utf8Consume(str), buf_start); break;
 
 			case SCC_NEWGRF_PRINT_STRING_ID:
-				*argv = _newgrf_textrefstack->PopUnsignedWord();
-				if (*argv == STR_NULL) *argv = STR_EMPTY;
+				*argv = TTDPStringIDToOTTDStringIDMapping(_newgrf_textrefstack->PopUnsignedWord());
 				break;
 		}
 	}
@@ -652,6 +657,12 @@ uint RemapNewGRFStringControlCode(uint scc, char **buff, const char **str, int64
 		case SCC_NEWGRF_PRINT_UNSIGNED_WORD:
 			return SCC_COMMA;
 
+		case SCC_NEWGRF_PRINT_HEX_BYTE:
+		case SCC_NEWGRF_PRINT_HEX_WORD:
+		case SCC_NEWGRF_PRINT_HEX_DWORD:
+		case SCC_NEWGRF_PRINT_HEX_QWORD:
+			return SCC_HEX;
+
 		case SCC_NEWGRF_PRINT_DWORD_CURRENCY:
 		case SCC_NEWGRF_PRINT_QWORD_CURRENCY:
 			return SCC_CURRENCY;
