@@ -1,4 +1,4 @@
-/* $Id: road_cmd.cpp 19439 2010-03-16 20:54:26Z rubidium $ */
+/* $Id: road_cmd.cpp 20429 2010-08-09 21:38:18Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -455,15 +455,15 @@ CommandCost CmdBuildRoad(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		p2 = (town != NULL) ? town->index : (TownID)INVALID_TOWN;
 	}
 
-	RoadBits pieces = Extract<RoadBits, 0>(p1);
+	RoadBits pieces = Extract<RoadBits, 0, 4>(p1);
 
 	/* do not allow building 'zero' road bits, code wouldn't handle it */
 	if (pieces == ROAD_NONE) return CMD_ERROR;
 
-	RoadType rt = (RoadType)GB(p1, 4, 2);
+	RoadType rt = Extract<RoadType, 4, 2>(p1);
 	if (!IsValidRoadType(rt) || !ValParamRoadType(rt)) return CMD_ERROR;
 
-	DisallowedRoadDirections toggle_drd = (DisallowedRoadDirections)GB(p1, 6, 2);
+	DisallowedRoadDirections toggle_drd = Extract<DisallowedRoadDirections, 6, 2>(p1);
 
 	Slope tileh = GetTileSlope(tile, NULL);
 
@@ -491,11 +491,20 @@ CommandCost CmdBuildRoad(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 							Owner owner = GetRoadOwner(tile, ROADTYPE_ROAD);
 							if (owner != OWNER_NONE && !CheckOwnership(owner, tile)) return CMD_ERROR;
 
-							if (!EnsureNoVehicleOnGround(tile)) return CMD_ERROR;
+							DisallowedRoadDirections dis_existing = GetDisallowedRoadDirections(tile);
+							DisallowedRoadDirections dis_new      = dis_existing ^ toggle_drd;
+
+							/* We allow removing disallowed directions to break up
+							 * deadlocks, but adding them can break articulated
+							 * vehicles. As such, only when less is disallowed,
+							 * i.e. bits are removed, we skip the vehicle check. */
+							if (CountBits(dis_existing) <= CountBits(dis_new)) {
+								if (!EnsureNoVehicleOnGround(tile)) return CMD_ERROR;
+							}
 
 							/* Ignore half built tiles */
 							if ((flags & DC_EXEC) && rt != ROADTYPE_TRAM && IsStraightRoad(existing)) {
-								SetDisallowedRoadDirections(tile, GetDisallowedRoadDirections(tile) ^ toggle_drd);
+								SetDisallowedRoadDirections(tile, dis_new);
 								MarkTileDirtyByTile(tile);
 							}
 							return CommandCost();
@@ -532,6 +541,10 @@ CommandCost CmdBuildRoad(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 			if (GetRailTileType(tile) != RAIL_TILE_NORMAL) goto do_clear;
 
+			if (RailNoLevelCrossings(GetRailType(tile))) {
+				return_cmd_error(STR_ERROR_CROSSING_DISALLOWED);
+			}
+
 			Axis roaddir;
 			switch (GetTrackBits(tile)) {
 				case TRACK_BIT_X:
@@ -714,10 +727,10 @@ CommandCost CmdBuildLongRoad(TileIndex start_tile, DoCommandFlag flags, uint32 p
 	if (p1 >= MapSize()) return CMD_ERROR;
 
 	TileIndex end_tile = p1;
-	RoadType rt = (RoadType)GB(p2, 3, 2);
+	RoadType rt = Extract<RoadType, 3, 2>(p2);
 	if (!IsValidRoadType(rt) || !ValParamRoadType(rt)) return CMD_ERROR;
 
-	Axis axis = Extract<Axis, 2>(p2);
+	Axis axis = Extract<Axis, 2, 1>(p2);
 	/* Only drag in X or Y direction dictated by the direction variable */
 	if (axis == AXIS_X && TileY(start_tile) != TileY(end_tile)) return CMD_ERROR; // x-axis
 	if (axis == AXIS_Y && TileX(start_tile) != TileX(end_tile)) return CMD_ERROR; // y-axis
@@ -801,10 +814,10 @@ CommandCost CmdRemoveLongRoad(TileIndex start_tile, DoCommandFlag flags, uint32
 	if (p1 >= MapSize()) return CMD_ERROR;
 
 	TileIndex end_tile = p1;
-	RoadType rt = (RoadType)GB(p2, 3, 2);
+	RoadType rt = Extract<RoadType, 3, 2>(p2);
 	if (!IsValidRoadType(rt)) return CMD_ERROR;
 
-	Axis axis = Extract<Axis, 2>(p2);
+	Axis axis = Extract<Axis, 2, 1>(p2);
 	/* Only drag in X or Y direction dictated by the direction variable */
 	if (axis == AXIS_X && TileY(start_tile) != TileY(end_tile)) return CMD_ERROR; // x-axis
 	if (axis == AXIS_Y && TileX(start_tile) != TileX(end_tile)) return CMD_ERROR; // y-axis
@@ -864,8 +877,8 @@ CommandCost CmdRemoveLongRoad(TileIndex start_tile, DoCommandFlag flags, uint32
  */
 CommandCost CmdBuildRoadDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	DiagDirection dir = Extract<DiagDirection, 0>(p1);
-	RoadType rt = (RoadType)GB(p1, 2, 2);
+	DiagDirection dir = Extract<DiagDirection, 0, 2>(p1);
+	RoadType rt = Extract<RoadType, 2, 2>(p1);
 
 	if (!IsValidRoadType(rt) || !ValParamRoadType(rt)) return CMD_ERROR;
 
@@ -1170,12 +1183,12 @@ static void DrawTile_Road(TileInfo *ti)
 		case ROAD_TILE_CROSSING: {
 			if (ti->tileh != SLOPE_FLAT) DrawFoundation(ti, FOUNDATION_LEVELED);
 
+			PaletteID pal = PAL_NONE;
 			const RailtypeInfo *rti = GetRailTypeInfo(GetRailType(ti->tile));
 
 			if (rti->UsesOverlay()) {
 				Axis axis = GetCrossingRailAxis(ti->tile);
 				SpriteID road = SPR_ROAD_Y + axis;
-				PaletteID pal = PAL_NONE;
 
 				Roadside roadside = GetRoadside(ti->tile);
 
@@ -1194,34 +1207,30 @@ static void DrawTile_Road(TileInfo *ti)
 				SpriteID rail = GetCustomRailSprite(rti, ti->tile, RTSG_CROSSING) + axis;
 				DrawGroundSprite(rail, PAL_NONE);
 				DrawRailTileSeq(ti, &_crossing_layout, TO_CATENARY, rail, 0, PAL_NONE);
-
-				if (HasCatenaryDrawn(GetRailType(ti->tile))) DrawCatenary(ti);
-				break;
-			}
-
-			SpriteID image = rti->base_sprites.crossing;
-			PaletteID pal = PAL_NONE;
-
-			if (GetCrossingRoadAxis(ti->tile) == AXIS_X) image++;
-			if (IsCrossingBarred(ti->tile)) image += 2;
-
-			Roadside roadside = GetRoadside(ti->tile);
-
-			if (AlwaysDrawUnpavedRoads(ti->tile, roadside)) {
-				image += 8;
 			} else {
-				switch (roadside) {
-					case ROADSIDE_BARREN: pal = PALETTE_TO_BARE_LAND; break;
-					case ROADSIDE_GRASS:  break;
-					default:              image += 4; break; // Paved
+				SpriteID image = rti->base_sprites.crossing;
+
+				if (GetCrossingRoadAxis(ti->tile) == AXIS_X) image++;
+				if (IsCrossingBarred(ti->tile)) image += 2;
+
+				Roadside roadside = GetRoadside(ti->tile);
+
+				if (AlwaysDrawUnpavedRoads(ti->tile, roadside)) {
+					image += 8;
+				} else {
+					switch (roadside) {
+						case ROADSIDE_BARREN: pal = PALETTE_TO_BARE_LAND; break;
+						case ROADSIDE_GRASS:  break;
+						default:              image += 4; break; // Paved
+					}
 				}
-			}
 
-			DrawGroundSprite(image, pal);
+				DrawGroundSprite(image, pal);
 
-			/* PBS debugging, draw reserved tracks darker */
-			if (_game_mode != GM_MENU && _settings_client.gui.show_track_reservation && HasCrossingReservation(ti->tile)) {
-				DrawGroundSprite(GetCrossingRoadAxis(ti->tile) == AXIS_Y ? GetRailTypeInfo(GetRailType(ti->tile))->base_sprites.single_x : GetRailTypeInfo(GetRailType(ti->tile))->base_sprites.single_y, PALETTE_CRASH);
+				/* PBS debugging, draw reserved tracks darker */
+				if (_game_mode != GM_MENU && _settings_client.gui.show_track_reservation && HasCrossingReservation(ti->tile)) {
+					DrawGroundSprite(GetCrossingRoadAxis(ti->tile) == AXIS_Y ? GetRailTypeInfo(GetRailType(ti->tile))->base_sprites.single_x : GetRailTypeInfo(GetRailType(ti->tile))->base_sprites.single_y, PALETTE_CRASH);
+				}
 			}
 
 			if (HasTileRoadType(ti->tile, ROADTYPE_TRAM)) {
@@ -1500,6 +1509,10 @@ static void GetTileDesc_Road(TileIndex tile, TileDesc *td)
 			rail_owner = GetTileOwner(tile);
 			if (HasBit(rts, ROADTYPE_ROAD)) road_owner = GetRoadOwner(tile, ROADTYPE_ROAD);
 			if (HasBit(rts, ROADTYPE_TRAM)) tram_owner = GetRoadOwner(tile, ROADTYPE_TRAM);
+
+			const RailtypeInfo *rti = GetRailTypeInfo(GetRailType(tile));
+			td->rail_speed = rti->max_speed;
+
 			break;
 		}
 
