@@ -1,4 +1,4 @@
-/* $Id: road_func.h 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: road_func.h 19552 2010-04-03 19:57:23Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -28,16 +28,6 @@ static inline bool IsValidRoadType(RoadType rt)
 	return rt == ROADTYPE_ROAD || rt == ROADTYPE_TRAM;
 }
 
-/**
- * Are the given bits pointing to valid roadtypes?
- * @param rts the roadtypes to check for validness
- * @return true if and only if valid
- */
-static inline bool AreValidRoadTypes(RoadTypes rts)
-{
-	return HasBit(rts, ROADTYPE_ROAD) || HasBit(rts, ROADTYPE_TRAM);
-}
-
 /**
  * Maps a RoadType to the corresponding RoadTypes value
  *
