@@ -1,4 +1,4 @@
-/* $Id: roadveh_cmd.cpp 19938 2010-06-05 21:13:35Z rubidium $ */
+/* $Id: roadveh_cmd.cpp 20094 2010-07-08 19:52:28Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -413,7 +413,7 @@ CommandCost CmdTurnRoadVeh(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			v->overtaking != 0 ||
 			v->state == RVSB_WORMHOLE ||
 			v->IsInDepot() ||
-			v->cur_speed < 5) {
+			v->current_order.IsType(OT_LOADING)) {
 		return CMD_ERROR;
 	}
 
@@ -1541,12 +1541,10 @@ static bool RoadVehController(RoadVehicle *v)
 	if (v->reverse_ctr != 0) v->reverse_ctr--;
 
 	/* handle crashed */
-	if (v->vehstatus & VS_CRASHED) {
+	if (v->vehstatus & VS_CRASHED || RoadVehCheckTrainCrash(v)) {
 		return RoadVehIsCrashed(v);
 	}
 
-	RoadVehCheckTrainCrash(v);
-
 	/* road vehicle has broken down? */
 	if (v->breakdown_ctr != 0) {
 		if (v->breakdown_ctr <= 2) {
