@@ -1,4 +1,4 @@
-/* $Id: afterload.cpp 19430 2010-03-15 22:52:39Z rubidium $ */
+/* $Id: afterload.cpp 20698 2010-08-30 16:52:37Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -559,6 +559,9 @@ bool AfterLoadGame()
 		default: break;
 	}
 
+	/* The value of _date_fract got divided, so make sure that old games are converted correctly. */
+	if (CheckSavegameVersionOldStyle(11, 1) || (CheckSavegameVersion(147) && _date_fract > DAY_TICKS)) _date_fract /= 885;
+
 	/* Update current year
 	 * must be done before loading sprites as some newgrfs check it */
 	SetDate(_date);
@@ -1295,11 +1298,11 @@ bool AfterLoadGame()
 
 	if (CheckSavegameVersion(50)) {
 		Aircraft *v;
-		/* Aircraft units changed from 8 mph to 1 km/h */
+		/* Aircraft units changed from 8 mph to 1 km-ish/h */
 		FOR_ALL_AIRCRAFT(v) {
 			if (v->subtype <= AIR_AIRCRAFT) {
 				const AircraftVehicleInfo *avi = AircraftVehInfo(v->engine_type);
-				v->cur_speed *= 129;
+				v->cur_speed *= 128;
 				v->cur_speed /= 10;
 				v->max_speed = avi->max_speed;
 				v->acceleration = avi->acceleration;
@@ -2065,6 +2068,33 @@ bool AfterLoadGame()
 		}
 	}
 
+	if (CheckSavegameVersion(141)) {
+		for (TileIndex t = 0; t < map_size; t++) {
+			/* Reset tropic zone for VOID tiles, they shall not have any. */
+			if (IsTileType(t, MP_VOID)) SetTropicZone(t, TROPICZONE_NORMAL);
+		}
+	}
+
+	/* In old versions it was possible to remove an airport while a plane was
+	 * taking off or landing. This gives all kind of problems when building
+	 * another airport in the same station so we don't allow that anymore.
+	 * For old savegames with such aircraft we just throw them in the air and
+	 * treat the aircraft like they were flying already. */
+	if (CheckSavegameVersion(146)) {
+		Aircraft *v;
+		FOR_ALL_AIRCRAFT(v) {
+			if (!v->IsNormalAircraft()) continue;
+			Station *st = GetTargetAirportIfValid(v);
+			if (st == NULL && v->state != FLYING) {
+				v->state = FLYING;
+				UpdateAircraftCache(v);
+				AircraftNextAirportPos_and_Order(v);
+				/* get aircraft back on running altitude */
+				if ((v->vehstatus & VS_CRASHED) == 0) SetAircraftPosition(v, v->x_pos, v->y_pos, GetAircraftFlyingAltitude(v));
+			}
+		}
+	}
+
 	/* Road stops is 'only' updating some caches */
 	AfterLoadRoadStops();
 	AfterLoadLabelMaps();
