@@ -1,4 +1,4 @@
-/* $Id: oldloader_sl.cpp 19393 2010-03-12 21:12:35Z rubidium $ */
+/* $Id: oldloader_sl.cpp 20430 2010-08-09 21:48:19Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -1479,7 +1479,7 @@ static bool LoadOldMapPart1(LoadgameState *ls, int num)
 		}
 	}
 
-	return !ls->failed;
+	return true;
 }
 
 static bool LoadOldMapPart2(LoadgameState *ls, int num)
@@ -1493,7 +1493,7 @@ static bool LoadOldMapPart2(LoadgameState *ls, int num)
 		_m[i].m5 = ReadByte(ls);
 	}
 
-	return !ls->failed;
+	return true;
 }
 
 static bool LoadTTDPatchExtraChunks(LoadgameState *ls, int num)
@@ -1549,7 +1549,7 @@ static bool LoadTTDPatchExtraChunks(LoadgameState *ls, int num)
 		}
 	}
 
-	return !ls->failed;
+	return true;
 }
 
 extern TileIndex _cur_tileloop_tile;
@@ -1733,11 +1733,17 @@ bool LoadTTDMain(LoadgameState *ls)
 	SmallStackSafeStackAlloc<byte, OLD_MAP_SIZE * 2> map3;
 	_old_map3 = map3.data;
 	_old_vehicle_names = NULL;
-	if (!LoadChunk(ls, NULL, main_chunk)) {
-		DEBUG(oldloader, 0, "Loading failed");
+	try {
+		if (!LoadChunk(ls, NULL, main_chunk)) {
+			DEBUG(oldloader, 0, "Loading failed");
+			free(_old_vehicle_names);
+			return false;
+		}
+	} catch (...) {
 		free(_old_vehicle_names);
-		return false;
+		throw;
 	}
+
 	DEBUG(oldloader, 3, "Done, converting game data...");
 
 	FixTTDMapArray();
