@@ -1,4 +1,11 @@
-/* $Id: squirrel.hpp 16835 2009-07-15 20:29:45Z rubidium $ */
+/* $Id: squirrel.hpp 18862 2010-01-18 15:41:38Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file squirrel.hpp defines the Squirrel class */
 
@@ -48,6 +55,7 @@ protected:
 public:
 	friend class AIScanner;
 	friend class AIInstance;
+	friend void squirrel_register_std(Squirrel *engine);
 
 	Squirrel();
 	~Squirrel();
@@ -77,6 +85,12 @@ public:
 	 */
 	void AddConst(const char *var_name, int value);
 
+	/**
+	 * Adds a const to the stack. Depending on the current state this means
+	 *  either a const to a class or to the global space.
+	 */
+	void AddConst(const char *var_name, bool value);
+
 	/**
 	 * Adds a class to the global scope. Make sure to call AddClassEnd when you
 	 *  are done adding methods.
@@ -100,6 +114,11 @@ public:
 	 */
 	bool Resume(int suspend = -1);
 
+	/**
+	 * Resume the VM with an error so it prints a stack trace.
+	 */
+	void ResumeError();
+
 	/**
 	 * Tell the VM to do a garbage collection run.
 	 */
@@ -125,6 +144,7 @@ public:
 
 	/**
 	 * Creates a class instance.
+	 * @param vm The VM to create the class instance for
 	 * @param class_name The name of the class of which we create an instance.
 	 * @param real_instance The instance to the real class, if it represents a real class.
 	 * @param instance Returning value with the pointer to the instance.
@@ -155,7 +175,7 @@ public:
 	/**
 	 * Convert a Squirrel-object to a string.
 	 */
-	static const char *ObjectToString(HSQOBJECT *ptr) { return FS2OTTD(sq_objtostring(ptr)); }
+	static const char *ObjectToString(HSQOBJECT *ptr) { return SQ2OTTD(sq_objtostring(ptr)); }
 
 	/**
 	 * Convert a Squirrel-object to an integer.
@@ -186,7 +206,7 @@ public:
 	/**
 	 * Throw a Squirrel error that will be nicely displayed to the user.
 	 */
-	void ThrowError(const char *error) { sq_throwerror(this->vm, OTTD2FS(error)); }
+	void ThrowError(const char *error) { sq_throwerror(this->vm, OTTD2SQ(error)); }
 
 	/**
 	 * Release a SQ object.
