@@ -1,4 +1,4 @@
-/* $Id: settings_internal.h 18943 2010-01-28 23:17:28Z yexo $ */
+/* $Id: settings_internal.h 19875 2010-05-21 16:03:29Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -86,7 +86,7 @@ typedef SettingDesc SettingDescGlobVarList;
 
 const SettingDesc *GetSettingFromName(const char *name, uint *i);
 bool SetSettingValue(uint index, int32 value, bool force_newgame = false);
-bool SetSettingValue(uint index, const char *value);
+bool SetSettingValue(uint index, const char *value, bool force_newgame = false);
 void SetCompanySetting(uint index, int32 value);
 
 extern VehicleDefaultSettings _old_vds;
