@@ -1,4 +1,4 @@
-/* $Id: stdafx.h 19665 2010-04-17 22:27:49Z rubidium $ */
+/* $Id: stdafx.h 19858 2010-05-18 21:49:59Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -248,6 +248,7 @@
 
 		/* XXX - WinCE without MSVCRT doesn't support wfopen, so it seems */
 		#if !defined(WINCE)
+			namespace std { using ::_tfopen; }
 			#define fopen(file, mode) _tfopen(OTTD2FS(file), _T(mode))
 			#define unlink(file) _tunlink(OTTD2FS(file))
 		#endif /* WINCE */
@@ -395,7 +396,13 @@ void NORETURN CDECL error(const char *str, ...) WARN_FORMAT(1, 2);
 	#define _stricmp strcasecmp
 #endif
 
-#if !defined(MAX_PATH)
+#if defined(MAX_PATH)
+	/* It's already defined, no need to override */
+#elif defined(PATH_MAX) && PATH_MAX > 0
+	/* Use the value from PATH_MAX, if it exists */
+	#define MAX_PATH PATH_MAX
+#else
+	/* If all else fails, hardcode something :( */
 	#define MAX_PATH 260
 #endif
 
