@@ -1,4 +1,11 @@
-/* $Id: strings_type.h 16278 2009-05-11 15:35:26Z rubidium $ */
+/* $Id: strings_type.h 18603 2009-12-22 18:34:07Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file strings_type.h Types related to strings. */
 
@@ -10,6 +17,7 @@
  */
 typedef uint16 StringID;
 static const StringID INVALID_STRING_ID = 0xFFFF;  ///< Constant representing an invalid string
+static const int MAX_CHAR_LENGTH = 4; ///< Max. length of UTF-8 encoded unicode character
 
 enum {
 	MAX_LANG = 64, ///< Maximal number of languages supported by the game
@@ -76,7 +84,6 @@ enum SpecialStrings {
 
 	SPECSTR_ANDCO_NAME         = 0x70E6,
 	SPECSTR_PRESIDENT_NAME     = 0x70E7,
-	SPECSTR_SONGNAME           = 0x70E8,
 
 	/* reserve MAX_LANG strings for the *.lng files */
 	SPECSTR_LANGUAGE_START     = 0x7100,
