@@ -1,12 +1,16 @@
-/* $Id: tree_cmd.cpp 12199 2008-02-20 17:49:50Z frosch $ */
+/* $Id: tree_cmd.cpp 19646 2010-04-16 22:02:33Z rubidium $ */
 
-/** @file tree_cmd.cpp */
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/** @file tree_cmd.cpp Handling of tree tiles. */
 
 #include "stdafx.h"
-#include "openttd.h"
-#include "bridge_map.h"
 #include "clear_map.h"
-#include "tile_cmd.h"
 #include "landscape.h"
 #include "tree_map.h"
 #include "viewport_func.h"
@@ -17,15 +21,18 @@
 #include "genworld.h"
 #include "transparency.h"
 #include "functions.h"
-#include "player_func.h"
+#include "company_func.h"
 #include "sound_func.h"
-#include "settings_type.h"
 #include "water_map.h"
 #include "water.h"
+#include "landscape_type.h"
+#include "company_base.h"
+#include "core/random_func.hpp"
 
 #include "table/strings.h"
 #include "table/sprites.h"
 #include "table/tree_land.h"
+#include "table/clear_land.h"
 
 /**
  * List of tree placer algorithm.
@@ -38,6 +45,14 @@ enum TreePlacer {
 	TP_IMPROVED, ///< A 'improved' algorithm
 };
 
+/** Where to place trees while in-game? */
+enum ExtraTreePlacement {
+	ETP_NONE,       ///< Place trees on no tiles
+	ETP_RAINFOREST, ///< Place trees only on rainforest tiles
+	ETP_ALL,        ///< Place trees on all tiles
+};
+
+
 /**
  * Tests if a tile can be converted to MP_TREES
  * This is true for clear ground without farms or rocks.
@@ -53,7 +68,7 @@ static bool CanPlantTreesOnTile(TileIndex tile, bool allow_desert)
 			return !IsBridgeAbove(tile) && IsCoast(tile) && !IsSlopeWithOneCornerRaised(GetTileSlope(tile, NULL));
 
 		case MP_CLEAR:
-			return !IsBridgeAbove(tile) && !IsClearGround(tile, CLEAR_FIELDS) && !IsClearGround(tile, CLEAR_ROCKS) &&
+			return !IsBridgeAbove(tile) && !IsClearGround(tile, CLEAR_FIELDS) && GetRawClearGround(tile) != CLEAR_ROCKS &&
 			       (allow_desert || !IsClearGround(tile, CLEAR_DESERT));
 
 		default: return false;
@@ -67,7 +82,7 @@ static bool CanPlantTreesOnTile(TileIndex tile, bool allow_desert)
  * @pre the tile must be suitable for trees.
  *
  * @param tile where to plant the trees.
- * @param type The type of the tree
+ * @param treetype The type of the tree
  * @param count the number of trees (minus 1)
  * @param growth the growth status
  */
@@ -86,10 +101,12 @@ static void PlantTreesOnTile(TileIndex tile, TreeType treetype, uint count, uint
 
 		case MP_CLEAR:
 			switch (GetClearGround(tile)) {
-				case CLEAR_GRASS:  ground = TREE_GROUND_GRASS;       density = GetClearDensity(tile); break;
-				case CLEAR_ROUGH:  ground = TREE_GROUND_ROUGH;                                        break;
-				default:           ground = TREE_GROUND_SNOW_DESERT; density = GetClearDensity(tile); break;
+				case CLEAR_GRASS:  ground = TREE_GROUND_GRASS;       break;
+				case CLEAR_ROUGH:  ground = TREE_GROUND_ROUGH;       break;
+				case CLEAR_SNOW:   ground = GetRawClearGround(tile) == CLEAR_ROUGH ? TREE_GROUND_ROUGH_SNOW : TREE_GROUND_SNOW_DESERT; break;
+				default:           ground = TREE_GROUND_SNOW_DESERT; break;
 			}
+			if (GetClearGround(tile) != CLEAR_ROUGH) density = GetClearDensity(tile);
 			break;
 
 		default: NOT_REACHED();
@@ -111,7 +128,7 @@ static void PlantTreesOnTile(TileIndex tile, TreeType treetype, uint count, uint
  */
 static TreeType GetRandomTreeType(TileIndex tile, uint seed)
 {
-	switch (_opt.landscape) {
+	switch (_settings_game.game_creation.landscape) {
 		case LT_TEMPERATE:
 			return (TreeType)(seed * TREE_COUNT_TEMPERATE / 256 + TREE_TEMPERATE);
 
@@ -148,7 +165,7 @@ static void PlaceTree(TileIndex tile, uint32 r)
 
 		/* Rerandomize ground, if neither snow nor shore */
 		TreeGround ground = GetTreeGround(tile);
-		if (ground != TREE_GROUND_SNOW_DESERT && ground != TREE_GROUND_SHORE) {
+		if (ground != TREE_GROUND_SNOW_DESERT && ground != TREE_GROUND_ROUGH_SNOW && ground != TREE_GROUND_SHORE) {
 			SetTreeGroundDensity(tile, (TreeGround)GB(r, 28, 1), 3);
 		}
 
@@ -175,9 +192,9 @@ static void DoPlaceMoreTrees(TileIndex tile)
 		int x = GB(r, 0, 5) - 16;
 		int y = GB(r, 8, 5) - 16;
 		uint dist = abs(x) + abs(y);
-		TileIndex cur_tile = TILE_MASK(tile + TileDiffXY(x, y));
+		TileIndex cur_tile = TileAddWrap(tile, x, y);
 
-		if (dist <= 13 && CanPlantTreesOnTile(cur_tile, true)) {
+		if (cur_tile != INVALID_TILE && dist <= 13 && CanPlantTreesOnTile(cur_tile, true)) {
 			PlaceTree(cur_tile, r);
 		}
 	}
@@ -213,7 +230,8 @@ static void PlaceTreeAtSameHeight(TileIndex tile, uint height)
 		uint32 r = Random();
 		int x = GB(r, 0, 5) - 16;
 		int y = GB(r, 8, 5) - 16;
-		TileIndex cur_tile = TILE_MASK(tile + TileDiffXY(x, y));
+		TileIndex cur_tile = TileAddWrap(tile, x, y);
+		if (cur_tile == INVALID_TILE) continue;
 
 		/* Keep in range of the existing tree */
 		if (abs(x) + abs(y) > 16) continue;
@@ -248,7 +266,7 @@ void PlaceTreesRandomly()
 
 		if (CanPlantTreesOnTile(tile, true)) {
 			PlaceTree(tile, r);
-			if (_patches.tree_placer != TP_IMPROVED) continue;
+			if (_settings_game.game_creation.tree_placer != TP_IMPROVED) continue;
 
 			/* Place a number of trees based on the tile height.
 			 *  This gives a cool effect of multiple trees close together.
@@ -258,7 +276,7 @@ void PlaceTreesRandomly()
 			j = GetTileZ(tile) / TILE_HEIGHT * 2;
 			while (j--) {
 				/* Above snowline more trees! */
-				if (_opt.landscape == LT_ARCTIC && ht > GetSnowLine()) {
+				if (_settings_game.game_creation.landscape == LT_ARCTIC && ht > GetSnowLine()) {
 					PlaceTreeAtSameHeight(tile, ht);
 					PlaceTreeAtSameHeight(tile, ht);
 				};
@@ -269,7 +287,7 @@ void PlaceTreesRandomly()
 	} while (--i);
 
 	/* place extra trees at rainforest area */
-	if (_opt.landscape == LT_TROPIC) {
+	if (_settings_game.game_creation.landscape == LT_TROPIC) {
 		i = ScaleByMapSize(15000);
 
 		do {
@@ -295,18 +313,18 @@ void GenerateTrees()
 {
 	uint i, total;
 
-	if (_patches.tree_placer == TP_NONE) return;
+	if (_settings_game.game_creation.tree_placer == TP_NONE) return;
 
-	if (_opt.landscape != LT_TOYLAND) PlaceMoreTrees();
+	if (_settings_game.game_creation.landscape != LT_TOYLAND) PlaceMoreTrees();
 
-	switch (_patches.tree_placer) {
-		case TP_ORIGINAL: i = _opt.landscape == LT_ARCTIC ? 15 : 6; break;
-		case TP_IMPROVED: i = _opt.landscape == LT_ARCTIC ?  4 : 2; break;
-		default: NOT_REACHED(); return;
+	switch (_settings_game.game_creation.tree_placer) {
+		case TP_ORIGINAL: i = _settings_game.game_creation.landscape == LT_ARCTIC ? 15 : 6; break;
+		case TP_IMPROVED: i = _settings_game.game_creation.landscape == LT_ARCTIC ?  4 : 2; break;
+		default: NOT_REACHED();
 	}
 
 	total = ScaleByMapSize(1000);
-	if (_opt.landscape == LT_TROPIC) total += ScaleByMapSize(15000);
+	if (_settings_game.game_creation.landscape == LT_TROPIC) total += ScaleByMapSize(15000);
 	total *= i;
 	SetGeneratingWorldProgress(GWP_TREE, total);
 
@@ -318,105 +336,105 @@ void GenerateTrees()
 /** Plant a tree.
  * @param tile start tile of area-drag of tree plantation
  * @param flags type of operation
- * @param p1 tree type, -1 means random.
+ * @param p1 tree type, TREE_INVALID means random.
  * @param p2 end tile of area-drag
+ * @param text unused
+ * @return the cost of this operation or an error
  */
-CommandCost CmdPlantTree(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
+CommandCost CmdPlantTree(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	StringID msg = INVALID_STRING_ID;
 	CommandCost cost(EXPENSES_OTHER);
-	int ex;
-	int ey;
-	int sx, sy, x, y;
+	const byte tree_to_plant = GB(p1, 0, 8); // We cannot use Extract as min and max are climate specific.
 
 	if (p2 >= MapSize()) return CMD_ERROR;
-	/* Check the tree type. It can be random or some valid value within the current climate */
-	if (p1 != (uint)-1 && p1 - _tree_base_by_landscape[_opt.landscape] >= _tree_count_by_landscape[_opt.landscape]) return CMD_ERROR;
+	/* Check the tree type within the current climate */
+	if (tree_to_plant != TREE_INVALID && !IsInsideBS(tree_to_plant, _tree_base_by_landscape[_settings_game.game_creation.landscape], _tree_count_by_landscape[_settings_game.game_creation.landscape])) return CMD_ERROR;
 
-	// make sure sx,sy are smaller than ex,ey
-	ex = TileX(tile);
-	ey = TileY(tile);
-	sx = TileX(p2);
-	sy = TileY(p2);
-	if (ex < sx) Swap(ex, sx);
-	if (ey < sy) Swap(ey, sy);
+	TileArea ta(tile, p2);
+	TILE_AREA_LOOP(tile, ta) {
+		switch (GetTileType(tile)) {
+			case MP_TREES:
+				/* no more space for trees? */
+				if (_game_mode != GM_EDITOR && GetTreeCount(tile) == 4) {
+					msg = STR_ERROR_TREE_ALREADY_HERE;
+					continue;
+				}
 
-	for (x = sx; x <= ex; x++) {
-		for (y = sy; y <= ey; y++) {
-			TileIndex tile = TileXY(x, y);
+				if (flags & DC_EXEC) {
+					AddTreeCount(tile, 1);
+					MarkTileDirtyByTile(tile);
+				}
+				/* 2x as expensive to add more trees to an existing tile */
+				cost.AddCost(_price[PR_BUILD_TREES] * 2);
+				break;
 
-			switch (GetTileType(tile)) {
-				case MP_TREES:
-					/* no more space for trees? */
-					if (_game_mode != GM_EDITOR && GetTreeCount(tile) == 3) {
-						msg = STR_2803_TREE_ALREADY_HERE;
-						continue;
-					}
+			case MP_WATER:
+				if (!IsCoast(tile) || IsSlopeWithOneCornerRaised(GetTileSlope(tile, NULL))) {
+					msg = STR_ERROR_CAN_T_BUILD_ON_WATER;
+					continue;
+				}
+			/* FALL THROUGH */
+			case MP_CLEAR: {
+				if (IsBridgeAbove(tile)) {
+					msg = STR_ERROR_SITE_UNSUITABLE;
+					continue;
+				}
 
-					if (flags & DC_EXEC) {
-						AddTreeCount(tile, 1);
-						MarkTileDirtyByTile(tile);
-					}
-					/* 2x as expensive to add more trees to an existing tile */
-					cost.AddCost(_price.build_trees * 2);
-					break;
+				TreeType treetype = (TreeType)tree_to_plant;
+				/* Be a bit picky about which trees go where. */
+				if (_settings_game.game_creation.landscape == LT_TROPIC && treetype != TREE_INVALID && (
+						/* No cacti outside the desert */
+						(treetype == TREE_CACTUS && GetTropicZone(tile) != TROPICZONE_DESERT) ||
+						/* No rain forest trees outside the rain forest, except in the editor mode where it makes those tiles rain forest tile */
+						(IsInsideMM(treetype, TREE_RAINFOREST, TREE_CACTUS) && GetTropicZone(tile) != TROPICZONE_RAINFOREST && _game_mode != GM_EDITOR) ||
+						/* And no subtropical trees in the desert/rain forest */
+						(IsInsideMM(treetype, TREE_SUB_TROPICAL, TREE_TOYLAND) && GetTropicZone(tile) != TROPICZONE_NORMAL))) {
+					msg = STR_ERROR_TREE_WRONG_TERRAIN_FOR_TREE_TYPE;
+					continue;
+				}
 
-				case MP_WATER:
-					if (!IsCoast(tile) || IsSlopeWithOneCornerRaised(GetTileSlope(tile, NULL))) {
-						msg = STR_3807_CAN_T_BUILD_ON_WATER;
-						continue;
-					}
-				/* FALL THROUGH */
-				case MP_CLEAR:
-					if (IsBridgeAbove(tile)) {
-						msg = STR_2804_SITE_UNSUITABLE;
-						continue;
-					}
-
-					if (IsTileType(tile, MP_CLEAR)) {
-						/* Remove fields or rocks. Note that the ground will get barrened */
-						switch (GetClearGround(tile)) {
-							case CLEAR_FIELDS:
-							case CLEAR_ROCKS: {
-								CommandCost ret = DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
-								if (CmdFailed(ret)) return ret;
-								cost.AddCost(ret);
-								break;
-							}
-
-							default: break;
-						}
-					}
-
-					if (_game_mode != GM_EDITOR && IsValidPlayer(_current_player)) {
-						Town *t = ClosestTownFromTile(tile, _patches.dist_local_authority);
-						if (t != NULL) ChangeTownRating(t, RATING_TREE_UP_STEP, RATING_TREE_MAXIMUM);
-					}
-
-					if (flags & DC_EXEC) {
-						TreeType treetype;
-
-						treetype = (TreeType)p1;
-						if (treetype == TREE_INVALID) {
-							treetype = GetRandomTreeType(tile, GB(Random(), 24, 8));
-							if (treetype == TREE_INVALID) treetype = TREE_CACTUS;
+				if (IsTileType(tile, MP_CLEAR)) {
+					/* Remove fields or rocks. Note that the ground will get barrened */
+					switch (GetRawClearGround(tile)) {
+						case CLEAR_FIELDS:
+						case CLEAR_ROCKS: {
+							CommandCost ret = DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+							if (ret.Failed()) return ret;
+							cost.AddCost(ret);
+							break;
 						}
 
-						/* Plant full grown trees in scenario editor */
-						PlantTreesOnTile(tile, treetype, 0, _game_mode == GM_EDITOR ? 3 : 0);
-						MarkTileDirtyByTile(tile);
-
-						/* When planting rainforest-trees, set tropiczone to rainforest in editor. */
-						if (_game_mode == GM_EDITOR && IsInsideMM(treetype, TREE_RAINFOREST, TREE_CACTUS))
-							SetTropicZone(tile, TROPICZONE_RAINFOREST);
+						default: break;
 					}
-					cost.AddCost(_price.build_trees);
-					break;
+				}
 
-				default:
-					msg = STR_2804_SITE_UNSUITABLE;
-					break;
-			}
+				if (_game_mode != GM_EDITOR && Company::IsValidID(_current_company)) {
+					Town *t = ClosestTownFromTile(tile, _settings_game.economy.dist_local_authority);
+					if (t != NULL) ChangeTownRating(t, RATING_TREE_UP_STEP, RATING_TREE_MAXIMUM, flags);
+				}
+
+				if (flags & DC_EXEC) {
+					if (treetype == TREE_INVALID) {
+						treetype = GetRandomTreeType(tile, GB(Random(), 24, 8));
+						if (treetype == TREE_INVALID) treetype = TREE_CACTUS;
+					}
+
+					/* Plant full grown trees in scenario editor */
+					PlantTreesOnTile(tile, treetype, 0, _game_mode == GM_EDITOR ? 3 : 0);
+					MarkTileDirtyByTile(tile);
+
+					/* When planting rainforest-trees, set tropiczone to rainforest in editor. */
+					if (_game_mode == GM_EDITOR && IsInsideMM(treetype, TREE_RAINFOREST, TREE_CACTUS)) {
+						SetTropicZone(tile, TROPICZONE_RAINFOREST);
+					}
+				}
+				cost.AddCost(_price[PR_BUILD_TREES]);
+			} break;
+
+			default:
+				msg = STR_ERROR_SITE_UNSUITABLE;
+				break;
 		}
 	}
 
@@ -427,9 +445,7 @@ CommandCost CmdPlantTree(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
 	}
 }
 
-struct TreeListEnt {
-	SpriteID image;
-	SpriteID pal;
+struct TreeListEnt : PalSpriteID {
 	byte x, y;
 };
 
@@ -439,27 +455,19 @@ static void DrawTile_Trees(TileInfo *ti)
 		case TREE_GROUND_SHORE: DrawShoreTile(ti->tileh); break;
 		case TREE_GROUND_GRASS: DrawClearLandTile(ti, GetTreeDensity(ti->tile)); break;
 		case TREE_GROUND_ROUGH: DrawHillyLandTile(ti); break;
-		default: DrawGroundSprite(_tree_sprites_1[GetTreeDensity(ti->tile)] + _tileh_to_sprite[ti->tileh], PAL_NONE); break;
+		default: DrawGroundSprite(_clear_land_sprites_snow_desert[GetTreeDensity(ti->tile)] + _tileh_to_sprite[ti->tileh], PAL_NONE); break;
 	}
 
 	DrawClearLandFence(ti);
 
-	/* Do not draw trees when the invisible trees patch and transparency tree are set */
-	if (IsTransparencySet(TO_TREES) && _patches.invisible_trees) return;
+	/* Do not draw trees when the invisible trees setting is set */
+	if (IsInvisibilitySet(TO_TREES)) return;
 
-	uint16 tmp = ti->x;
-
-	tmp = ROR(tmp, 2);
-	tmp -= ti->y;
-	tmp = ROR(tmp, 3);
-	tmp -= ti->x;
-	tmp = ROR(tmp, 1);
-	tmp += ti->y;
-
-	uint index = GB(tmp, 6, 2) + (GetTreeType(ti->tile) << 2);
+	uint tmp = CountBits(ti->tile + ti->x + ti->y);
+	uint index = GB(tmp, 0, 2) + (GetTreeType(ti->tile) << 2);
 
 	/* different tree styles above one of the grounds */
-	if (GetTreeGround(ti->tile) == TREE_GROUND_SNOW_DESERT &&
+	if ((GetTreeGround(ti->tile) == TREE_GROUND_SNOW_DESERT || GetTreeGround(ti->tile) == TREE_GROUND_ROUGH_SNOW) &&
 			GetTreeDensity(ti->tile) >= 2 &&
 			IsInsideMM(index, TREE_SUB_ARCTIC << 2, TREE_RAINFOREST << 2)) {
 		index += 164 - (TREE_SUB_ARCTIC << 2);
@@ -468,7 +476,7 @@ static void DrawTile_Trees(TileInfo *ti)
 	assert(index < lengthof(_tree_layout_sprite));
 
 	const PalSpriteID *s = _tree_layout_sprite[index];
-	const TreePos *d = _tree_layout_xy[GB(tmp, 4, 2)];
+	const TreePos *d = _tree_layout_xy[GB(tmp, 2, 2)];
 
 	/* combine trees into one sprite object */
 	StartSpriteCombine();
@@ -476,14 +484,14 @@ static void DrawTile_Trees(TileInfo *ti)
 	TreeListEnt te[4];
 
 	/* put the trees to draw in a list */
-	uint trees = GetTreeCount(ti->tile) + 1;
+	uint trees = GetTreeCount(ti->tile);
 
 	for (uint i = 0; i < trees; i++) {
-		SpriteID image = s[0].sprite + (i == trees - 1 ? GetTreeGrowth(ti->tile) : 3);
-		SpriteID pal = s[0].pal;
+		SpriteID sprite = s[0].sprite + (i == trees - 1 ? GetTreeGrowth(ti->tile) : 3);
+		PaletteID pal = s[0].pal;
 
-		te[i].image = image;
-		te[i].pal   = pal;
+		te[i].sprite = sprite;
+		te[i].pal    = pal;
 		te[i].x = d->x;
 		te[i].y = d->y;
 		s++;
@@ -504,7 +512,7 @@ static void DrawTile_Trees(TileInfo *ti)
 			}
 		}
 
-		AddSortableSpriteToDraw(te[mi].image, te[mi].pal, ti->x + te[mi].x, ti->y + te[mi].y, 16 - te[mi].x, 16 - te[mi].y, 0x30, z, IsTransparencySet(TO_TREES), -te[mi].x, -te[mi].y);
+		AddSortableSpriteToDraw(te[mi].sprite, te[mi].pal, ti->x + te[mi].x, ti->y + te[mi].y, 16 - te[mi].x, 16 - te[mi].y, 0x30, z, IsTransparencySet(TO_TREES), -te[mi].x, -te[mi].y);
 
 		/* replace the removed one with the last one */
 		te[mi] = te[trees - 1];
@@ -527,26 +535,21 @@ static Foundation GetFoundation_Trees(TileIndex tile, Slope tileh)
 	return FOUNDATION_NONE;
 }
 
-static CommandCost ClearTile_Trees(TileIndex tile, byte flags)
+static CommandCost ClearTile_Trees(TileIndex tile, DoCommandFlag flags)
 {
 	uint num;
 
-	if (IsValidPlayer(_current_player)) {
-		Town *t = ClosestTownFromTile(tile, _patches.dist_local_authority);
-		if (t != NULL) ChangeTownRating(t, RATING_TREE_DOWN_STEP, RATING_TREE_MINIMUM);
+	if (Company::IsValidID(_current_company)) {
+		Town *t = ClosestTownFromTile(tile, _settings_game.economy.dist_local_authority);
+		if (t != NULL) ChangeTownRating(t, RATING_TREE_DOWN_STEP, RATING_TREE_MINIMUM, flags);
 	}
 
-	num = GetTreeCount(tile) + 1;
+	num = GetTreeCount(tile);
 	if (IsInsideMM(GetTreeType(tile), TREE_RAINFOREST, TREE_CACTUS)) num *= 4;
 
 	if (flags & DC_EXEC) DoClearSquare(tile);
 
-	return CommandCost(EXPENSES_CONSTRUCTION, num * _price.remove_trees);
-}
-
-static void GetAcceptedCargo_Trees(TileIndex tile, AcceptedCargo ac)
-{
-	/* not used */
+	return CommandCost(EXPENSES_CONSTRUCTION, num * _price[PR_CLEAR_TREES]);
 }
 
 static void GetTileDesc_Trees(TileIndex tile, TileDesc *td)
@@ -554,17 +557,12 @@ static void GetTileDesc_Trees(TileIndex tile, TileDesc *td)
 	TreeType tt = GetTreeType(tile);
 
 	if (IsInsideMM(tt, TREE_RAINFOREST, TREE_CACTUS)) {
-		td->str = STR_280F_RAINFOREST;
+		td->str = STR_LAI_TREE_NAME_RAINFOREST;
 	} else {
-		td->str = tt == TREE_CACTUS ? STR_2810_CACTUS_PLANTS : STR_280E_TREES;
+		td->str = tt == TREE_CACTUS ? STR_LAI_TREE_NAME_CACTUS_PLANTS : STR_LAI_TREE_NAME_TREES;
 	}
 
-	td->owner = GetTileOwner(tile);
-}
-
-static void AnimateTile_Trees(TileIndex tile)
-{
-	/* not used */
+	td->owner[0] = GetTileOwner(tile);
 }
 
 static void TileLoopTreesDesert(TileIndex tile)
@@ -599,14 +597,19 @@ static void TileLoopTreesAlps(TileIndex tile)
 	int k = GetTileZ(tile) - GetSnowLine() + TILE_HEIGHT;
 
 	if (k < 0) {
-		if (GetTreeGround(tile) != TREE_GROUND_SNOW_DESERT) return;
-		SetTreeGroundDensity(tile, TREE_GROUND_GRASS, 3);
+		switch (GetTreeGround(tile)) {
+			case TREE_GROUND_SNOW_DESERT: SetTreeGroundDensity(tile, TREE_GROUND_GRASS, 3); break;
+			case TREE_GROUND_ROUGH_SNOW:  SetTreeGroundDensity(tile, TREE_GROUND_ROUGH, 3); break;
+			default: return;
+		}
 	} else {
 		uint density = min((uint)k / TILE_HEIGHT, 3);
 
-		if (GetTreeGround(tile) != TREE_GROUND_SNOW_DESERT ||
-				GetTreeDensity(tile) != density) {
-			SetTreeGroundDensity(tile, TREE_GROUND_SNOW_DESERT, density);
+		if (GetTreeGround(tile) != TREE_GROUND_SNOW_DESERT && GetTreeGround(tile) != TREE_GROUND_ROUGH_SNOW) {
+			TreeGround tg = GetTreeGround(tile) == TREE_GROUND_ROUGH ? TREE_GROUND_ROUGH_SNOW : TREE_GROUND_SNOW_DESERT;
+			SetTreeGroundDensity(tile, tg, density);
+		} else if (GetTreeDensity(tile) != density) {
+			SetTreeGroundDensity(tile, GetTreeGround(tile), density);
 		} else {
 			if (GetTreeDensity(tile) == 3) {
 				uint32 r = Random();
@@ -625,7 +628,7 @@ static void TileLoop_Trees(TileIndex tile)
 	if (GetTreeGround(tile) == TREE_GROUND_SHORE) {
 		TileLoop_Water(tile);
 	} else {
-		switch (_opt.landscape) {
+		switch (_settings_game.game_creation.landscape) {
 			case LT_TROPIC: TileLoopTreesDesert(tile); break;
 			case LT_ARCTIC: TileLoopTreesAlps(tile);   break;
 		}
@@ -635,7 +638,7 @@ static void TileLoop_Trees(TileIndex tile)
 
 	uint treeCounter = GetTreeCounter(tile);
 
-	/* Handle growth of grass at every 8th processings, like it's done for grass */
+	/* Handle growth of grass (under trees/on MP_TREES tiles) at every 8th processings, like it's done for grass on MP_CLEAR tiles. */
 	if ((treeCounter & 7) == 7 && GetTreeGround(tile) == TREE_GROUND_GRASS) {
 		uint density = GetTreeDensity(tile);
 		if (density < 3) {
@@ -650,26 +653,33 @@ static void TileLoop_Trees(TileIndex tile)
 	SetTreeCounter(tile, 0);
 
 	switch (GetTreeGrowth(tile)) {
-		case 3: /* regular sized tree */
-			if (_opt.landscape == LT_TROPIC &&
+		case 3: // regular sized tree
+			if (_settings_game.game_creation.landscape == LT_TROPIC &&
 					GetTreeType(tile) != TREE_CACTUS &&
 					GetTropicZone(tile) == TROPICZONE_DESERT) {
 				AddTreeGrowth(tile, 1);
 			} else {
 				switch (GB(Random(), 0, 3)) {
-					case 0: /* start destructing */
+					case 0: // start destructing
 						AddTreeGrowth(tile, 1);
 						break;
 
-					case 1: /* add a tree */
-						if (GetTreeCount(tile) < 3) {
+					case 1: // add a tree
+						if (GetTreeCount(tile) < 4) {
 							AddTreeCount(tile, 1);
 							SetTreeGrowth(tile, 0);
 							break;
 						}
 						/* FALL THROUGH */
 
-					case 2: { /* add a neighbouring tree */
+					case 2: { // add a neighbouring tree
+						/* Don't plant extra trees if that's not allowed. */
+						if ((_settings_game.game_creation.landscape == LT_TROPIC && GetTropicZone(tile) == TROPICZONE_RAINFOREST) ?
+								_settings_game.construction.extra_tree_placement == ETP_NONE :
+								_settings_game.construction.extra_tree_placement != ETP_ALL) {
+							break;
+						}
+
 						TreeType treetype = GetTreeType(tile);
 
 						tile += TileOffsByDir((Direction)(Random() & 7));
@@ -691,8 +701,8 @@ static void TileLoop_Trees(TileIndex tile)
 			}
 			break;
 
-		case 6: /* final stage of tree destruction */
-			if (GetTreeCount(tile) > 0) {
+		case 6: // final stage of tree destruction
+			if (GetTreeCount(tile) > 1) {
 				/* more than one tree, delete it */
 				AddTreeCount(tile, -1);
 				SetTreeGrowth(tile, 3);
@@ -702,8 +712,20 @@ static void TileLoop_Trees(TileIndex tile)
 					case TREE_GROUND_SHORE: MakeShore(tile); break;
 					case TREE_GROUND_GRASS: MakeClear(tile, CLEAR_GRASS, GetTreeDensity(tile)); break;
 					case TREE_GROUND_ROUGH: MakeClear(tile, CLEAR_ROUGH, 3); break;
+					case TREE_GROUND_ROUGH_SNOW: {
+						uint density = GetTreeDensity(tile);
+						MakeClear(tile, CLEAR_ROUGH, 3);
+						MakeSnow(tile, density);
+						break;
+					}
 					default: // snow or desert
-						MakeClear(tile, _opt.landscape == LT_TROPIC ? CLEAR_DESERT : CLEAR_SNOW, GetTreeDensity(tile));
+						if (_settings_game.game_creation.landscape == LT_TROPIC) {
+							MakeClear(tile, CLEAR_DESERT, GetTreeDensity(tile));
+						} else {
+							uint density = GetTreeDensity(tile);
+							MakeClear(tile, CLEAR_GRASS, 3);
+							MakeSnow(tile, density);
+						}
 						break;
 				}
 			}
@@ -719,12 +741,15 @@ static void TileLoop_Trees(TileIndex tile)
 
 void OnTick_Trees()
 {
+	/* Don't place trees if that's not allowed */
+	if (_settings_game.construction.extra_tree_placement == ETP_NONE) return;
+
 	uint32 r;
 	TileIndex tile;
 	TreeType tree;
 
 	/* place a tree at a random rainforest spot */
-	if (_opt.landscape == LT_TROPIC &&
+	if (_settings_game.game_creation.landscape == LT_TROPIC &&
 			(r = Random(), tile = RandomTileSeed(r), GetTropicZone(tile) == TROPICZONE_RAINFOREST) &&
 			CanPlantTreesOnTile(tile, false) &&
 			(tree = GetRandomTreeType(tile, GB(r, 24, 8))) != TREE_INVALID) {
@@ -732,27 +757,22 @@ void OnTick_Trees()
 	}
 
 	/* byte underflow */
-	if (--_trees_tick_ctr != 0) return;
+	if (--_trees_tick_ctr != 0 || _settings_game.construction.extra_tree_placement != ETP_ALL) return;
 
 	/* place a tree at a random spot */
 	r = Random();
-	tile = TILE_MASK(r);
+	tile = RandomTileSeed(r);
 	if (CanPlantTreesOnTile(tile, false) && (tree = GetRandomTreeType(tile, GB(r, 24, 8))) != TREE_INVALID) {
 		PlantTreesOnTile(tile, tree, 0, 0);
 	}
 }
 
-static void ClickTile_Trees(TileIndex tile)
-{
-	/* not used */
-}
-
 static TrackStatus GetTileTrackStatus_Trees(TileIndex tile, TransportType mode, uint sub_mode, DiagDirection side)
 {
 	return 0;
 }
 
-static void ChangeTileOwner_Trees(TileIndex tile, PlayerID old_player, PlayerID new_player)
+static void ChangeTileOwner_Trees(TileIndex tile, Owner old_owner, Owner new_owner)
 {
 	/* not used */
 }
@@ -762,25 +782,25 @@ void InitializeTrees()
 	_trees_tick_ctr = 0;
 }
 
-static CommandCost TerraformTile_Trees(TileIndex tile, uint32 flags, uint z_new, Slope tileh_new)
+static CommandCost TerraformTile_Trees(TileIndex tile, DoCommandFlag flags, uint z_new, Slope tileh_new)
 {
 	return DoCommand(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 }
 
 
 extern const TileTypeProcs _tile_type_trees_procs = {
-	DrawTile_Trees,           /* draw_tile_proc */
-	GetSlopeZ_Trees,          /* get_slope_z_proc */
-	ClearTile_Trees,          /* clear_tile_proc */
-	GetAcceptedCargo_Trees,   /* get_accepted_cargo_proc */
-	GetTileDesc_Trees,        /* get_tile_desc_proc */
-	GetTileTrackStatus_Trees, /* get_tile_track_status_proc */
-	ClickTile_Trees,          /* click_tile_proc */
-	AnimateTile_Trees,        /* animate_tile_proc */
-	TileLoop_Trees,           /* tile_loop_clear */
-	ChangeTileOwner_Trees,    /* change_tile_owner_clear */
-	NULL,                     /* get_produced_cargo_proc */
-	NULL,                     /* vehicle_enter_tile_proc */
-	GetFoundation_Trees,      /* get_foundation_proc */
-	TerraformTile_Trees,      /* terraform_tile_proc */
+	DrawTile_Trees,           // draw_tile_proc
+	GetSlopeZ_Trees,          // get_slope_z_proc
+	ClearTile_Trees,          // clear_tile_proc
+	NULL,                     // add_accepted_cargo_proc
+	GetTileDesc_Trees,        // get_tile_desc_proc
+	GetTileTrackStatus_Trees, // get_tile_track_status_proc
+	NULL,                     // click_tile_proc
+	NULL,                     // animate_tile_proc
+	TileLoop_Trees,           // tile_loop_clear
+	ChangeTileOwner_Trees,    // change_tile_owner_clear
+	NULL,                     // add_produced_cargo_proc
+	NULL,                     // vehicle_enter_tile_proc
+	GetFoundation_Trees,      // get_foundation_proc
+	TerraformTile_Trees,      // terraform_tile_proc
 };
