@@ -1,65 +1,27 @@
-/* $Id: vehicle_base.h 16742 2009-07-04 17:39:00Z rubidium $ */
+/* $Id: vehicle_base.h 18929 2010-01-27 20:07:29Z frosch $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file  vehicle_base.h Base class for all vehicles. */
 
 #ifndef VEHICLE_BASE_H
 #define VEHICLE_BASE_H
 
-#include "vehicle_type.h"
 #include "track_type.h"
-#include "rail_type.h"
-#include "road_type.h"
-#include "cargo_type.h"
 #include "direction_type.h"
-#include "gfx_type.h"
 #include "command_type.h"
-#include "date_type.h"
-#include "company_base.h"
-#include "company_type.h"
-#include "oldpool.h"
 #include "order_base.h"
 #include "cargopacket.h"
 #include "texteff.hpp"
-#include "group_type.h"
 #include "engine_type.h"
 #include "order_func.h"
 #include "transport_type.h"
 
-/** Road vehicle states */
-enum RoadVehicleStates {
-	/*
-	 * Lower 4 bits are used for vehicle track direction. (Trackdirs)
-	 * When in a road stop (bit 5 or bit 6 set) these bits give the
-	 * track direction of the entry to the road stop.
-	 * As the entry direction will always be a diagonal
-	 * direction (X_NE, Y_SE, X_SW or Y_NW) only bits 0 and 3
-	 * are needed to hold this direction. Bit 1 is then used to show
-	 * that the vehicle is using the second road stop bay.
-	 * Bit 2 is then used for drive-through stops to show the vehicle
-	 * is stopping at this road stop.
-	 */
-
-	/* Numeric values */
-	RVSB_IN_DEPOT                = 0xFE,                      ///< The vehicle is in a depot
-	RVSB_WORMHOLE                = 0xFF,                      ///< The vehicle is in a tunnel and/or bridge
-
-	/* Bit numbers */
-	RVS_USING_SECOND_BAY         =    1,                      ///< Only used while in a road stop
-	RVS_IS_STOPPING              =    2,                      ///< Only used for drive-through stops. Vehicle will stop here
-	RVS_DRIVE_SIDE               =    4,                      ///< Only used when retrieving move data
-	RVS_IN_ROAD_STOP             =    5,                      ///< The vehicle is in a road stop
-	RVS_IN_DT_ROAD_STOP          =    6,                      ///< The vehicle is in a drive-through road stop
-
-	/* Bit sets of the above specified bits */
-	RVSB_IN_ROAD_STOP            = 1 << RVS_IN_ROAD_STOP,     ///< The vehicle is in a road stop
-	RVSB_IN_ROAD_STOP_END        = RVSB_IN_ROAD_STOP + TRACKDIR_END,
-	RVSB_IN_DT_ROAD_STOP         = 1 << RVS_IN_DT_ROAD_STOP,  ///< The vehicle is in a drive-through road stop
-	RVSB_IN_DT_ROAD_STOP_END     = RVSB_IN_DT_ROAD_STOP + TRACKDIR_END,
-
-	RVSB_TRACKDIR_MASK           = 0x0F,                      ///< The mask used to extract track dirs
-	RVSB_ROAD_STOP_TRACKDIR_MASK = 0x09                       ///< Only bits 0 and 3 are used to encode the trackdir for road stops
-};
-
 enum VehStatus {
 	VS_HIDDEN          = 0x01,
 	VS_STOPPED         = 0x02,
@@ -80,124 +42,27 @@ enum VehicleFlags {
 	VF_AUTOFILL_PRES_WAIT_TIME, ///< Whether non-destructive auto-fill should preserve waiting times
 };
 
-struct VehicleRail {
-	/* Link between the two ends of a multiheaded engine */
-	Vehicle *other_multiheaded_part;
-
-	/* Cached wagon override spritegroup */
-	const struct SpriteGroup *cached_override;
-
-	uint16 last_speed; // NOSAVE: only used in UI
-	uint16 crash_anim_pos;
-
-	/* cached values, recalculated on load and each time a vehicle is added to/removed from the consist. */
-	uint32 cached_power;        ///< total power of the consist.
-	uint16 cached_max_speed;    ///< max speed of the consist. (minimum of the max speed of all vehicles in the consist)
-	uint16 cached_total_length; ///< Length of the whole train, valid only for first engine.
-	uint8 cached_veh_length;    ///< length of this vehicle in units of 1/8 of normal length, cached because this can be set by a callback
-	bool cached_tilt;           ///< train can tilt; feature provides a bonus in curves
-
-	/* cached values, recalculated when the cargo on a train changes (in addition to the conditions above) */
-	uint32 cached_weight;     ///< total weight of the consist.
-	uint32 cached_veh_weight; ///< weight of the vehicle.
-	uint32 cached_max_te;     ///< max tractive effort of consist
-
-	/**
-	 * Position/type of visual effect.
-	 * bit 0 - 3 = position of effect relative to vehicle. (0 = front, 8 = centre, 15 = rear)
-	 * bit 4 - 5 = type of effect. (0 = default for engine class, 1 = steam, 2 = diesel, 3 = electric)
-	 * bit     6 = disable visual effect.
-	 * bit     7 = disable powered wagons.
-	 */
-	byte cached_vis_effect;
-	byte user_def_data;
-
-	/* NOSAVE: for wagon override - id of the first engine in train
-	 * 0xffff == not in train */
-	EngineID first_engine;
-
-	uint16 flags;
-	TrackBitsByte track;
-	byte force_proceed;
-	RailTypeByte railtype;
-	RailTypes compatible_railtypes;
+/** Cached oftenly queried (NewGRF) values */
+struct VehicleCache {
+	uint8 cache_valid;   ///< Whether the caches are valid
+	uint32 cached_var40; ///< Cache for NewGRF var 40
+	uint32 cached_var41; ///< Cache for NewGRF var 41
+	uint32 cached_var42; ///< Cache for NewGRF var 42
+	uint32 cached_var43; ///< Cache for NewGRF var 43
 };
 
-enum VehicleRailFlags {
-	VRF_REVERSING         = 0,
-
-	/* used to calculate if train is going up or down */
-	VRF_GOINGUP           = 1,
-	VRF_GOINGDOWN         = 2,
-
-	/* used to store if a wagon is powered or not */
-	VRF_POWEREDWAGON      = 3,
-
-	/* used to reverse the visible direction of the vehicle */
-	VRF_REVERSE_DIRECTION = 4,
-
-	/* used to mark train as lost because PF can't find the route */
-	VRF_NO_PATH_TO_DESTINATION = 5,
-
-	/* used to mark that electric train engine is allowed to run on normal rail */
-	VRF_EL_ENGINE_ALLOWED_NORMAL_RAIL = 6,
-
-	/* used for vehicle var 0xFE bit 8 (toggled each time the train is reversed, accurate for first vehicle only) */
-	VRF_TOGGLE_REVERSE = 7,
-
-	/* used to mark a train that can't get a path reservation */
-	VRF_TRAIN_STUCK    = 8,
-};
-
-struct VehicleAir {
-	uint16 crashed_counter;
-	uint16 cached_max_speed;
-	byte pos;
-	byte previous_pos;
-	StationID targetairport;
-	byte state;
-};
-
-struct VehicleRoad {
-	byte state;             ///< @see RoadVehicleStates
-	byte frame;
-	uint16 blocked_ctr;
-	byte overtaking;
-	byte overtaking_ctr;
-	uint16 crashed_ctr;
-	byte reverse_ctr;
-	struct RoadStop *slot;
-	byte slot_age;
-	EngineID first_engine;
-	byte cached_veh_length;
-
-	RoadType roadtype;
-	RoadTypes compatible_roadtypes;
-};
-
-struct VehicleEffect {
-	uint16 animation_state;
-	byte animation_substate;
-};
-
-struct VehicleDisaster {
-	uint16 image_override;
-	VehicleID big_ufo_destroyer_target;
-};
-
-struct VehicleShip {
-	TrackBitsByte state;
-};
-
-DECLARE_OLD_POOL(Vehicle, Vehicle, 9, 125)
+typedef Pool<Vehicle, VehicleID, 512, 64000> VehiclePool;
+extern VehiclePool _vehicle_pool;
 
 /* Some declarations of functions, so we can make them friendly */
 struct SaveLoad;
 extern const SaveLoad *GetVehicleDescription(VehicleType vt);
 struct LoadgameState;
 extern bool LoadOldVehicle(LoadgameState *ls, int num);
+extern bool AfterLoadGame();
+extern void FixOldVehicles();
 
-struct Vehicle : PoolItem<Vehicle, VehicleID, &_Vehicle_pool>, BaseVehicle {
+struct Vehicle : VehiclePool::PoolItem<&_vehicle_pool>, BaseVehicle {
 private:
 	Vehicle *next;           ///< pointer to the next vehicle in the chain
 	Vehicle *previous;       ///< NOSAVE: pointer to the previous vehicle in the chain
@@ -207,6 +72,8 @@ private:
 	Vehicle *previous_shared; ///< NOSAVE: pointer to the previous vehicle in the shared order chain
 public:
 	friend const SaveLoad *GetVehicleDescription(VehicleType vt); ///< So we can use private/protected variables in the saveload code
+	friend bool AfterLoadGame();
+	friend void FixOldVehicles();
 	friend void AfterLoadVehicles(bool part_of_load);             ///< So we can set the previous and first pointers while loading
 	friend bool LoadOldVehicle(LoadgameState *ls, int num);       ///< So we can set the proper next pointer while loading
 
@@ -230,12 +97,13 @@ public:
 	/* Used for timetabling. */
 	uint32 current_order_time;     ///< How many ticks have passed since this order started.
 	int32 lateness_counter;        ///< How many ticks late (or early if negative) this vehicle is.
+	Date timetable_start;          ///< When the vehicle is supposed to start the timetable.
 
 	/* Boundaries for the current position in the world and a next hash link.
 	 * NOSAVE: All of those can be updated with VehiclePositionChanged() */
 	Rect coord;
-	Vehicle *next_hash;
-	Vehicle *next_new_hash;
+	Vehicle *next_hash, **prev_hash;
+	Vehicle *next_new_hash, **prev_new_hash;
 	Vehicle **old_new_hash;
 
 	SpriteID colourmap; // NOSAVE: cached colour mapping
@@ -291,7 +159,7 @@ public:
 	CargoID cargo_type;      ///< type of cargo this vehicle is carrying
 	byte cargo_subtype;      ///< Used for livery refits (NewGRF variations)
 	uint16 cargo_cap;        ///< total capacity
-	CargoList cargo;         ///< The cargo this vehicle is carrying
+	VehicleCargoList cargo;  ///< The cargo this vehicle is carrying
 
 	byte day_counter;        ///< Increased by one for each day
 	byte tick_counter;       ///< Increased by one for each tick
@@ -307,38 +175,18 @@ public:
 	} orders;
 
 	byte vehicle_flags;             ///< Used for gradual loading and other miscellaneous things (@see VehicleFlags enum)
-	uint16 load_unload_time_rem;
+
+	/** Ticks to wait before starting next cycle. */
+	uint16 load_unload_ticks;
 
 	GroupID group_id;               ///< Index of group Pool array
 
 	byte subtype;                   ///< subtype (Filled with values from EffectVehicles/TrainSubTypes/AircraftSubTypes)
 
-	union {
-		VehicleRail rail;
-		VehicleAir air;
-		VehicleRoad road;
-		VehicleEffect effect;
-		VehicleDisaster disaster;
-		VehicleShip ship;
-	} u;
-
-	/* cached oftenly queried NewGRF values */
-	uint8 cache_valid;   ///< Whether the caches are valid
-	uint32 cached_var40; ///< Cache for NewGRF var 40
-	uint32 cached_var41; ///< Cache for NewGRF var 41
-	uint32 cached_var42; ///< Cache for NewGRF var 42
-	uint32 cached_var43; ///< Cache for NewGRF var 43
-
-	/**
-	 * Allocates a lot of vehicles.
-	 * @param vl pointer to an array of vehicles to get allocated. Can be NULL if the vehicles aren't needed (makes it test only)
-	 * @param num number of vehicles to allocate room for
-	 * @return true if there is room to allocate all the vehicles
-	 */
-	static bool AllocateList(Vehicle **vl, int num);
+	VehicleCache vcache;            ///< Cache of often used calculated values
 
 	/** Create a new vehicle */
-	Vehicle();
+	Vehicle(VehicleType type = VEH_INVALID);
 
 	/** Destroy all stuff that (still) needs the virtual functions to work properly */
 	void PreDestructor();
@@ -407,7 +255,7 @@ public:
 	 */
 	FORCEINLINE void InvalidateNewGRFCache()
 	{
-		this->cache_valid = 0;
+		this->vcache.cache_valid = 0;
 	}
 
 	/**
@@ -453,14 +301,51 @@ public:
 
 	/**
 	 * Calls the tick handler of the vehicle
+	 * @return is this vehicle still valid?
 	 */
-	virtual void Tick() {};
+	virtual bool Tick() { return true; };
 
 	/**
 	 * Calls the new day handler of the vehicle
 	 */
 	virtual void OnNewDay() {};
 
+	/**
+	 * Crash the (whole) vehicle chain.
+	 * @param flooded whether the cause of the crash is flooding or not.
+	 * @return the number of lost souls.
+	 */
+	virtual uint Crash(bool flooded = false);
+
+	/**
+	 * Update vehicle sprite- and position caches
+	 * @param moved Was the vehicle moved?
+	 * @param turned Did the vehicle direction change?
+	 */
+	inline void UpdateViewport(bool moved, bool turned)
+	{
+		extern void VehicleMove(Vehicle *v, bool update_viewport);
+
+		if (turned) this->UpdateDeltaXY(this->direction);
+		SpriteID old_image = this->cur_image;
+		this->cur_image = this->GetImage(this->direction);
+		if (moved || this->cur_image != old_image) VehicleMove(this, true);
+	}
+
+	/**
+	 * Returns the Trackdir on which the vehicle is currently located.
+	 * Works for trains and ships.
+	 * Currently works only sortof for road vehicles, since they have a fuzzy
+	 * concept of being "on" a trackdir. Dunno really what it returns for a road
+	 * vehicle that is halfway a tile, never really understood that part. For road
+	 * vehicles that are at the beginning or end of the tile, should just return
+	 * the diagonal trackdir on which they are driving. I _think_.
+	 * For other vehicles types, or vehicles with no clear trackdir (such as those
+	 * in depots), returns 0xFF.
+	 * @return the trackdir of the vehicle
+	 */
+	virtual Trackdir GetVehicleTrackdir() const { return INVALID_TRACKDIR; }
+
 	/**
 	 * Gets the running cost of a vehicle  that can be sent into SetDParam for string processing.
 	 * @return the vehicle's running cost
@@ -505,6 +390,27 @@ public:
 	 */
 	inline Vehicle *First() const { return this->first; }
 
+	/**
+	 * Get the last vehicle of this vehicle chain.
+	 * @return the last vehicle of the chain.
+	 */
+	inline Vehicle *Last()
+	{
+		Vehicle *v = this;
+		while (v->Next() != NULL) v = v->Next();
+		return v;
+	}
+
+	/**
+	 * Get the last vehicle of this vehicle chain.
+	 * @return the last vehicle of the chain.
+	 */
+	inline const Vehicle *Last() const
+	{
+		const Vehicle *v = this;
+		while (v->Next() != NULL) v = v->Next();
+		return v;
+	}
 
 	/**
 	 * Get the first order of the vehicles order list.
@@ -573,6 +479,11 @@ public:
 
 		this->current_order_time = src->current_order_time;
 		this->lateness_counter = src->lateness_counter;
+		this->timetable_start = src->timetable_start;
+
+		if (HasBit(src->vehicle_flags, VF_TIMETABLE_STARTED)) SetBit(this->vehicle_flags, VF_TIMETABLE_STARTED);
+		if (HasBit(src->vehicle_flags, VF_AUTOFILL_TIMETABLE)) SetBit(this->vehicle_flags, VF_AUTOFILL_TIMETABLE);
+		if (HasBit(src->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME)) SetBit(this->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
 
 		this->service_interval = src->service_interval;
 	}
@@ -620,81 +531,164 @@ public:
 	 * @return the cost of the depot action.
 	 */
 	CommandCost SendToDepot(DoCommandFlag flags, DepotCommand command);
+
+	/**
+	 * Increments cur_order_index, keeps care of the wrap-around and invalidates the GUI.
+	 * Note: current_order is not invalidated.
+	 */
+	void IncrementOrderIndex()
+	{
+		this->cur_order_index++;
+		if (this->cur_order_index >= this->GetNumOrders()) this->cur_order_index = 0;
+		InvalidateVehicleOrder(this, 0);
+	}
+
+	/**
+	 * Returns order 'index' of a vehicle or NULL when it doesn't exists
+	 * @param index the order to fetch
+	 * @return the found (or not) order
+	 */
+	inline Order *GetOrder(int index) const
+	{
+		return (this->orders.list == NULL) ? NULL : this->orders.list->GetOrderAt(index);
+	}
+
+	/**
+	 * Returns the last order of a vehicle, or NULL if it doesn't exists
+	 * @return last order of a vehicle, if available
+	 */
+	inline Order *GetLastOrder() const
+	{
+		return (this->orders.list == NULL) ? NULL : this->orders.list->GetLastOrder();
+	}
+
+	bool IsEngineCountable() const;
 };
 
-/**
- * This class 'wraps' Vehicle; you do not actually instantiate this class.
- * You create a Vehicle using AllocateVehicle, so it is added to the pool
- * and you reinitialize that to a Train using:
- *   v = new (v) Train();
- *
- * As side-effect the vehicle type is set correctly.
- */
-struct DisasterVehicle : public Vehicle {
-	/** Initializes the Vehicle to a disaster vehicle */
-	DisasterVehicle() { this->type = VEH_DISASTER; }
+#define FOR_ALL_VEHICLES_FROM(var, start) FOR_ALL_ITEMS_FROM(Vehicle, vehicle_index, var, start)
+#define FOR_ALL_VEHICLES(var) FOR_ALL_VEHICLES_FROM(var, 0)
 
+/**
+ * Class defining several overloaded accessors so we don't
+ * have to cast vehicle types that often
+ */
+template <class T, VehicleType Type>
+struct SpecializedVehicle : public Vehicle {
+	static const VehicleType EXPECTED_TYPE = Type; ///< Specialized type
+
+	/**
+	 * Set vehicle type correctly
+	 */
+	FORCEINLINE SpecializedVehicle<T, Type>() : Vehicle(Type) { }
+
+	/**
+	 * Get the first vehicle in the chain
+	 * @return first vehicle in the chain
+	 */
+	FORCEINLINE T *First() const { return (T *)this->Vehicle::First(); }
+
+	/**
+	 * Get the last vehicle in the chain
+	 * @return last vehicle in the chain
+	 */
+	FORCEINLINE T *Last() { return (T *)this->Vehicle::Last(); }
+
+	/**
+	 * Get the last vehicle in the chain
+	 * @return last vehicle in the chain
+	 */
+	FORCEINLINE const T *Last() const { return (const T *)this->Vehicle::Last(); }
+
+	/**
+	 * Get next vehicle in the chain
+	 * @return next vehicle in the chain
+	 */
+	FORCEINLINE T *Next() const { return (T *)this->Vehicle::Next(); }
+
+	/**
+	 * Get previous vehicle in the chain
+	 * @return previous vehicle in the chain
+	 */
+	FORCEINLINE T *Previous() const { return (T *)this->Vehicle::Previous(); }
+
+
+	/**
+	 * Tests whether given index is a valid index for vehicle of this type
+	 * @param index tested index
+	 * @return is this index valid index of T?
+	 */
+	static FORCEINLINE bool IsValidID(size_t index)
+	{
+		return Vehicle::IsValidID(index) && Vehicle::Get(index)->type == Type;
+	}
+
+	/**
+	 * Gets vehicle with given index
+	 * @return pointer to vehicle with given index casted to T *
+	 */
+	static FORCEINLINE T *Get(size_t index)
+	{
+		return (T *)Vehicle::Get(index);
+	}
+
+	/**
+	 * Returns vehicle if the index is a valid index for this vehicle type
+	 * @return pointer to vehicle with given index if it's a vehicle of this type
+	 */
+	static FORCEINLINE T *GetIfValid(size_t index)
+	{
+		return IsValidID(index) ? Get(index) : NULL;
+	}
+
+	/**
+	 * Converts a Vehicle to SpecializedVehicle with type checking.
+	 * @param v Vehicle pointer
+	 * @return pointer to SpecializedVehicle
+	 */
+	static FORCEINLINE T *From(Vehicle *v)
+	{
+		assert(v->type == Type);
+		return (T *)v;
+	}
+
+	/**
+	 * Converts a const Vehicle to const SpecializedVehicle with type checking.
+	 * @param v Vehicle pointer
+	 * @return pointer to SpecializedVehicle
+	 */
+	static FORCEINLINE const T *From(const Vehicle *v)
+	{
+		assert(v->type == Type);
+		return (const T *)v;
+	}
+};
+
+#define FOR_ALL_VEHICLES_OF_TYPE(name, var) FOR_ALL_ITEMS_FROM(name, vehicle_index, var, 0) if (var->type == name::EXPECTED_TYPE)
+
+/**
+ * Disasters, like submarines, skyrangers and their shadows, belong to this class.
+ */
+struct DisasterVehicle : public SpecializedVehicle<DisasterVehicle, VEH_DISASTER> {
+	uint16 image_override;
+	VehicleID big_ufo_destroyer_target;
+
+	/** We don't want GCC to zero our struct! It already is zeroed and has an index! */
+	DisasterVehicle() : SpecializedVehicle<DisasterVehicle, VEH_DISASTER>() {}
 	/** We want to 'destruct' the right class. */
 	virtual ~DisasterVehicle() {}
 
 	const char *GetTypeString() const { return "disaster vehicle"; }
 	void UpdateDeltaXY(Direction direction);
-	void Tick();
+	bool Tick();
 };
 
-/**
- * This class 'wraps' Vehicle; you do not actually instantiate this class.
- * You create a Vehicle using AllocateVehicle, so it is added to the pool
- * and you reinitialize that to a Train using:
- *   v = new (v) Train();
- *
- * As side-effect the vehicle type is set correctly.
- */
-struct InvalidVehicle : public Vehicle {
-	/** Initializes the Vehicle to a invalid vehicle */
-	InvalidVehicle() { this->type = VEH_INVALID; }
-
-	/** We want to 'destruct' the right class. */
-	virtual ~InvalidVehicle() {}
-
-	const char *GetTypeString() const { return "invalid vehicle"; }
-	void Tick() {}
-};
-
-static inline VehicleID GetMaxVehicleIndex()
-{
-	/* TODO - This isn't the real content of the function, but
-	 *  with the new pool-system this will be replaced with one that
-	 *  _really_ returns the highest index. Now it just returns
-	 *  the next safe value we are sure about everything is below.
-	 */
-	return GetVehiclePoolSize() - 1;
-}
-
-static inline uint GetNumVehicles()
-{
-	return GetVehiclePoolSize();
-}
-
-#define FOR_ALL_VEHICLES_FROM(v, start) for (v = GetVehicle(start); v != NULL; v = (v->index + 1U < GetVehiclePoolSize()) ? GetVehicle(v->index + 1) : NULL) if (v->IsValid())
-#define FOR_ALL_VEHICLES(v) FOR_ALL_VEHICLES_FROM(v, 0)
-
-/**
- * Check if an index is a vehicle-index (so between 0 and max-vehicles)
- * @param index of the vehicle to query
- * @return Returns true if the vehicle-id is in range
- */
-static inline bool IsValidVehicleID(uint index)
-{
-	return index < GetVehiclePoolSize() && GetVehicle(index)->IsValid();
-}
-
+#define FOR_ALL_DISASTERVEHICLES(var) FOR_ALL_VEHICLES_OF_TYPE(DisasterVehicle, var)
 
 /** Generates sequence of free UnitID numbers */
 struct FreeUnitIDGenerator {
 	bool *cache;  ///< array of occupied unit id numbers
 	UnitID maxid; ///< maximum ID at the moment of constructor call
-	UnitID curid; ///< last ID returned ; 0 if none
+	UnitID curid; ///< last ID returned; 0 if none
 
 	/** Initializes the structure. Vehicle unit numbers are supposed not to change after
 	 * struct initialization, except after each call to this->NextID() the returned value
@@ -711,31 +705,6 @@ struct FreeUnitIDGenerator {
 	~FreeUnitIDGenerator() { free(this->cache); }
 };
 
-/* Returns order 'index' of a vehicle or NULL when it doesn't exists */
-static inline Order *GetVehicleOrder(const Vehicle *v, int index) { return (v->orders.list == NULL) ? NULL : v->orders.list->GetOrderAt(index); }
-
-/**
- * Returns the last order of a vehicle, or NULL if it doesn't exists
- * @param v Vehicle to query
- * @return last order of a vehicle, if available
- */
-static inline Order *GetLastVehicleOrder(const Vehicle *v) { return (v->orders.list == NULL) ? NULL : v->orders.list->GetLastOrder(); }
-
-/**
- * Returns the Trackdir on which the vehicle is currently located.
- * Works for trains and ships.
- * Currently works only sortof for road vehicles, since they have a fuzzy
- * concept of being "on" a trackdir. Dunno really what it returns for a road
- * vehicle that is halfway a tile, never really understood that part. For road
- * vehicles that are at the beginning or end of the tile, should just return
- * the diagonal trackdir on which they are driving. I _think_.
- * For other vehicles types, or vehicles with no clear trackdir (such as those
- * in depots), returns 0xFF.
- */
-Trackdir GetVehicleTrackdir(const Vehicle *v);
-
-void CheckVehicle32Day(Vehicle *v);
-
 static const int32 INVALID_COORD = 0x7fffffff;
 
 #endif /* VEHICLE_BASE_H */
