@@ -1,4 +1,4 @@
-/* $Id: window.cpp 19258 2010-02-25 21:18:38Z rubidium $ */
+/* $Id: window.cpp 20585 2010-08-20 18:28:46Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -53,7 +53,7 @@ Point _cursorpos_drag_start;
 
 int _scrollbar_start_pos;
 int _scrollbar_size;
-byte _scroller_click_timeout;
+byte _scroller_click_timeout = 0;
 
 bool _scrolling_scrollbar;
 bool _scrolling_viewport;
@@ -548,8 +548,7 @@ void Window::ReInit(int rx, int ry)
 	if (this->resize.step_height > 1) dy -= dy % (int)this->resize.step_height;
 
 	ResizeWindow(this, dx, dy);
-	this->OnResize();
-	this->SetDirty();
+	/* ResizeWindow() does this->SetDirty() already, no need to do it again here. */
 }
 
 /** Set the shaded state of the window to \a make_shaded.
@@ -831,22 +830,22 @@ static void BringWindowToFront(Window *w)
 
 /**
  * Initializes the data (except the position and initial size) of a new Window.
- * @param cls           Class of the window, used for identification and grouping. @see WindowClass
+ * @param desc          Window description.
  * @param window_number Number being assigned to the new window
- * @param desc_flags    Window flags. @see WindowDefaultFlag
  * @return Window pointer of the newly created window
  * @pre If nested widgets are used (\a widget is \c NULL), #nested_root and #nested_array_size must be initialized.
  *      In addition, #nested_array is either \c NULL, or already initialized.
  */
-void Window::InitializeData(WindowClass cls, int window_number, uint32 desc_flags)
+void Window::InitializeData(const WindowDesc *desc, WindowNumber window_number)
 {
 	/* Set up window properties; some of them are needed to set up smallest size below */
-	this->window_class = cls;
+	this->window_class = desc->cls;
 	this->flags4 |= WF_WHITE_BORDER_MASK; // just opened windows have a white border
+	if (desc->default_pos == WDP_CENTER) this->flags4 |= WF_CENTERED;
 	this->owner = INVALID_OWNER;
 	this->nested_focus = NULL;
 	this->window_number = window_number;
-	this->desc_flags = desc_flags;
+	this->desc_flags = desc->flags;
 
 	this->OnInit();
 	/* Initialize nested widget tree. */
@@ -962,11 +961,12 @@ void Window::FindWindowPlacementAndResize(int def_width, int def_height)
 		if (this->resize.step_height > 1) enlarge_y -= enlarge_y % (int)this->resize.step_height;
 
 		ResizeWindow(this, enlarge_x, enlarge_y);
+		/* ResizeWindow() calls this->OnResize(). */
+	} else {
+		/* Always call OnResize; that way the scrollbars and matrices get initialized. */
+		this->OnResize();
 	}
 
-	/* Always call OnResize; that way the scrollbars and matrices get initialized */
-	this->OnResize();
-
 	int nx = this->left;
 	int ny = this->top;
 
@@ -1230,7 +1230,7 @@ void Window::CreateNestedTree(const WindowDesc *desc, bool fill_nested)
  */
 void Window::FinishInitNested(const WindowDesc *desc, WindowNumber window_number)
 {
-	this->InitializeData(desc->cls, window_number, desc->flags);
+	this->InitializeData(desc, window_number);
 	Point pt = this->OnInitialPosition(desc, this->nested_root->smallest_x, this->nested_root->smallest_y, window_number);
 	this->InitializePositionSize(pt.x, pt.y, this->nested_root->smallest_x, this->nested_root->smallest_y);
 	this->FindWindowPlacementAndResize(desc->default_width, desc->default_height);
@@ -1321,7 +1321,7 @@ static void DecreaseWindowCounters()
 	Window *w;
 	FOR_ALL_WINDOWS_FROM_FRONT(w) {
 		/* Unclick scrollbar buttons if they are pressed. */
-		if (w->flags4 & (WF_SCROLL_DOWN | WF_SCROLL_UP)) {
+		if (_scroller_click_timeout == 0 && w->flags4 & (WF_SCROLL_DOWN | WF_SCROLL_UP)) {
 			w->flags4 &= ~(WF_SCROLL_DOWN | WF_SCROLL_UP);
 			w->SetDirty();
 		}
@@ -1413,18 +1413,21 @@ static bool HandleMouseOver()
  */
 void ResizeWindow(Window *w, int delta_x, int delta_y)
 {
-	if (delta_x == 0 && delta_y == 0) return;
+	if (delta_x != 0 || delta_y != 0) {
+		w->SetDirty();
 
-	w->SetDirty();
+		uint new_xinc = max(0, (w->nested_root->resize_x == 0) ? 0 : (int)(w->nested_root->current_x - w->nested_root->smallest_x) + delta_x);
+		uint new_yinc = max(0, (w->nested_root->resize_y == 0) ? 0 : (int)(w->nested_root->current_y - w->nested_root->smallest_y) + delta_y);
+		assert(w->nested_root->resize_x == 0 || new_xinc % w->nested_root->resize_x == 0);
+		assert(w->nested_root->resize_y == 0 || new_yinc % w->nested_root->resize_y == 0);
 
-	uint new_xinc = max(0, (w->nested_root->resize_x == 0) ? 0 : (int)(w->nested_root->current_x - w->nested_root->smallest_x) + delta_x);
-	uint new_yinc = max(0, (w->nested_root->resize_y == 0) ? 0 : (int)(w->nested_root->current_y - w->nested_root->smallest_y) + delta_y);
-	assert(w->nested_root->resize_x == 0 || new_xinc % w->nested_root->resize_x == 0);
-	assert(w->nested_root->resize_y == 0 || new_yinc % w->nested_root->resize_y == 0);
+		w->nested_root->AssignSizePosition(ST_RESIZE, 0, 0, w->nested_root->smallest_x + new_xinc, w->nested_root->smallest_y + new_yinc, _dynlang.text_dir == TD_RTL);
+		w->width  = w->nested_root->current_x;
+		w->height = w->nested_root->current_y;
+	}
 
-	w->nested_root->AssignSizePosition(ST_RESIZE, 0, 0, w->nested_root->smallest_x + new_xinc, w->nested_root->smallest_y + new_yinc, _dynlang.text_dir == TD_RTL);
-	w->width  = w->nested_root->current_x;
-	w->height = w->nested_root->current_y;
+	/* Always call OnResize to make sure everything is initialised correctly if it needs to be. */
+	w->OnResize();
 	w->SetDirty();
 }
 
@@ -1679,7 +1682,6 @@ static bool HandleWindowDragging()
 
 			/* ResizeWindow sets both pre- and after-size to dirty for redrawal */
 			ResizeWindow(w, x, y);
-			w->OnResize();
 			return false;
 		}
 	}
@@ -1695,6 +1697,7 @@ static bool HandleWindowDragging()
 static void StartWindowDrag(Window *w)
 {
 	w->flags4 |= WF_DRAGGING;
+	w->flags4 &= ~WF_CENTERED;
 	_dragging_window = true;
 
 	_drag_delta.x = w->left - _cursor.pos.x;
@@ -1712,6 +1715,7 @@ static void StartWindowDrag(Window *w)
 static void StartWindowSizing(Window *w, bool to_left)
 {
 	w->flags4 |= to_left ? WF_SIZING_LEFT : WF_SIZING_RIGHT;
+	w->flags4 &= ~WF_CENTERED;
 	_dragging_window = true;
 
 	_drag_delta.x = _cursor.pos.x;
@@ -2222,6 +2226,17 @@ static void CheckSoftLimit()
  */
 void InputLoop()
 {
+	/*
+	 * During the generation of the world, there might be
+	 * another thread that is currently building for example
+	 * a road. To not interfere with those tasks, we should
+	 * NOT change the _current_company here.
+	 *
+	 * This is not necessary either, as the only events that
+	 * can be handled are the 'close application' events
+	 */
+	if (!IsGeneratingWorld()) _current_company = _local_company;
+
 	CheckSoftLimit();
 	HandleKeyScrolling();
 
@@ -2373,11 +2388,7 @@ void InvalidateWindowClassesData(WindowClass cls, int data)
  */
 void CallWindowTickEvent()
 {
-	if (_scroller_click_timeout > 3) {
-		_scroller_click_timeout -= 3;
-	} else {
-		_scroller_click_timeout = 0;
-	}
+	if (_scroller_click_timeout != 0) _scroller_click_timeout--;
 
 	Window *w;
 	FOR_ALL_WINDOWS_FROM_FRONT(w) {
@@ -2548,22 +2559,12 @@ void RelocateAllWindows(int neww, int newh)
 		 * in a 'backup'-desc that the window should always be centred. */
 		switch (w->window_class) {
 			case WC_MAIN_TOOLBAR:
-				if (neww - w->width != 0) {
-					ResizeWindow(w, min(neww, 640) - w->width, 0);
-					w->OnResize();
-				}
+				if (neww - w->width != 0) ResizeWindow(w, min(neww, 640) - w->width, 0);
 
 				top = w->top;
 				left = PositionMainToolbar(w); // changes toolbar orientation
 				break;
 
-			case WC_SELECT_GAME:
-			case WC_GAME_OPTIONS:
-			case WC_NETWORK_WINDOW:
-				top = (newh - w->height) >> 1;
-				left = (neww - w->width) >> 1;
-				break;
-
 			case WC_NEWS_WINDOW:
 				top = newh - w->height;
 				left = (neww - w->width) >> 1;
@@ -2586,6 +2587,12 @@ void RelocateAllWindows(int neww, int newh)
 				continue;
 
 			default: {
+				if (w->flags4 & WF_CENTERED) {
+					top = (newh - w->height) >> 1;
+					left = (neww - w->width) >> 1;
+					break;
+				}
+
 				left = w->left;
 				if (left + (w->width >> 1) >= neww) left = neww - w->width;
 				if (left < 0) left = 0;
